@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *   "$Id: arch.c 14097 2010-11-15 07:17:37Z mwedel $";
  */
 
 /*
@@ -208,7 +208,6 @@ void init_archetypes(void) {
  */
 void arch_info(object *op) {
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
-        "%d searches and %d strcmp()'s",
         "%d searches and %d strcmp()'s",
         arch_search, arch_cmp);
 }
@@ -227,8 +226,13 @@ static void init_archetable(void) {
     archetype *at;
 
     LOG(llevDebug, " Setting up archetable...\n");
-    for (at = first_archetype; at != NULL; at = (at->more == NULL) ? at->next : at->more)
+    for (at = first_archetype; at != NULL; at = (at->more == NULL) ? at->next : at->more) {
+        if (at->name == NULL) {
+            LOG(llevError, "archetype without name? %s\n", at->clone.name ? at->clone.name : "(no clone name)");
+            abort();
+        }
         add_arch(at);
+    }
     LOG(llevDebug, "done\n");
 }
 
@@ -484,20 +488,66 @@ static void second_arch_pass(FILE *fp) {
     }
 }
 
-#ifdef DEBUG
 /**
- * Check all generators have the other_arch set.
- * @note
- * doesn't work anymore, as generators can use their inventory.
+ * Check all generators have the other_arch set or something in inventory.
  */
 void check_generators(void) {
-    archetype *at;
+    const archetype *at;
+    int abort = 0;
 
-    for (at = first_archetype; at != NULL; at = at->next)
-        if (QUERY_FLAG(&at->clone, FLAG_GENERATOR) && at->clone.other_arch == NULL)
-            LOG(llevError, "Warning: %s is generator but lacks other_arch.\n", at->name);
+    for (at = first_archetype; at != NULL; at = at->next) {
+        if (!QUERY_FLAG(&at->clone, FLAG_GENERATOR))
+            continue;
+
+        if (!QUERY_FLAG(&at->clone, FLAG_CONTENT_ON_GEN) && at->clone.other_arch == NULL) {
+            LOG(llevError, "Fatal: %s is generator without content_on_gen but lacks other_arch.\n", at->name);
+            abort = 1;
+            continue;
+        }
+        if (QUERY_FLAG(&at->clone, FLAG_CONTENT_ON_GEN) && at->clone.inv == NULL) {
+            LOG(llevError, "Fatal: %s is generator with content_on_gen but lacks inventory.\n", at->name);
+            abort = 1;
+            continue;
+        }
+    }
+
+    if (abort)
+        fatal(SEE_LAST_ERROR);
+}
+
+/**
+ * This checks all summonable items for move_type and other things.
+ * Will call fatal() if an error is found.
+ */
+void check_summoned(void) {
+    const archetype *at;
+
+    for (at = first_archetype; at != NULL; at = at->next) {
+        if (at->clone.type == SPELL && at->clone.subtype == SP_SUMMON_GOLEM && at->clone.other_arch) {
+            if (at->clone.other_arch->clone.move_type == 0) {
+                LOG(llevError, "Summonable archetype %s [%s] has no move_type defined!\n", at->clone.other_arch->name, at->clone.other_arch->clone.name);
+                fatal(SEE_LAST_ERROR);
+            }
+        }
+    }
+}
+
+/**
+ * This ensures all spells have a skill defined, calling fatal() if any error was found.
+ */
+static void check_spells(void) {
+    int abort = 0;
+    const archetype *at;
+
+    for (at = first_archetype; at != NULL; at = at->next) {
+        if (at->clone.type == SPELL && at->clone.skill == NULL) {
+            LOG(llevError, "Spell archetype %s [%s] has no skill defined!\n", at->name, at->clone.name);
+            abort = 1;
+        }
+    }
+    if (abort)
+        fatal(SEE_LAST_ERROR);
 }
-#endif
 
 /**
  * Loads all archetypes and treasures.
@@ -557,9 +607,9 @@ static void load_archetypes(void) {
     LOG(llevDebug, "arch-pass 2...\n");
     second_arch_pass(fp);
     LOG(llevDebug, " done\n");
-#ifdef DEBUG
     check_generators();
-#endif
+    check_spells();
+    check_summoned();
     close_and_delete(fp, comp);
     LOG(llevDebug, " done\n");
 }
@@ -757,4 +807,42 @@ object *object_create_arch(archetype *at) {
     return (head);
 }
 
+/**
+ * Checks if the specified type is a valid one for a Crossfire object.
+ * It would be better (not sure how) if it could automatically
+ * extract this somehow from define.h .  Otherwise, it will complain
+ * if a new type is added, but if a type is removed and this
+ * function not updated, that will no be noticed.
+ *
+ * @param type value to check.
+ * @return 1 if the type is valid, 0 else.
+ */
+int is_type_valid(uint8 type) {
+    if (type >= OBJECT_TYPE_MAX)
+        return 0;
+
+    if (type == 11 || type == 12)
+        return 0;
+    if (type == 19 || type == 25 || type == 35 || type == 38 || type == 47)
+        return 0;
+    if (type == 61 || type == 63 || type == 76 || type == 78 || type == 81)
+        return 0;
+    if (type == 81 || type == 84 || type == 86 || type == 88 || type == 89)
+        return 0;
+    if (type == 96 || type == 97 || type == 107 || type == 108 || type == 110)
+        return 0;
+    if (type >= 117 && type <= 120)
+        return 0;
+    if (type >= 125 && type <= 129)
+        return 0;
+    if (type >= 131 && type <= 137)
+        return 0;
+    if (type >= 140 && type <= 149)
+        return 0;
+    if (type >= 151 && type <= 153)
+        return 0;
+
+    return 1;
+}
+
 /*** end of arch.c ***/
