@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_image_c =
- *   "$Id: image.c 12597 2010-03-07 20:37:38Z akirschbaum $";
+ *   "$Id: image.c 14029 2010-10-24 11:58:25Z ryo_saeba $";
  */
 
 /*
@@ -323,10 +323,6 @@ void read_bmap_names(void) {
  */
 unsigned find_face(const char *name, unsigned error) {
     struct bmappair *bp, tmp;
-    char *p;
-
-    if ((p = strchr(name, '\n')))
-        *p = '\0';
 
     tmp.name = (char *)name;
     bp = (struct bmappair *)bsearch(&tmp, xbm, nroffiles, sizeof(struct bmappair), (int (*)(const void *, const void *))compar);
@@ -362,6 +358,9 @@ int read_smooth(void) {
         if (*buf == '#')
             continue;
 
+        if ((p = strchr(buf, '\n')))
+            *p = '\0';
+
         p = strchr(buf, ' ');
         if (!p)
             continue;
