@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_info_c =
- *    "$Id: info.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *    "$Id: info.c 13542 2010-08-01 10:22:09Z ryo_saeba $";
  */
 
 /*
@@ -39,7 +39,7 @@
  */
 void dump_abilities(void) {
     archetype *at;
-    char name[VERY_BIG_BUF];
+    char *name;
 
     for (at = first_archetype; at; at = at->next) {
         const char *gen_name = "";
@@ -59,9 +59,10 @@ void dump_abilities(void) {
             }
         }
 
-        describe_item(&at->clone, NULL, name, VERY_BIG_BUF);
+        name = stringbuffer_finish(describe_item(&at->clone, NULL, NULL));
         printf("%-16s|%6"FMT64"|%4d|%3d|%s|%s|%s\n", at->clone.name, at->clone.stats.exp,
                at->clone.stats.hp, at->clone.stats.ac, name, at->name, gen_name);
+        free(name);
     }
 }
 
