@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c 12597 2010-03-07 20:37:38Z akirschbaum $";
+ *   "$Id: init.c 14209 2010-12-18 16:19:30Z ryo_saeba $";
  */
 
 /*
@@ -42,7 +42,8 @@ static void init_dynamic(void);
 static void init_clocks(void);
 static void init_attackmess(void);
 
-/** You unforunately need to looking in include/global.h to see what these
+/*
+ * You unforunately need to looking in include/global.h to see what these
  * correspond to.
  */
 struct Settings settings = {
@@ -108,6 +109,10 @@ struct Settings settings = {
     0, /* allow_broken_converters */
     0, /* log_timestamp */
     NULL, /* log_timestamp_format */
+    3,      /* starting_stat_min */
+    18,     /* starting_stat_max */
+    85,     /* starting_stat_points */
+    115,    /* roll_stat_points */
 };
 
 struct Statistics statistics;
@@ -390,7 +395,7 @@ static void init_defaults(void) {
 static void init_dynamic(void) {
     archetype *at = first_archetype;
     while (at) {
-        if (at->clone.type == MAP) {
+        if (at->clone.type == MAP && at->clone.subtype == MAP_TYPE_LEGACY) {
             if (at->clone.race) {
                 strcpy(first_map_ext_path, at->clone.race);
             }
