@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c 12055 2009-07-11 13:48:24Z akirschbaum $";
+ *   "$Id: item.c 14102 2010-11-19 06:35:42Z mwedel $";
  */
 
 /*
@@ -35,6 +35,7 @@
  */
 
 #include <stdio.h>
+#include <assert.h>
 #include <global.h>
 #include <living.h>
 #include <spells.h>
@@ -198,7 +199,7 @@ static const typedata item_types[] = {
     { DIRECTOR, "director", "directors", 0, 0 },
     { GIRDLE, "girdle", "girdles", SK_SMITHERY, 0 },
     { FORCE, "force", "forces", 0, 0 },
-    { POTION_EFFECT, "potion effect", "potion effects", 0, 0 },
+    { POTION_RESIST_EFFECT, "potion effect", "potion effects", 0, 0 },
     { CLOSE_CON, "closed container", "closed container", 0, 0 },
     { CONTAINER, "container", "containers", SK_ALCHEMY, 0 },
     { ARMOUR_IMPROVER, "armour improver", "armour improvers", 0, 0 },
@@ -249,18 +250,12 @@ int get_power_from_ench(int ench) {
  * rating should be.  This should only really be used by the treasure
  * generation code, and when loading legacy objects.  It returns
  * the item_power it calculates.
- * If flag is 1, we return the number of enchantment, and not the
- * the power.  This is used in the treasure code.
  *
  * @param op
  * object of which to compute the item_power
- * @param flag
- * unused
- *
- * @todo
- * fix function, and remove unused flag variable.
+ * @return op's item power.
  */
-int calc_item_power(const object *op, int flag) {
+int calc_item_power(const object *op) {
     int i, tmp, enc;
 
     enc = 0;
@@ -389,26 +384,27 @@ const typedata *get_typedata_by_name(const char *name) {
  * object we want information about.
  * @param newline
  * If TRUE, we don't put parens around the description
- * but do put a newline at the end.  Useful when dumping to files
- * @param[out] buf
- * buffer that will receive the description.
- * @param size
- * buffer size.
+ * but do put a newline at the end. Useful when dumping to files
+ * @param buf
+ * buffer that will receive the description. Can be NULL.
+ * @return buf, a new StringBuffer the caller should free if buf was NULL.
  */
-void describe_resistance(const object *op, int newline, char *buf, size_t size) {
-    char *p;
+StringBuffer *describe_resistance(const object *op, int newline, StringBuffer *buf) {
     int tmpvar;
 
-    p = buf;
+    if (buf == NULL)
+        buf = stringbuffer_new();
+
     for (tmpvar = 0; tmpvar < NROFATTACKS; tmpvar++) {
         if (op->resist[tmpvar] && (op->type != FLESH || atnr_is_dragon_enabled(tmpvar) == 1)) {
             if (!newline)
-                snprintf(p, buf+size-p, "(%s %+d)", resist_plus[tmpvar], op->resist[tmpvar]);
+                stringbuffer_append_printf(buf, "(%s %+d)", resist_plus[tmpvar], op->resist[tmpvar]);
             else
-                snprintf(p, buf+size-p, "%s %d\n", resist_plus[tmpvar], op->resist[tmpvar]);
-            p = strchr(p, '\0');
+                stringbuffer_append_printf(buf, "%s %d\n", resist_plus[tmpvar], op->resist[tmpvar]);
         }
     }
+
+    return buf;
 }
 
 /**
@@ -458,82 +454,75 @@ void get_levelnumber(int i, char *buf, size_t size) {
 }
 
 /**
- * Describes a ring or amulet.
+ * Describes a ring or amulet, or a skill.
  * @param op
- * item to describe.
+ * item to describe, must be RING, AMULET or SKILL.
  * @param buf
- * buffer that will contain the description.
- * @param size
- * size of buffer.
- *
- * @note
- * These are taken from old query_name(), but it would work better
- * if describle_item() would be called to get this information and
- * caller would handle FULL_RING_DESCRIPTION definition.
- * Or make FULL_RING_DESCRIPTION standard part of a game and let
- * client handle names.
- * @note
- * Aug 95 modified this slightly so that Skill tools don't have magic bonus
- * from stats.sp - b.t.
- *
- * @todo
- * Use safe string functions. Check if really ring/amulet?
+ * buffer that will contain the description. If NULL a new one is created.
+ * @return buf, or a new StringBuffer the caller should free if buf was NULL.
+ * @todo why does this also describe a SKILL?
+ * @todo make static when check (test/unit/common/check_item.c) is removed.
  */
-static void ring_desc(const object *op, char *buf, size_t size) {
+StringBuffer *ring_desc(const object *op, StringBuffer *buf) {
     int attr, val;
     size_t len;
 
-    buf[0] = 0;
+    assert(op != NULL);
+    assert(op->type == RING || op->type == AMULET || op->type == SKILL);
+
+    if (buf == NULL)
+        buf = stringbuffer_new();
+    len = stringbuffer_length(buf);
 
     if (!QUERY_FLAG(op, FLAG_IDENTIFIED))
-        return;
+        return buf;
 
     for (attr = 0; attr < NUM_STATS; attr++) {
         if ((val = get_attr_value(&(op->stats), attr)) != 0) {
-            snprintf(buf+strlen(buf), size-strlen(buf), "(%s%+d)", short_stat_name[attr], val);
+            stringbuffer_append_printf(buf, "(%s%+d)", short_stat_name[attr], val);
         }
     }
     if (op->stats.exp)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(speed %+"FMT64")", op->stats.exp);
+        stringbuffer_append_printf(buf, "(speed %+"FMT64")", op->stats.exp);
     if (op->stats.wc)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(wc%+d)", op->stats.wc);
+        stringbuffer_append_printf(buf, "(wc%+d)", op->stats.wc);
     if (op->stats.dam)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(dam%+d)", op->stats.dam);
+        stringbuffer_append_printf(buf, "(dam%+d)", op->stats.dam);
     if (op->stats.ac)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(ac%+d)", op->stats.ac);
+        stringbuffer_append_printf(buf, "(ac%+d)", op->stats.ac);
 
-    describe_resistance(op, 0, buf+strlen(buf), size-strlen(buf));
+    describe_resistance(op, 0, buf);
 
     if (op->stats.food != 0)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(sustenance%+d)", op->stats.food);
-/*  else if (op->stats.food < 0)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(hunger%+d)", op->stats.food); */
+        stringbuffer_append_printf(buf, "(sustenance%+d)", op->stats.food);
     if (op->stats.grace)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(grace%+d)", op->stats.grace);
+        stringbuffer_append_printf(buf, "(grace%+d)", op->stats.grace);
     if (op->stats.sp && op->type != SKILL)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(magic%+d)", op->stats.sp);
+        stringbuffer_append_printf(buf, "(magic%+d)", op->stats.sp);
     if (op->stats.hp)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(regeneration%+d)", op->stats.hp);
+        stringbuffer_append_printf(buf, "(regeneration%+d)", op->stats.hp);
     if (op->stats.luck)
-        snprintf(buf+strlen(buf), size-strlen(buf), "(luck%+d)", op->stats.luck);
+        stringbuffer_append_printf(buf, "(luck%+d)", op->stats.luck);
     if (QUERY_FLAG(op, FLAG_LIFESAVE))
-        snprintf(buf+strlen(buf), size-strlen(buf), "(lifesaving)");
+        stringbuffer_append_printf(buf, "(lifesaving)");
     if (QUERY_FLAG(op, FLAG_REFL_SPELL))
-        snprintf(buf+strlen(buf), size-strlen(buf), "(reflect spells)");
+        stringbuffer_append_printf(buf, "(reflect spells)");
     if (QUERY_FLAG(op, FLAG_REFL_MISSILE))
-        snprintf(buf+strlen(buf), size-strlen(buf), "(reflect missiles)");
+        stringbuffer_append_printf(buf, "(reflect missiles)");
     if (QUERY_FLAG(op, FLAG_STEALTH))
-        snprintf(buf+strlen(buf), size-strlen(buf), "(stealth)");
-    /* Shorten some of the names, so they appear better in the windows */
-    len = strlen(buf);
-    DESCRIBE_PATH_SAFE(buf, op->path_attuned, "Attuned", &len, size);
-    DESCRIBE_PATH_SAFE(buf, op->path_repelled, "Repelled", &len, size);
-    DESCRIBE_PATH_SAFE(buf, op->path_denied, "Denied", &len, size);
+        stringbuffer_append_printf(buf, "(stealth)");
 
-/*  if (op->item_power)
+    describe_spellpath_attenuation("Attuned", op->path_attuned, buf);
+    describe_spellpath_attenuation("Repelled", op->path_repelled, buf);
+    describe_spellpath_attenuation("Denied", op->path_denied, buf);
+
+    /* item_power is done by the caller */
+ /*   if (op->item_power)
         snprintf(buf+strlen(buf), size-strlen(buf), "(item_power %+d)", op->item_power);*/
-    if (buf[0] == 0 && op->type != SKILL)
-        snprintf(buf, size, "of adornment");
+    if (stringbuffer_length(buf) == len && op->type != SKILL)
+        stringbuffer_append_string(buf, "of adornment");
+
+    return buf;
 }
 
 /**
@@ -595,13 +584,14 @@ void query_short_name(const object *op, char *buf, size_t size) {
     case RING:
         if (!op->title) {
             /* If ring has a title, full description isn't so useful */
-            char desc[VERY_BIG_BUF];
+            char* desc;
 
-            ring_desc(op, desc, VERY_BIG_BUF);
+            desc = stringbuffer_finish(ring_desc(op, NULL));
             if (desc[0]) {
                 safe_strcat(buf, " ", &len, size);
                 safe_strcat(buf, desc, &len, size);
             }
+            free(desc);
         }
         break;
 
@@ -797,13 +787,14 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
     case RING:
         if (!op->title) {
             /* If ring has a title, full description isn't so useful */
-            char s[MAX_BUF];
+            char* s;
 
-            ring_desc(op, s, MAX_BUF);
+            s = stringbuffer_finish(ring_desc(op, NULL));
             if (s[0]) {
                 safe_strcat(buf, " ", &len, size);
                 safe_strcat(buf, s, &len, size);
             }
+            free(s);
         }
         break;
 
@@ -819,27 +810,23 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
  * Describes a monster.
  *
  * @param op
- * monster to describe. Must not be NULL.
- * @param retbuf
- * buffer that will contain the description. Must not be NULL.
- * @param size
- * buffer size.
- *
- * @note
- * Break this off from describe_item - that function was way
- * too long, making it difficult to read.  This function deals
- * with describing the monsters & players abilities.  It should only
- * be called with monster & player objects.
+ * monster to describe. Must not be NULL, and must have FLAG_MONSTER or be a PLAYER.
+ * @param buf
+ * buffer that will contain the description. Can be NULL.
+ * @return buf, or a new StringBuffer the caller should clear if buf was NULL.
  *
  * @todo
  * Rename to describe_living (or equivalent) since called for player too.
- * Use safe string functions. Fix weird sustenance logic.
+ * Fix weird sustenance logic.
  */
-void describe_monster(const object *op, char *retbuf, size_t size) {
+StringBuffer *describe_monster(const object *op, StringBuffer *buf) {
     int i;
-    size_t len;
 
-    retbuf[0] = '\0';
+    assert(op != NULL);
+    assert(QUERY_FLAG(op, FLAG_MONSTER) || op->type == PLAYER);
+
+    if (buf == NULL)
+        buf = stringbuffer_new();
 
     /* Note that the resolution this provides for players really isn't
      * very good.  Any player with a speed greater than .67 will
@@ -848,67 +835,67 @@ void describe_monster(const object *op, char *retbuf, size_t size) {
     if (FABS(op->speed) > MIN_ACTIVE_SPEED) {
         switch ((int)((FABS(op->speed))*15)) {
         case 0:
-            snprintf(retbuf, size, "(very slow movement)");
+            stringbuffer_append_string(buf, "(very slow movement)");
             break;
 
         case 1:
-            snprintf(retbuf, size, "(slow movement)");
+            stringbuffer_append_string(buf, "(slow movement)");
             break;
 
         case 2:
-            snprintf(retbuf, size, "(normal movement)");
+            stringbuffer_append_string(buf, "(normal movement)");
             break;
 
         case 3:
         case 4:
-            snprintf(retbuf, size, "(fast movement)");
+            stringbuffer_append_string(buf, "(fast movement)");
             break;
 
         case 5:
         case 6:
-            snprintf(retbuf, size, "(very fast movement)");
+            stringbuffer_append_string(buf, "(very fast movement)");
             break;
 
         case 7:
         case 8:
         case 9:
         case 10:
-            snprintf(retbuf, size, "(extremely fast movement)");
+            stringbuffer_append_string(buf, "(extremely fast movement)");
             break;
 
         default:
-            snprintf(retbuf, size, "(lightning fast movement)");
+            stringbuffer_append_string(buf, "(lightning fast movement)");
             break;
         }
     }
     if (QUERY_FLAG(op, FLAG_UNDEAD))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(undead)");
+        stringbuffer_append_string(buf, "(undead)");
     if (QUERY_FLAG(op, FLAG_SEE_INVISIBLE))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(see invisible)");
+        stringbuffer_append_string(buf, "(see invisible)");
     if (QUERY_FLAG(op, FLAG_USE_WEAPON))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(wield weapon)");
+        stringbuffer_append_string(buf, "(wield weapon)");
     if (QUERY_FLAG(op, FLAG_USE_BOW))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(archer)");
+        stringbuffer_append_string(buf, "(archer)");
     if (QUERY_FLAG(op, FLAG_USE_ARMOUR))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(wear armour)");
+        stringbuffer_append_string(buf, "(wear armour)");
     if (QUERY_FLAG(op, FLAG_USE_RING))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(wear ring)");
+        stringbuffer_append_string(buf, "(wear ring)");
     if (QUERY_FLAG(op, FLAG_USE_SCROLL))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(read scroll)");
+        stringbuffer_append_string(buf, "(read scroll)");
     if (QUERY_FLAG(op, FLAG_USE_RANGE))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(fires wand/rod/horn)");
+        stringbuffer_append_string(buf, "(fires wand/rod/horn)");
     if (QUERY_FLAG(op, FLAG_CAN_USE_SKILL))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(skill user)");
+        stringbuffer_append_string(buf, "(skill user)");
     if (QUERY_FLAG(op, FLAG_CAST_SPELL))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(spellcaster)");
+        stringbuffer_append_string(buf, "(spellcaster)");
     if (QUERY_FLAG(op, FLAG_FRIENDLY))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(friendly)");
+        stringbuffer_append_string(buf, "(friendly)");
     if (QUERY_FLAG(op, FLAG_UNAGGRESSIVE))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(unaggressive)");
+        stringbuffer_append_string(buf, "(unaggressive)");
     if (QUERY_FLAG(op, FLAG_HITBACK))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(hitback)");
+        stringbuffer_append_string(buf, "(hitback)");
     if (QUERY_FLAG(op, FLAG_STEALTH))
-        snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(stealthy)");
+        stringbuffer_append_string(buf, "(stealthy)");
     if (op->randomitems != NULL) {
         treasure *t;
         int first = 1;
@@ -917,32 +904,31 @@ void describe_monster(const object *op, char *retbuf, size_t size) {
             if (t->item && (t->item->clone.type == SPELL)) {
                 if (first) {
                     first = 0;
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(Spell abilities:)");
+                    stringbuffer_append_string(buf, "(Spell abilities:)");
                 }
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(%s)",  t->item->clone.name);
+                stringbuffer_append_printf(buf, "(%s)",  t->item->clone.name);
             }
     }
     if (op->type == PLAYER) {
         if (op->contr->digestion) {
             if (op->contr->digestion != 0)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(sustenance%+d)", op->contr->digestion);
+                stringbuffer_append_printf(buf, "(sustenance%+d)", op->contr->digestion);
         }
         if (op->contr->gen_grace) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(grace%+d)", op->contr->gen_grace);
+            stringbuffer_append_printf(buf, "(grace%+d)", op->contr->gen_grace);
         }
         if (op->contr->gen_sp) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(magic%+d)", op->contr->gen_sp);
+            stringbuffer_append_printf(buf, "(magic%+d)", op->contr->gen_sp);
         }
         if (op->contr->gen_hp) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(regeneration%+d)", op->contr->gen_hp);
+            stringbuffer_append_printf(buf, "(regeneration%+d)", op->contr->gen_hp);
         }
         if (op->stats.luck) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(luck%+d)", op->stats.luck);
+            stringbuffer_append_printf(buf, "(luck%+d)", op->stats.luck);
         }
     }
 
     /* describe attacktypes */
-    len = strlen(retbuf);
     if (is_dragon_pl(op)) {
         /* for dragon players display the attacktypes from clawing skill
          * Break apart the for loop - move the comparison checking down -
@@ -952,21 +938,23 @@ void describe_monster(const object *op, char *retbuf, size_t size) {
 
         tmp = object_find_by_type_and_name(op, SKILL, "clawing");
         if (tmp && tmp->attacktype != 0) {
-            DESCRIBE_ABILITY_SAFE(retbuf, tmp->attacktype, "Claws", &len, size);
+            describe_attacktype("Claws", tmp->attacktype, buf);
         } else {
-            DESCRIBE_ABILITY_SAFE(retbuf, op->attacktype, "Attacks", &len, size);
+            describe_attacktype("Attacks", op->attacktype, buf);
         }
     } else {
-        DESCRIBE_ABILITY_SAFE(retbuf, op->attacktype, "Attacks", &len, size);
+        describe_attacktype("Attacks", op->attacktype, buf);
     }
-    DESCRIBE_PATH_SAFE(retbuf, op->path_attuned, "Attuned", &len, size);
-    DESCRIBE_PATH_SAFE(retbuf, op->path_repelled, "Repelled", &len, size);
-    DESCRIBE_PATH_SAFE(retbuf, op->path_denied, "Denied", &len, size);
+    describe_spellpath_attenuation("Attuned", op->path_attuned &~ op->path_denied, buf);
+    describe_spellpath_attenuation("Repelled", op->path_repelled &~ op->path_denied, buf );
+    describe_spellpath_attenuation("Denied", op->path_denied, buf);
     for (i = 0; i < NROFATTACKS; i++) {
         if (op->resist[i]) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(%s %+d)", resist_plus[i], op->resist[i]);
+            stringbuffer_append_printf(buf, "(%s %+d)", resist_plus[i], op->resist[i]);
         }
     }
+
+    return buf;
 }
 
 /**
@@ -995,10 +983,9 @@ void describe_monster(const object *op, char *retbuf, size_t size) {
  * object to describe. Must not be NULL.
  * @param owner
  * player examining the object.
- * @param retbuf
- * buffer that will contain the description. Must not be NULL.
- * @param size
- * size of buffer.
+ * @param buf
+ * buffer that will contain the description. Can be NULL.
+ * @return buf, or new StringBuffer the caller must free if buf was NULL.
  *
  * @note
  * This function is really much more complicated than it should
@@ -1007,24 +994,25 @@ void describe_monster(const object *op, char *retbuf, size_t size) {
  * means these special cases need to be worked out.
  *
  * @todo
- * Check whether owner is really needed. Use safe string functions.
- * Check spurious food logic.
+ * Check whether owner is really needed.
  */
-void describe_item(const object *op, const object *owner, char *retbuf, size_t size) {
+StringBuffer *describe_item(const object *op, const object *owner, StringBuffer *buf) {
     int identified, i;
 
-    retbuf[0] = '\0';
+    if (buf == NULL)
+        buf = stringbuffer_new();
+
     if (QUERY_FLAG(op, FLAG_MONSTER) || op->type == PLAYER) {
-        describe_monster(op, retbuf, size);
-        return;
+        return describe_monster(op, buf);
     }
+
     /* figure this out once, instead of making multiple calls to need_identify.
      * also makes the code easier to read.
      */
     if (!need_identify(op) || QUERY_FLAG(op, FLAG_IDENTIFIED))
         identified = 1;
     else {
-        snprintf(retbuf, size, "(unidentified)");
+        stringbuffer_append_string(buf, "(unidentified)");
         identified = 0;
     }
     switch (op->type) {
@@ -1050,31 +1038,31 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
     case POWER_CRYSTAL:
         /* Avoid division by zero... */
         if (op->stats.maxsp == 0) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(capacity %d).", op->stats.maxsp);
+            stringbuffer_append_printf(buf, "(capacity %d).", op->stats.maxsp);
         } else {
             if (op->stats.maxsp > 1000) { /*higher capacity crystals*/
                 i = (op->stats.maxsp%1000)/100;
                 if (i)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(capacity %d.%dk). It is ", op->stats.maxsp/1000, i);
+                    stringbuffer_append_printf(buf, "(capacity %d.%dk). It is ", op->stats.maxsp/1000, i);
                 else
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(capacity %dk). It is ", op->stats.maxsp/1000);
+                    stringbuffer_append_printf(buf, "(capacity %dk). It is ", op->stats.maxsp/1000);
             } else
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(capacity %d). It is ", op->stats.maxsp);
+                stringbuffer_append_printf(buf, "(capacity %d). It is ", op->stats.maxsp);
             i = (op->stats.sp*10)/op->stats.maxsp;
             if (op->stats.sp == 0)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "empty.");
+                stringbuffer_append_string(buf, "empty.");
             else if (i == 0)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "almost empty.");
+                stringbuffer_append_string(buf, "almost empty.");
             else if (i < 3)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "partially filled.");
+                stringbuffer_append_string(buf, "partially filled.");
             else if (i < 6)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "half full.");
+                stringbuffer_append_string(buf, "half full.");
             else if (i < 9)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "well charged.");
+                stringbuffer_append_string(buf, "well charged.");
             else if (op->stats.sp == op->stats.maxsp)
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "fully charged.");
+                stringbuffer_append_string(buf, "fully charged.");
             else
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "almost full.");
+                stringbuffer_append_string(buf, "almost full.");
         }
         break;
 
@@ -1082,22 +1070,22 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
     case FLESH:
     case DRINK:
         if (identified || QUERY_FLAG(op, FLAG_BEEN_APPLIED)) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(food+%d)", op->stats.food);
+            stringbuffer_append_printf(buf, "(food+%d)", op->stats.food);
 
             if (op->type == FLESH && op->last_eat > 0 && atnr_is_dragon_enabled(op->last_eat)) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(%s metabolism)", change_resist_msg[op->last_eat]);
+                stringbuffer_append_printf(buf, "(%s metabolism)", change_resist_msg[op->last_eat]);
             }
 
             if (!QUERY_FLAG(op, FLAG_CURSED)) {
                 if (op->stats.hp)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(heals)");
+                    stringbuffer_append_string(buf, "(heals)");
                 if (op->stats.sp)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(spellpoint regen)");
+                    stringbuffer_append_string(buf, "(spellpoint regen)");
             } else {
                 if (op->stats.hp)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(damages)");
+                    stringbuffer_append_string(buf, "(damages)");
                 if (op->stats.sp)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(spellpoint depletion)");
+                    stringbuffer_append_string(buf, "(spellpoint depletion)");
             }
         }
         break;
@@ -1106,15 +1094,15 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
     case RING:
     case AMULET:
         if (op->item_power) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(item_power %+d)", op->item_power);
+            stringbuffer_append_printf(buf, "(item_power %+d)", op->item_power);
         }
         if (op->title) {
-            ring_desc(op, retbuf+strlen(retbuf), size-strlen(retbuf));
+            ring_desc(op, buf);
         }
-        return;
+        return buf;
 
     default:
-        return;
+        return buf;
     }
 
     /* Down here, we more further describe equipment type items.
@@ -1125,11 +1113,11 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
 
         for (attr = 0; attr < NUM_STATS; attr++) {
             if ((val = get_attr_value(&(op->stats), attr)) != 0) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(%s%+d)", short_stat_name[attr], val);
+                stringbuffer_append_printf(buf, "(%s%+d)", short_stat_name[attr], val);
             }
         }
         if (op->glow_radius)
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(glowing)");
+            stringbuffer_append_string(buf, "(glowing)");
 
         switch (op->type) {
         case FLESH:
@@ -1137,7 +1125,7 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
 
         default:
             if (op->stats.exp) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(speed %+"FMT64")", op->stats.exp);
+                stringbuffer_append_printf(buf, "(speed %+"FMT64")", op->stats.exp);
             }
             break;
         }
@@ -1158,16 +1146,16 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
         case FORCE:
         case CLOAK:
             if (op->stats.wc) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(wc%+d)", op->stats.wc);
+                stringbuffer_append_printf(buf, "(wc%+d)", op->stats.wc);
             }
             if (op->stats.dam) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(dam%+d)", op->stats.dam);
+                stringbuffer_append_printf(buf, "(dam%+d)", op->stats.dam);
             }
             if (op->stats.ac) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(ac%+d)", op->stats.ac);
+                stringbuffer_append_printf(buf, "(ac%+d)", op->stats.ac);
             }
             if ((op->type == WEAPON || op->type == BOW) && op->level > 0) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(improved %d/%d)", op->last_eat, op->level);
+                stringbuffer_append_printf(buf, "(improved %d/%d)", op->last_eat, op->level);
             }
             break;
 
@@ -1175,24 +1163,24 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
             break;
         }
         if (QUERY_FLAG(op, FLAG_XRAYS))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(xray-vision)");
+            stringbuffer_append_string(buf, "(xray-vision)");
         if (QUERY_FLAG(op, FLAG_SEE_IN_DARK))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(infravision)");
+            stringbuffer_append_string(buf, "(infravision)");
 
         /* levitate was what is was before, so we'll keep it */
         if (op->move_type&MOVE_FLY_LOW)
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(levitate)");
+            stringbuffer_append_string(buf, "(levitate)");
 
         if (op->move_type&MOVE_FLY_HIGH)
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(fly)");
+            stringbuffer_append_string(buf, "(fly)");
 
         if (op->move_type&MOVE_SWIM)
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(swim)");
+            stringbuffer_append_string(buf, "(swim)");
 
         /* walking is presumed as 'normal', so doesn't need mentioning */
 
         if (op->item_power) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(item_power %+d)", op->item_power);
+            stringbuffer_append_printf(buf, "(item_power %+d)", op->item_power);
         }
     } /* End if identified or applied */
 
@@ -1202,7 +1190,6 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
       */
     if (identified) {
         int more_info = 0;
-        size_t len;
 
         switch (op->type) {
         case ROD:  /* These use stats.sp for spell selection and stats.food */
@@ -1225,10 +1212,10 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
         case BRACERS:
         case CLOAK:
             if (ARMOUR_SPEED(op)) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(Max speed %1.2f)", ARMOUR_SPEED(op)/10.0);
+                stringbuffer_append_printf(buf, "(Max speed %1.2f)", ARMOUR_SPEED(op)/10.0);
             }
             if (ARMOUR_SPELLS(op)) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(Spell regen penalty %d)", ARMOUR_SPELLS(op));
+                stringbuffer_append_printf(buf, "(Spell regen penalty %d)", ARMOUR_SPELLS(op));
             }
             more_info = 1;
             break;
@@ -1241,53 +1228,53 @@ void describe_item(const object *op, const object *owner, char *retbuf, size_t s
             if (i < 0)
                 i = 0;
 
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(weapon speed %d)", i);
+            stringbuffer_append_printf(buf, "(weapon speed %d)", i);
             more_info = 1;
             break;
         }
         if (more_info) {
             if (op->stats.food) {
                 if (op->stats.food != 0)
-                    snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(sustenance%+d)", op->stats.food);
+                    stringbuffer_append_printf(buf, "(sustenance%+d)", op->stats.food);
             }
             if (op->stats.grace) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(grace%+d)", op->stats.grace);
+                stringbuffer_append_printf(buf, "(grace%+d)", op->stats.grace);
             }
             if (op->stats.sp) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(magic%+d)", op->stats.sp);
+                stringbuffer_append_printf(buf, "(magic%+d)", op->stats.sp);
             }
             if (op->stats.hp) {
-                snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(regeneration%+d)", op->stats.hp);
+                stringbuffer_append_printf(buf, "(regeneration%+d)", op->stats.hp);
             }
         }
 
         if (op->stats.luck) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(luck%+d)", op->stats.luck);
+            stringbuffer_append_printf(buf, "(luck%+d)", op->stats.luck);
         }
         if (QUERY_FLAG(op, FLAG_LIFESAVE))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(lifesaving)");
+            stringbuffer_append_string(buf, "(lifesaving)");
         if (QUERY_FLAG(op, FLAG_REFL_SPELL))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(reflect spells)");
+            stringbuffer_append_string(buf, "(reflect spells)");
         if (QUERY_FLAG(op, FLAG_REFL_MISSILE))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(reflect missiles)");
+            stringbuffer_append_string(buf, "(reflect missiles)");
         if (QUERY_FLAG(op, FLAG_STEALTH))
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(stealth)");
+            stringbuffer_append_string(buf, "(stealth)");
         if (op->slaying != NULL && op->type != FOOD) {
-            snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "(slay %s)", op->slaying);
+            stringbuffer_append_printf(buf, "(slay %s)", op->slaying);
         }
-        len = strlen(retbuf);
-        DESCRIBE_ABILITY_SAFE(retbuf, op->attacktype, "Attacks", &len, size);
+        describe_attacktype("Attacks", op->attacktype, buf);
         /* resistance on flesh is only visible for dragons.  If
          * non flesh, everyone can see its resistances
          */
         if (op->type != FLESH || (owner && is_dragon_pl(owner))) {
-            describe_resistance(op, 0, retbuf+strlen(retbuf), size-strlen(retbuf));
+            describe_resistance(op, 0, buf);
         }
-        len = strlen(retbuf);
-        DESCRIBE_PATH_SAFE(retbuf, op->path_attuned, "Attuned", &len, size);
-        DESCRIBE_PATH_SAFE(retbuf, op->path_repelled, "Repelled", &len, size);
-        DESCRIBE_PATH_SAFE(retbuf, op->path_denied, "Denied", &len, size);
+        describe_spellpath_attenuation("Attuned", op->path_attuned &~ op->path_denied, buf);
+        describe_spellpath_attenuation("Repelled", op->path_repelled &~ op->path_denied, buf);
+        describe_spellpath_attenuation("Denied", op->path_denied, buf);
     }
+
+    return buf;
 }
 
 /**
@@ -1432,12 +1419,18 @@ int need_identify(const object *op) {
 /**
  * Identifies an item.
  * Supposed to fix face-values as well here, but later.
+ * Note - this may merge op with other object, so
+ * this function returns either the merged object
+ * or the original if no merge happened.
  *
  * @param op
  * item to identify. Can be already identified without ill effects.
+ * @retval object
+ * The identify object - this may vary from op if the object was
+ * merged.
  */
-void identify(object *op) {
-    object *pl;
+object *identify(object *op) {
+    object *pl, *op1;
 
     SET_FLAG(op, FLAG_IDENTIFIED);
     CLEAR_FLAG(op, FLAG_KNOWN_MAGICAL);
@@ -1465,15 +1458,29 @@ void identify(object *op) {
         }
     }
 
-    /* If the object is on a map, make sure we update its face */
-    if (op->map)
+    if (op->map) {
+        /* If the object is on a map, make sure we update its face.
+         * Also send name and such information to a player standing on it.
+         */
+        object *player = map_find_by_type(op->map, op->x, op->y, PLAYER);
+
         object_update(op, UP_OBJ_FACE);
-    else {
+        op1 = object_merge(op, GET_MAP_TOP(op->map, op->x, op->y));
+        if (op1) op = op1;
+
+        if (player)
+            esrv_update_item(UPD_FACE | UPD_NAME | UPD_FLAGS, player, op);
+
+    } else {
         pl = object_get_player_container(op->env);
+        op1 = object_merge(op, op->env->inv);
+        if (op1) op = op1;
+
         if (pl)
             /* A lot of the values can change from an update - might as well send
              * it all.
              */
             esrv_update_item(UPD_ALL, pl, op);
     }
+    return op;
 }
