@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c 12801 2010-04-03 18:06:12Z akirschbaum $";
+ *   "$Id: living.c 14035 2010-10-24 18:09:08Z kbulgrien $";
  */
 
 /*
@@ -35,6 +35,7 @@
  */
 
 #include <stdlib.h>
+#include <assert.h>
 #include <global.h>
 #include <sproto.h>
 
@@ -524,7 +525,7 @@ void check_stat_bounds(living *stats, sint8 min_stat, sint8 max_stat) {
  * function since some of the values passed to draw_ext_info are hardcoded.
  */
 #define DIFF_MSG(flag, subtype1, subtype2, msg1, msg2) \
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, (flag > 0) ? subtype1 : subtype2, (flag > 0) ? msg1 : msg2, NULL);
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, (flag > 0) ? subtype1 : subtype2, (flag > 0) ? msg1 : msg2);
 
 /**
  * Permanently alters an object's stats/flags based on another object.
@@ -660,7 +661,7 @@ int change_abil(object *op, object *tmp) {
          */
         if (tmp->move_type&MOVE_FLY_LOW && !(op->move_type&MOVE_FLY_HIGH)) {
             DIFF_MSG(flag, MSG_TYPE_ATTRIBUTE_MOVE, MSG_TYPE_ATTRIBUTE_MOVE,
-                     "You start to float in the air!.",
+                     "You start to float in the air!",
                      "You float down to the ground.");
         }
 
@@ -669,7 +670,7 @@ int change_abil(object *op, object *tmp) {
              * in that case, you don't actually land
              */
             DIFF_MSG(flag, MSG_TYPE_ATTRIBUTE_MOVE, MSG_TYPE_ATTRIBUTE_MOVE,
-                     "You soar into the air air!.",
+                     "You soar into the air!.",
                      (op->move_type&MOVE_FLY_LOW ? "You fly lower in the air":
                       "You float down to the ground."));
         }
@@ -692,7 +693,7 @@ int change_abil(object *op, object *tmp) {
                 if (op->race)
                     free_string(op->race);
                 op->race = add_string("undead");
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_RACE, "Your lifeforce drains away!", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_RACE, "Your lifeforce drains away!");
             } else {
                 if (op->race)
                     free_string(op->race);
@@ -700,7 +701,7 @@ int change_abil(object *op, object *tmp) {
                     op->race = add_string(op->arch->clone.race);
                 else
                     op->race = NULL;
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_RACE, "Your lifeforce returns!", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_RACE, "Your lifeforce returns!");
             }
         }
 
@@ -723,18 +724,18 @@ int change_abil(object *op, object *tmp) {
         success = 1;
         if (flag > 0) {
             if (QUERY_FLAG(op, FLAG_WIZ))
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START, "Your mortal self is blinded.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START, "Your mortal self is blinded.");
             else {
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START, "You are blinded.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START, "You are blinded.");
                 SET_FLAG(op, FLAG_BLIND);
                 if (op->type == PLAYER)
                     op->contr->do_los = 1;
             }
         } else {
             if (QUERY_FLAG(op, FLAG_WIZ))
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END, "Your mortal self can now see again.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END, "Your mortal self can now see again.");
             else {
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END, "Your vision returns.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END, "Your vision returns.");
                 CLEAR_FLAG(op, FLAG_BLIND);
                 if (op->type == PLAYER)
                     op->contr->do_los = 1;
@@ -755,17 +756,17 @@ int change_abil(object *op, object *tmp) {
         success = 1;
         if (flag > 0) {
             if (QUERY_FLAG(op, FLAG_WIZ))
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_START, "Your vision becomes a little clearer.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_START, "Your vision becomes a little clearer.");
             else {
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_START, "Everything becomes transparent.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_START, "Everything becomes transparent.");
                 if (op->type == PLAYER)
                     op->contr->do_los = 1;
             }
         } else {
             if (QUERY_FLAG(op, FLAG_WIZ))
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_END, "Your vision becomes a bit out of focus.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_END, "Your vision becomes a bit out of focus.");
             else {
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_END, "Everything suddenly looks very solid.", NULL);
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOOD_EFFECT_END, "Everything suddenly looks very solid.");
                 if (op->type == PLAYER)
                     op->contr->do_los = 1;
             }
@@ -817,12 +818,10 @@ int change_abil(object *op, object *tmp) {
             success = 1;
             if (op->resist[i] > refop.resist[i])
                 draw_ext_info_format(NDI_UNIQUE|NDI_BLUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_PROTECTION_GAIN,
-                                     "Your resistance to %s rises to %d%%.",
                                      "Your resistance to %s rises to %d%%.",
                                      change_resist_msg[i], op->resist[i]);
             else
                 draw_ext_info_format(NDI_UNIQUE|NDI_BLUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_PROTECTION_LOSS,
-                                     "Your resistance to %s drops to %d%%.",
                                      "Your resistance to %s drops to %d%%.",
                                      change_resist_msg[i], op->resist[i]);
         }
@@ -876,7 +875,7 @@ void drain_specific_stat(object *op, int deplete_stats) {
         }
     }
 
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_STAT_LOSS, drain_msg[deplete_stats], NULL);
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_STAT_LOSS, drain_msg[deplete_stats]);
     change_attr_value(&tmp->stats, deplete_stats, -1);
     fix_object(op);
 }
@@ -908,7 +907,7 @@ int remove_depletion(object *op, int level) {
     for (i = 0; i < NUM_STATS; i++) {
         if (get_attr_value(&depl->stats, i)) {
             count++;
-            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_STAT_GAIN, restore_msg[i], NULL);
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_STAT_GAIN, restore_msg[i]);
         }
     }
 
@@ -1017,6 +1016,202 @@ void add_statbonus(object *op) {
     op->contr->orig_stats.Int += op->arch->clone.stats.Int;
 }
 
+/**
+ * Complement to fix_object() for player.
+ * Figure out the players sp/mana/hp totals. Also do encumberance and various speeds.
+ * @param op object being fixed, must not be NULL.
+ * @param ac current ac, may be adjusted.
+ * @param wc current wc, may be adjusted.
+ * @param grace_obj praying skill, can be NULL.
+ * @param mana_obj spell-casting skill, can be NULL.
+ * @param wc_obj applied combat skill, can be NULL.
+ * @param weapon_speed current weapon's speed.
+ * @param added_speed speed bonus from items.
+ */
+static void fix_player(object *op, int *ac, int *wc, const object *grace_obj, const object *mana_obj, const object *wc_obj, int weapon_speed, float added_speed)
+{
+    int pl_level, i, j;
+    float character_load = 0.0;
+
+    if (op->type != PLAYER)
+        return;
+
+    check_stat_bounds(&(op->stats), MIN_STAT, MAX_STAT);
+    pl_level = op->level;
+
+    if (pl_level < 1)
+        pl_level = 1; /* safety, we should always get 1 levels worth of hp! */
+
+    /* You basically get half a con bonus/level.  But we do take into account rounding,
+     * so if your bonus is 7, you still get 7 worth of bonus every 2 levels.
+     */
+    for (i = 1, op->stats.maxhp = 0; i <= pl_level && i <= 10; i++) {
+        j = op->contr->levhp[i]+get_con_bonus(op->stats.Con)/2;
+        if (i%2 && get_con_bonus(op->stats.Con)%2) {
+            if (get_con_bonus(op->stats.Con) > 0)
+                j++;
+            else
+                j--;
+        }
+        op->stats.maxhp += j > 1 ? j : 1; /* always get at least 1 hp/level */
+    }
+
+    if (op->level > 10)
+        op->stats.maxhp += 2 * (op->level - 10);
+
+    op->stats.maxhp += op->arch->clone.stats.maxhp;
+
+    if (op->stats.hp > op->stats.maxhp)
+        op->stats.hp = op->stats.maxhp;
+
+    /* Sp gain is controlled by the level of the player's
+     * relevant experience object (mana_obj, see above) */
+
+    /* set maxsp */
+    if (!mana_obj || !mana_obj->level) {
+        op->stats.maxsp = 1;
+    } else {
+        float sp_tmp = 0.0;
+        for (i = 1; i <= mana_obj->level && i <= 10; i++) {
+            float stmp;
+
+            /* Got some extra bonus at first level */
+            if (i < 2) {
+                stmp = op->contr->levsp[i]+(2.0*get_sp_bonus(op->stats.Pow)+get_sp_bonus(op->stats.Int))/6.0;
+            } else {
+                stmp = op->contr->levsp[i]+(2.0*get_sp_bonus(op->stats.Pow)+get_sp_bonus(op->stats.Int))/12.0;
+            }
+            if (stmp < 1.0)
+                stmp = 1.0;
+            sp_tmp += stmp;
+        }
+        op->stats.maxsp = (int)sp_tmp+op->arch->clone.stats.maxsp;
+
+        if (mana_obj->level > 10)
+            op->stats.maxsp += 2 * (mana_obj->level - 10);
+    }
+
+    /* Characters can get their sp supercharged via rune of transferrance */
+    if (op->stats.sp > op->stats.maxsp*2)
+        op->stats.sp = op->stats.maxsp*2;
+
+    /* set maxgrace, notice 3-4 lines below it depends on both Wis and Pow */
+    if (!grace_obj || !grace_obj->level) {
+        op->stats.maxgrace = 1;
+    } else {
+        /* store grace in a float - this way, the divisions below don't create
+         * big jumps when you go from level to level - with int's, it then
+         * becomes big jumps when the sums of the bonuses jump to the next
+         * step of 8 - with floats, even fractional ones are useful.
+         */
+        float sp_tmp = 0.0;
+        for (i = 1, op->stats.maxgrace = 0; i <= grace_obj->level && i <= 10; i++) {
+            float grace_tmp = 0.0;
+
+            /* Got some extra bonus at first level */
+            if (i < 2) {
+                grace_tmp = op->contr->levgrace[i]+(get_grace_bonus(op->stats.Pow)+2.0*get_grace_bonus(op->stats.Wis))/6.0;
+            } else {
+                grace_tmp = op->contr->levgrace[i]+(get_grace_bonus(op->stats.Pow)+2.0*get_grace_bonus(op->stats.Wis))/12.0;
+            }
+            if (grace_tmp < 1.0)
+                grace_tmp = 1.0;
+            sp_tmp += grace_tmp;
+        }
+        op->stats.maxgrace = (int)sp_tmp+op->arch->clone.stats.maxgrace;
+
+        /* two grace points per level after 11 */
+        if (grace_obj->level > 10)
+            op->stats.maxgrace += 2 * (grace_obj->level - 10);
+    }
+    /* No limit on grace vs maxgrace */
+
+    if (op->contr->braced) {
+        (*ac) += 2;
+        (*wc) += 4;
+    } else
+        (*ac) -= get_dex_bonus(op->stats.Dex);
+
+    /* In new exp/skills system, wc bonuses are related to
+     * the players level in a relevant exp object (wc_obj)
+     * not the general player level -b.t.
+     * I changed this slightly so that wc bonuses are better
+     * than before. This is to balance out the fact that
+     * the player no longer gets a personal weapon w/ 1
+     * improvement every level, now its fighterlevel/5. So
+     * we give the player a bonus here in wc and dam
+     * to make up for the change. Note that I left the
+     * monster bonus the same as before. -b.t.
+     */
+
+    if (wc_obj && wc_obj->level >= 1) {
+        const  char *wc_in = object_get_value(wc_obj, "wc_increase_rate");
+        int wc_increase_rate;
+
+        wc_increase_rate = wc_in?atoi(wc_in):5;
+        (*wc) -= get_thaco_bonus(op->stats.Str);
+        (*wc) -= (wc_obj->level-1)/wc_increase_rate;
+        op->stats.dam += (wc_obj->level-1)/4;
+    } else {
+        (*wc) -= (((op->level-1)/5)+get_thaco_bonus(op->stats.Str));
+    }
+    op->stats.dam += get_dam_bonus(op->stats.Str);
+
+    if (op->stats.dam < 1)
+        op->stats.dam = 1;
+
+    op->speed = MAX_PLAYER_SPEED+get_speed_bonus(op->stats.Dex);
+
+    if (settings.search_items && op->contr->search_str[0])
+        op->speed -= 0.25;
+
+    if (op->attacktype == 0)
+        op->attacktype = op->arch->clone.attacktype;
+
+
+    /* First block is for encumbrance of the player */
+
+    /* The check for FREE_PLAYER_LOAD_PERCENT < 1.0 is really a safety.  One would
+     * think that it should never be the case if that is set to 1.0, that carrying
+     * would be above the limit.  But it could be if character is weakened and
+     * was otherwise at limit.  Without that safety, could get divide by zeros.
+     */
+    if (op->carrying > (get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT)
+    && (FREE_PLAYER_LOAD_PERCENT < 1.0)) {
+        int extra_weight = op->carrying-get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT;
+
+        character_load = (float)extra_weight/(float)(get_weight_limit(op->stats.Str)*(1.0-FREE_PLAYER_LOAD_PERCENT));
+
+        /* character_load is used for weapon speed below, so sanitize value */
+        if (character_load >= 1.0)
+            character_load = 1.0;
+
+        /* If a character is fully loaded, they will always get cut down to min
+         * speed no matter what their dex.  Note that magic is below, so
+         * still helps out.
+         */
+        if (op->speed > MAX_PLAYER_SPEED)
+            op->speed -= character_load*(op->speed-MIN_PLAYER_SPEED);
+        else
+            op->speed -= character_load*(MAX_PLAYER_SPEED-MIN_PLAYER_SPEED);
+    }
+
+    /* This block is for weapon speed */
+    op->weapon_speed = BASE_WEAPON_SPEED+get_speed_bonus(op->stats.Dex)-weapon_speed/20.0+added_speed/10.0;
+    if (wc_obj) {
+        op->weapon_speed += 0.005*wc_obj->level;
+    } else
+        op->weapon_speed += 0.005*op->level;
+
+    /* character_load=1.0 means character is fully loaded, 0.0 is unloaded.  Multiplying
+     * by 0.2 for penalty is purely arbitrary, but slows things down without completely
+     * stopping them.
+     */
+    op->weapon_speed -= character_load*0.2;
+
+    if (op->weapon_speed < 0.05)
+        op->weapon_speed = 0.05;
+}
 /**
  * Updates all abilities given by applied objects in the inventory
  * of the given object.
@@ -1038,12 +1233,12 @@ void add_statbonus(object *op) {
  * this function is too long, and should be cleaned / split.
  */
 void fix_object(object *op) {
-    int i, j;
-    float max = 9, added_speed = 0, sp_tmp, speed_reduce_from_disease = 1;
+    int i;
+    float max = 9, added_speed = 0, speed_reduce_from_disease = 1;
     int weapon_weight = 0, weapon_speed = 0;
     int best_wc = 0, best_ac = 0, wc = 0, ac = 0;
     int prot[NROFATTACKS], vuln[NROFATTACKS], potion_resist[NROFATTACKS];
-    object *grace_obj = NULL, *mana_obj = NULL, *wc_obj = NULL;
+    const object *grace_obj = NULL, *mana_obj = NULL, *wc_obj = NULL;
 
     /* First task is to clear all the values back to their original values */
     if (op->type == PLAYER) {
@@ -1252,7 +1447,7 @@ void fix_object(object *op) {
                     /* Potential for cursed potions, in which case we just can use
                      * a straight MAX, as potion_resist is initialized to zero.
                      */
-                    if (tmp->type == POTION_EFFECT) {
+                    if (tmp->type == POTION_RESIST_EFFECT) {
                         if (potion_resist[i])
                             potion_resist[i] = MAX(potion_resist[i], tmp->resist[i]);
                         else
@@ -1444,203 +1639,7 @@ void fix_object(object *op) {
             op->resist[i] = potion_resist[i];
     }
 
-    /* Figure out the players sp/mana/hp totals.
-     * Also do encumberance.  This entire blob should
-     * probably be its own function.
-     */
-    if (op->type == PLAYER) {
-        int pl_level;
-        float character_load = 0.0;
-
-        check_stat_bounds(&(op->stats), MIN_STAT, MAX_STAT);
-        pl_level = op->level;
-
-        if (pl_level < 1)
-            pl_level = 1; /* safety, we should always get 1 levels worth of hp! */
-
-        /* You basically get half a con bonus/level.  But we do take into account rounding,
-         * so if your bonus is 7, you still get 7 worth of bonus every 2 levels.
-         */
-        for (i = 1, op->stats.maxhp = 0; i <= pl_level && i <= 10; i++) {
-            j = op->contr->levhp[i]+get_con_bonus(op->stats.Con)/2;
-            if (i%2 && get_con_bonus(op->stats.Con)%2) {
-                if (get_con_bonus(op->stats.Con) > 0)
-                    j++;
-                else
-                    j--;
-            }
-            op->stats.maxhp += j > 1 ? j : 1; /* always get at least 1 hp/level */
-        }
-
-        for (i = 11; i <= op->level; i++)
-            op->stats.maxhp += 2;
-
-        op->stats.maxhp += op->arch->clone.stats.maxhp;
-
-        if (op->stats.hp > op->stats.maxhp)
-            op->stats.hp = op->stats.maxhp;
-
-        /* Sp gain is controlled by the level of the player's
-         * relevant experience object (mana_obj, see above)
-         */
-        /* following happen when skills system is not used */
-        if (!mana_obj)
-            mana_obj = op;
-        if (!grace_obj)
-            grace_obj = op;
-
-        /* set maxsp */
-        if (!mana_obj || !mana_obj->level || op->type != PLAYER)
-            mana_obj = op;
-
-        if (mana_obj == op && op->type == PLAYER) {
-            op->stats.maxsp = 1;
-        } else {
-            sp_tmp = 0.0;
-            for (i = 1; i <= mana_obj->level && i <= 10; i++) {
-                float stmp;
-
-                /* Got some extra bonus at first level */
-                if (i < 2) {
-                    stmp = op->contr->levsp[i]+(2.0*get_sp_bonus(op->stats.Pow)+get_sp_bonus(op->stats.Int))/6.0;
-                } else {
-                    stmp = op->contr->levsp[i]+(2.0*get_sp_bonus(op->stats.Pow)+get_sp_bonus(op->stats.Int))/12.0;
-                }
-                if (stmp < 1.0)
-                    stmp = 1.0;
-                sp_tmp += stmp;
-            }
-            op->stats.maxsp = (int)sp_tmp+op->arch->clone.stats.maxsp;
-
-            for (i = 11; i <= mana_obj->level; i++)
-                op->stats.maxsp += 2;
-        }
-
-        /* Characters can get their sp supercharged via rune of transferrance */
-        if (op->stats.sp > op->stats.maxsp*2)
-            op->stats.sp = op->stats.maxsp*2;
-
-        /* set maxgrace, notice 3-4 lines below it depends on both Wis and Pow */
-        if (!grace_obj || !grace_obj->level || op->type != PLAYER)
-            grace_obj = op;
-
-        if (grace_obj == op && op->type == PLAYER) {
-            op->stats.maxgrace = 1;
-        } else {
-            /* store grace in a float - this way, the divisions below don't create
-             * big jumps when you go from level to level - with int's, it then
-             * becomes big jumps when the sums of the bonuses jump to the next
-             * step of 8 - with floats, even fractional ones are useful.
-             */
-            sp_tmp = 0.0;
-            for (i = 1, op->stats.maxgrace = 0; i <= grace_obj->level && i <= 10; i++) {
-                float grace_tmp = 0.0;
-
-                /* Got some extra bonus at first level */
-                if (i < 2) {
-                    grace_tmp = op->contr->levgrace[i]+(get_grace_bonus(op->stats.Pow)+2.0*get_grace_bonus(op->stats.Wis))/6.0;
-                } else {
-                    grace_tmp = op->contr->levgrace[i]+(get_grace_bonus(op->stats.Pow)+2.0*get_grace_bonus(op->stats.Wis))/12.0;
-                }
-                if (grace_tmp < 1.0)
-                    grace_tmp = 1.0;
-                sp_tmp += grace_tmp;
-            }
-            op->stats.maxgrace = (int)sp_tmp+op->arch->clone.stats.maxgrace;
-
-            /* two grace points per level after 11 */
-            for (i = 11; i <= grace_obj->level; i++)
-                op->stats.maxgrace += 2;
-        }
-        /* No limit on grace vs maxgrace */
-
-        if (op->contr->braced) {
-            ac += 2;
-            wc += 4;
-        } else
-            ac -= get_dex_bonus(op->stats.Dex);
-
-        /* In new exp/skills system, wc bonuses are related to
-         * the players level in a relevant exp object (wc_obj)
-         * not the general player level -b.t.
-         * I changed this slightly so that wc bonuses are better
-         * than before. This is to balance out the fact that
-         * the player no longer gets a personal weapon w/ 1
-         * improvement every level, now its fighterlevel/5. So
-         * we give the player a bonus here in wc and dam
-         * to make up for the change. Note that I left the
-         * monster bonus the same as before. -b.t.
-         */
-
-        if (op->type == PLAYER && wc_obj && wc_obj->level >= 1) {
-            const  char *wc_in = object_get_value(wc_obj, "wc_increase_rate");
-            int wc_increase_rate;
-
-            wc_increase_rate = wc_in?atoi(wc_in):5;
-            wc -= get_thaco_bonus(op->stats.Str);
-            wc -= (wc_obj->level-1)/wc_increase_rate;
-            op->stats.dam += (wc_obj->level-1)/4;
-        } else {
-            wc -= (((op->level-1)/5)+get_thaco_bonus(op->stats.Str));
-        }
-        op->stats.dam += get_dam_bonus(op->stats.Str);
-
-        if (op->stats.dam < 1)
-            op->stats.dam = 1;
-
-        op->speed = MAX_PLAYER_SPEED+get_speed_bonus(op->stats.Dex);
-
-        if (settings.search_items && op->contr->search_str[0])
-            op->speed -= 0.25;
-
-        if (op->attacktype == 0)
-            op->attacktype = op->arch->clone.attacktype;
-
-
-        /* First block is for encumbrance of the player */
-
-        /* The check for FREE_PLAYER_LOAD_PERCENT < 1.0 is really a safety.  One would
-         * think that it should never be the case if that is set to 1.0, that carrying
-         * would be above the limit.  But it could be if character is weakened and
-         * was otherwise at limit.  Without that safety, could get divide by zeros.
-         */
-        if (op->carrying > (get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT)
-        && (FREE_PLAYER_LOAD_PERCENT < 1.0)) {
-            int extra_weight = op->carrying-get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT;
-
-            character_load = (float)extra_weight/(float)(get_weight_limit(op->stats.Str)*(1.0-FREE_PLAYER_LOAD_PERCENT));
-
-            /* character_load is used for weapon speed below, so sanitize value */
-            if (character_load >= 1.0)
-                character_load = 1.0;
-
-            /* If a character is fully loaded, they will always get cut down to min
-             * speed no matter what their dex.  Note that magic is below, so
-             * still helps out.
-             */
-            if (op->speed > MAX_PLAYER_SPEED)
-                op->speed -= character_load*(op->speed-MIN_PLAYER_SPEED);
-            else
-                op->speed -= character_load*(MAX_PLAYER_SPEED-MIN_PLAYER_SPEED);
-        }
-
-        /* This block is for weapon speed */
-        op->weapon_speed = BASE_WEAPON_SPEED+get_speed_bonus(op->stats.Dex)-weapon_speed/20.0+added_speed/10.0;
-        if (wc_obj) {
-            op->weapon_speed += 0.005*wc_obj->level;
-        } else
-            op->weapon_speed += 0.005*op->level;
-
-        /* character_load=1.0 means character is fully loaded, 0.0 is unloaded.  Multiplying
-         * by 0.2 for penalty is purely arbitrary, but slows things down without completely
-         * stopping them.
-         */
-        op->weapon_speed -= character_load*0.2;
-
-        if (op->weapon_speed < 0.05)
-            op->weapon_speed = 0.05;
-
-    }
+    fix_player(op, &ac, &wc, grace_obj, mana_obj, wc_obj, weapon_speed, added_speed);
 
     op->speed = op->speed*speed_reduce_from_disease;
 
@@ -1804,7 +1803,6 @@ static void dragon_level_gain(object *who) {
         if (abil->last_eat > 0 && atnr_is_dragon_enabled(abil->last_eat)) {
             /* apply new ability focus */
             draw_ext_info_format(NDI_UNIQUE|NDI_BLUE, 0, who, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_RACE,
-                                 "Your metabolism now focuses on %s!",
                                  "Your metabolism now focuses on %s!",
                                  change_resist_msg[abil->last_eat]);
 
@@ -1871,9 +1869,9 @@ object *give_skill_by_name(object *op, const char *skill_name) {
  * Will tell the player about changed levels.
  *
  * @param who
- * player
+ * player, must not be NULL.
  * @param op
- * what we are checking to gain the level (eg, skill)
+ * what we are checking to gain the level (eg, skill), can be NULL.
  *
  * @note
  * this function can call itself recursively to check for multiple levels.
@@ -1881,6 +1879,8 @@ object *give_skill_by_name(object *op, const char *skill_name) {
 void player_lvl_adj(object *who, object *op) {
     char buf[MAX_BUF];
 
+    assert(who);
+
     if (!op)       /* when rolling stats */
         op = who;
 
@@ -1891,35 +1891,32 @@ void player_lvl_adj(object *who, object *op) {
             dragon_level_gain(who);
 
         /* Only roll these if it is the player (who) that gained the level */
-        if (who && op == who && (who->level < 11) && who->type == PLAYER) {
+        if (op == who && (who->level < 11) && who->type == PLAYER) {
             who->contr->levhp[who->level] = die_roll(2, 4, who, PREFER_HIGH)+1;
             who->contr->levsp[who->level] = die_roll(2, 3, who, PREFER_HIGH);
             who->contr->levgrace[who->level] = die_roll(2, 2, who, PREFER_HIGH)-1;
         }
 
-        if (who)
-            fix_object(who);
+        fix_object(who);
         if (op->level > 1) {
             if (op->type != PLAYER)
                 snprintf(buf, sizeof(buf), "You are now level %d in the %s skill.", op->level, op->name);
             else
                 snprintf(buf, sizeof(buf), "You are now level %d.", op->level);
 
-            if (who)
-                draw_ext_info(NDI_UNIQUE|NDI_RED, 0, who, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_LEVEL_GAIN, buf, buf);
+            draw_ext_info(NDI_UNIQUE|NDI_RED, 0, who, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_LEVEL_GAIN, buf);
         }
         player_lvl_adj(who, op); /* To increase more levels */
     } else if (op->level > 1 && op->stats.exp < level_exp(op->level, who->expmul)) {
         op->level--;
-        if (who)
-            fix_object(who);
-        if (op->type != PLAYER) {
-            if (who)
-                draw_ext_info_format(NDI_UNIQUE|NDI_RED, 0, who, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_LEVEL_LOSS,
-                                     "You are now level %d in the %s skill.",
-                                     "You are now level %d in the %s skill.",
-                                     op->level, op->name);
-        }
+        fix_object(who);
+        if (op->type != PLAYER)
+            snprintf(buf, sizeof(buf), "You are now level %d in the %s skill.", op->level, op->name);
+        else
+            snprintf(buf, sizeof(buf), "You are now level %d.", op->level);
+
+        draw_ext_info(NDI_UNIQUE|NDI_RED, 0, who, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_LEVEL_LOSS, buf);
+
         player_lvl_adj(who, op); /* To decrease more levels */
     }
     /* check if the spell data has changed */
@@ -2397,9 +2394,11 @@ static int get_grace_bonus(int stat) {
 
 /**
  * Limits a stat value to [0..<code>max_index</code>].
+ * @param stat
+ * index of the stat to get.
  * @param max_index
- * the maximum index (exclusive)
- * @return the limited index
+ * the maximum index (exclusive).
+ * @return the limited index.
  */
 static size_t get_index(int stat, size_t max_index) {
     size_t index;
