@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c 12653 2010-03-14 21:51:44Z ryo_saeba $";
+ *   "$Id: map.c 13962 2010-10-03 09:47:30Z ryo_saeba $";
  */
 
 /*
@@ -439,7 +439,7 @@ int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
                          */
                         draw_ext_info(NDI_UNIQUE|NDI_NAVY, 0, ob,
                                       MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_NOKEY,
-                                      tmp->msg, tmp->msg);
+                                      tmp->msg);
                     }
                     return 1;
                 }
@@ -451,7 +451,7 @@ int blocked_link(object *ob, mapstruct *m, int sx, int sy) {
                     if (tmp->msg) {
                         draw_ext_info(NDI_UNIQUE|NDI_NAVY, 0, ob,
                                       MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_NOKEY,
-                                      tmp->msg, tmp->msg);
+                                      tmp->msg);
                     }
                     return 1;
                 }
@@ -686,6 +686,15 @@ static void load_objects(mapstruct *m, FILE *fp, int mapflags) {
             continue;
         }
 
+        /*
+         * You can NOT have players on a map being loaded.
+         * Trying to use such a type leads to crashes everywhere as op->contr is NULL.
+         */
+        if (op->type == PLAYER) {
+            LOG(llevError, "Discarding invalid item with type PLAYER in map %s\n", m->path);
+            continue;
+        }
+
         switch (i) {
         case LL_NORMAL:
             /* if we are loading an overlay, put the floors on the bottom */
@@ -2034,9 +2043,9 @@ int change_map_light(mapstruct *m, int change) {
 
     /* inform all players on the map */
     if (change > 0)
-        ext_info_map(NDI_BLACK, m, MSG_TYPE_MISC, MSG_SUBTYPE_NONE, "It becomes darker.", NULL);
+        ext_info_map(NDI_BLACK, m, MSG_TYPE_MISC, MSG_SUBTYPE_NONE, "It becomes darker.");
     else
-        ext_info_map(NDI_BLACK, m, MSG_TYPE_MISC, MSG_SUBTYPE_NONE, "It becomes brighter.", NULL);
+        ext_info_map(NDI_BLACK, m, MSG_TYPE_MISC, MSG_SUBTYPE_NONE, "It becomes brighter.");
 
     /* Do extra checking.  since m->darkness is a unsigned value,
      * we need to be extra careful about negative values.
@@ -2676,3 +2685,25 @@ object *map_find_by_flag(mapstruct *map, int x, int y, int flag) {
     }
     return NULL;
 }
+
+/**
+ * Remove files containing the map's unique items.
+ * @param map
+ */
+void map_remove_unique_files(const mapstruct *map) {
+    char base[HUGE_BUF], path[HUGE_BUF];
+    int count;
+
+    if (map->unique) {
+        /* Unique maps have their full path already set. */
+        unlink(map->path);
+        return;
+    }
+
+    create_items_path(map->path, base, sizeof(base));
+
+    for (count = 0; count < 10; count++) {
+        snprintf(path, sizeof(path), "%s.v%02d", base, count);
+        unlink(path);
+    }
+}
