@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_porting_c =
- *   "$Id: porting.c 12597 2010-03-07 20:37:38Z akirschbaum $";
+ *   "$Id: porting.c 14250 2011-01-25 22:26:24Z ryo_saeba $";
  */
 
 /*
@@ -38,6 +38,7 @@
 #include <string.h>
 #ifdef WIN32 /* ---win32 exclude/include headers */
 #include "process.h"
+#include <fcntl.h>
 #define pid_t int  /* we include it non global, because there is a redefinition in python.h */
 #else
 #include <ctype.h>
@@ -106,9 +107,9 @@ char *tempnam_local(const char *dir, const char *pfx) {
             return(NULL);
         do {
 #ifdef HAVE_SNPRINTF
-            (void)snprintf(name, MAXPATHLEN, "%s/%s%hx.%u", dir, pfx, pid, curtmp);
+            (void)snprintf(name, MAXPATHLEN, "%s/%s%x.%u", dir, pfx, (unsigned int)pid, curtmp);
 #else
-            (void)sprintf(name, "%s/%s%hx%u", dir, pfx, pid, curtmp);
+            (void)sprintf(name, "%s/%s%x%u", dir, pfx, (unsigned int)pid, curtmp);
 #endif
             curtmp++;
         } while (access(name, F_OK) != -1);
@@ -531,14 +532,14 @@ char *strerror_local(int errnum, char *buf, size_t size) {
     buf[0] = 0;
     bbuf = (char *)strerror_r(errnum, buf, size);
     if ((buf[0] == 0) && (bbuf != NULL))
-        strncpy(buf, bbuf, size);
+        snprintf(buf, size, "%s", bbuf);
 # else
     if (strerror_r(errnum, buf, size) != 0) {
         /* EINVAL and ERANGE are possible errors from this strerror_r */
         if (errno == ERANGE) {
-            strncat(buf, "Too small buffer.", size);
+            snprintf(buf, size, "Too small buffer.");
         } else if (errno == EINVAL) {
-            strncat(buf, "Error number invalid.", size);
+            snprintf(buf, size, "Error number invalid.");
         }
     }
 # endif /* STRERROR_R_CHAR_P */
