@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_utils_c =
- *   "$Id: utils.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *   "$Id: utils.c 14079 2010-11-10 19:41:37Z ryo_saeba $";
  */
 
 /*
@@ -375,30 +375,6 @@ void set_materialname(object *op, int difficulty, materialtype_t *nmt) {
     }
 }
 
-/**
- * Strip out the media tags from a String.
- * Warning the input string will contain the result string
- */
-void strip_media_tag(char *message) {
-    int in_tag = 0;
-    char *dest;
-    char *src;
-    src = dest = message;
-
-    while (*src != '\0') {
-        if (*src == '[') {
-            in_tag = 1;
-        } else if (in_tag && (*src == ']'))
-            in_tag = 0;
-        else if (!in_tag) {
-            *dest = *src;
-            dest++;
-        }
-        src++;
-    }
-    *dest = '\0';
-}
-
 /**
  * Finds a string in a string.
  *
@@ -605,3 +581,59 @@ size_t split_string(char *str, char *array[], size_t array_size, char sep) {
     }
     return pos;
 }
+
+/**
+ * Describe the specified path attenuation.
+ * @param attenuation string describing if "Attenued", "Denied", "Repelled".
+ * @param value path value to describe.
+ * @param buf where to describe, can be NULL.
+ * @return buf, newly allocated StringBuffer the caller should free if buf was NULL.
+ */
+StringBuffer *describe_spellpath_attenuation(const char *attenuation, int value, StringBuffer *buf) {
+    if (buf == NULL)
+        buf = stringbuffer_new();
+
+    if (value) {
+        int i, j = 0;
+        stringbuffer_append_printf(buf, "(%s: ", attenuation);
+        for (i = 0; i < NRSPELLPATHS; i++)
+            if (value&(1<<i)) {
+                if (j)
+                    stringbuffer_append_string(buf, ", ");
+                else
+                    j = 1;
+                stringbuffer_append_string(buf, spellpathnames[i]);
+            }
+        stringbuffer_append_string(buf, ")");
+    }
+
+    return buf;
+}
+
+/**
+ * Describe the specified attack type.
+ * @param attack string describing the attack ("Clawing", and such).
+ * @param value attack type to describe.
+ * @param buf where to describe, can be NULL.
+ * @return buf, newly allocated StringBuffer the caller should free if buf was NULL.
+ */
+StringBuffer *describe_attacktype(const char *attack, int value, StringBuffer *buf) {
+    if (buf == NULL)
+        buf = stringbuffer_new();
+
+    if (value) {
+        int i, j = 0;
+        stringbuffer_append_printf(buf, "(%s: ", attack);
+        for (i = 0; i < NROFATTACKS; i++)
+            if (value&(1<<i)) {
+                if (j)
+                    stringbuffer_append_string(buf, ", ");
+                else
+                    j = 1;
+                stringbuffer_append_string(buf, attacks[i]);
+            }
+        stringbuffer_append_string(buf, ")");
+    }
+
+    return buf;
+}
