@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h 12291 2009-11-28 22:40:21Z anmaster $";
+ *   "$Id: define.h 14097 2010-11-15 07:17:37Z mwedel $";
  */
 
 /*
@@ -122,7 +122,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define POISON                      7
 #define BOOK                        8
 #define CLOCK                       9
-/*#define FBULLET                   10 */
+#define DRAGON_FOCUS                10    /**< Used during character creation */
 /*#define FBALL                     11 */
 /*#define LIGHTNING                 12 */
 #define ARROW                       13
@@ -266,8 +266,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define DIRECTOR                    112
 #define GIRDLE                      113
 #define FORCE                       114
-#define POTION_EFFECT               115   /**< A force, holding the effect of a
-                                           * potion */
+#define POTION_RESIST_EFFECT        115   /**< A force, holding the effect of a
+                                           * resistance potion */
 #define EVENT_CONNECTOR             116   /**< Lauwenmark: an invisible object
                                            * holding a plugin event hook */
 #define CLOSE_CON                   121   /**< Eneq((at)csd.uu.se): Id for
@@ -382,7 +382,7 @@ typedef struct typedata {
 #define PU_KEY                  0x00008000
 
 #define PU_MISSILEWEAPON        0x00010000
-#define PU_ALLWEAPON            0x00020000
+#define PU_MELEEWEAPON          0x00020000
 #define PU_MAGICAL              0x00040000
 #define PU_POTION               0x00080000
 
@@ -394,6 +394,7 @@ typedef struct typedata {
 #define PU_NOT_CURSED           0x01000000
 #define PU_JEWELS               0x02000000
 #define PU_FLESH                0x04000000
+#define PU_CONTAINER            0x08000000
 /*@}*/
 
 /* Instead of using arbitrary constants for indexing the
@@ -438,7 +439,7 @@ typedef struct typedata {
 /**
  * @defgroup FLAG_xxx Object flags
  *
- * Those flags are object-related flags, stored in the ::obj::flags fields.
+ * Those flags are object-related flags, stored in the obj::flags fields.
  *
  * Flag structure now changed.
  * Each flag is now a bit offset, starting at zero.  The macros
@@ -1138,8 +1139,6 @@ static inline void safe_strcat(char *dest, const char *orig, size_t *curlen, siz
  * Constructs a loop iterating over an object and all objects above it in the
  * same pile.
  * @param op_ the object to start with
- * @param it_ a variable name that holds the inventory objects; the variable
- * must be declared outside of the loop; modifications do not affect the loop
  */
 #define FOR_OB_AND_ABOVE_PREPARE(op_) FOR_OB_PREPARE(op_, above, __LINE__)
 /**
@@ -1151,8 +1150,6 @@ static inline void safe_strcat(char *dest, const char *orig, size_t *curlen, siz
  * Constructs a loop iterating over an object and all objects below it in the
  * same pile.
  * @param op_ the object to start with
- * @param it_ a variable name that holds the inventory objects; the variable
- * must be declared outside of the loop; modifications do not affect the loop
  */
 #define FOR_OB_AND_BELOW_PREPARE(op_) FOR_OB_PREPARE(op_, below, __LINE__)
 /**
