@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h 12636 2010-03-14 04:54:18Z mwedel $";
+ *   "$Id: global.h 14209 2010-12-18 16:19:30Z ryo_saeba $";
  */
 
 /*
@@ -83,8 +83,6 @@ typedef signed char     sint8;
 /** Strings that should be manipulated through add_string() and free_string(). */
 typedef const char *sstring;
 
-typedef unsigned short Fontindex;
-
 #ifdef WIN32
 /* Python plugin stuff defines SIZEOF_LONG_LONG as 8, and besides __int64 is a 64b type on MSVC...
  * So let's force the typedef */
@@ -249,7 +247,7 @@ EXTERN int num_animations, animations_allocated, bmaps_checksum;
 /* Rotate right from bsd sum. This is used in various places for checksumming */
 #define ROTATE_RIGHT(c) if ((c)&01) (c) = ((c)>>1)+0x80000000; else (c) >>= 1;
 
-#define SET_ANIMATION(ob, newanim) { if (ob->temp_animation_id) { ob->face = &new_faces[animations[ob->temp_animation_id].faces[newanim]]; } else { ob->face = &new_faces[animations[ob->animation_id].faces[newanim]]; } }
+#define SET_ANIMATION(ob, newanim) { if (ob->temp_animation_id) { ob->face = animations[ob->temp_animation_id].faces[newanim]; } else { ob->face = animations[ob->animation_id].faces[newanim]; } }
 #define GET_ANIMATION(ob, anim) (ob->temp_animation_id ? animations[ob->temp_animation_id].faces[anim] : animations[ob->animation_id].faces[anim])
 #define GET_ANIM_ID(ob) (ob->temp_animation_id ? ob->temp_animation_id : ob->animation_id)
 /* NUM_ANIMATIONS returns the number of animations allocated.  The last
@@ -284,15 +282,20 @@ extern socket_struct *init_sockets;
  */
 #define object_decrease_nrof_by_one(xyz) object_decrease_nrof(xyz, 1)
 
-/* FREE_AND_CLEAR frees the pointer and then sets it to NULL.
+/**
+ * Free the pointer and then set it to NULL.
  * This is generally done as a safety, and having this macro
  * makes the code a bit cleaner when doing so.
  */
 #define FREE_AND_CLEAR(xyz) { free(xyz); xyz = NULL; }
+/**
+ * Release the shared string, and set it to NULL.
+ * Must not be NULL.
+ */
 #define FREE_AND_CLEAR_STR(xyz) { free_string(xyz); xyz = NULL; }
 
-/* FREE_AND_COPY is for the shared string - it is handy enough
- * to use all over the place.
+/**
+ * Release the shared string if not NULL, and make it a reference to nv.
  */
 #define FREE_AND_COPY(sv, nv) { if (sv) free_string(sv); sv = add_string(nv); }
 
@@ -414,8 +417,15 @@ typedef struct Settings {
                                           * than the price of converted items. */
     int     log_timestamp;            /**< If set, log will comport a timestamp. */
     char    *log_timestamp_format;    /**< Format for timestap, if log_timestamp is set. */
+    uint8   starting_stat_min;        /**< Minimum value of a starting stat */
+    uint8   starting_stat_max;        /**< Maximum value of a starting stat */
+    uint8   starting_stat_points;     /**< How many stat points character starts with */
+    uint8   roll_stat_points;         /**< How many stat points legacy (rolled) chars start with */
 } Settings;
 
+/**
+ * Server settings.
+ */
 extern Settings settings;
 
 /**
@@ -434,6 +444,9 @@ typedef struct Statistics {
     uint64 spell_suppressions;  /**< Number of times ok_to_put_more() returned FALSE*/
 } Statistics;
 
+/**
+ * Merged spell statistics.
+ */
 extern Statistics statistics;
 
 /* 0.94.1 - change to GETTIMEOFDAY macro - SNI systems only one one option.
