@@ -26,6 +26,16 @@ extern object *create_archetype(const char *name);
 extern archetype *try_find_archetype(const char *name);
 extern archetype *find_archetype(const char *name);
 extern object *object_create_arch(archetype *at);
+extern int is_type_valid(uint8 type);
+/* artifact.c */
+extern void free_all_artifacts(void);
+extern void generate_artifact(object *op, int difficulty);
+extern void give_artifact_abilities(object *op, const object *artifact);
+extern int legal_artifact_combination(const object *op, const artifact *art);
+extern void add_abilities(object *op, const object *change);
+extern void init_artifacts(void);
+extern const artifactlist *find_artifactlist(int type);
+extern void dump_artifacts(void);
 /* button.c */
 extern void trigger_connected(objectlink *ol, object *cause, const int state);
 extern void push_button(object *op);
@@ -63,9 +73,9 @@ extern void fatal(int err);
 extern void init_gods(void);
 extern godlink *get_rand_god(void);
 extern const object *pntr_to_god_obj(godlink *godlnk);
+extern int describe_god(const object *god, int what, StringBuffer *buf, int maxlen);
 extern void free_all_god(void);
 extern void dump_gods(void);
-extern int describe_god(const object *god, int what, StringBuffer *buf, int maxlen);
 /* info.c */
 extern void dump_abilities(void);
 extern void print_monsters(void);
@@ -88,20 +98,20 @@ extern void init_objects(void);
 extern void write_todclock(void);
 /* item.c */
 extern int get_power_from_ench(int ench);
-extern int calc_item_power(const object *op, int flag);
+extern int calc_item_power(const object *op);
 extern const typedata *get_typedata(int itemtype);
 extern const typedata *get_typedata_by_name(const char *name);
-extern void describe_resistance(const object *op, int newline, char *buf, size_t size);
+extern StringBuffer *describe_resistance(const object *op, int newline, StringBuffer *buf);
 extern void query_weight(const object *op, char *buf, size_t size);
 extern void get_levelnumber(int i, char *buf, size_t size);
 extern void query_short_name(const object *op, char *buf, size_t size);
 extern void query_name(const object *op, char *buf, size_t size);
 extern void query_base_name(const object *op, int plural, char *buf, size_t size);
-extern void describe_monster(const object *op, char *retbuf, size_t size);
-extern void describe_item(const object *op, const object *owner, char *retbuf, size_t size);
+extern StringBuffer *describe_monster(const object *op, StringBuffer *buf);
+extern StringBuffer *describe_item(const object *op, const object *owner, StringBuffer *buf);
 extern int is_magical(const object *op);
 extern int need_identify(const object *op);
-extern void identify(object *op);
+object *identify(object *op);
 /* languages.c */
 extern int get_language(object *op);
 extern const char *i18n_translate(int language, int id);
@@ -190,6 +200,7 @@ extern int get_rangevector(object *op1, const object *op2, rv_vector *retval, in
 extern int get_rangevector_from_mapcoord(const mapstruct *m, int x, int y, const object *op2, rv_vector *retval, int flags);
 extern int on_same_map(const object *op1, const object *op2);
 extern object *map_find_by_flag(mapstruct *map, int x, int y, int flag);
+extern void map_remove_unique_files(const mapstruct *map);
 /* ob_methods.c */
 extern method_ret ob_apply(object *op, object *applier, int aflags);
 extern method_ret ob_process(object *op);
@@ -230,7 +241,7 @@ extern void object_update_speed(object *op);
 extern void object_remove_from_active_list(object *op);
 extern void object_update(object *op, int action);
 extern void object_free_drop_inventory(object *ob);
-extern void object_free2(object *ob, int free_inventory);
+extern void object_free2(object *ob, int flags);
 extern int object_count_free(void);
 extern int object_count_used(void);
 extern int object_count_active(void);
@@ -320,15 +331,17 @@ extern void player_set_own_title(struct pl *pl, const char *title);
 /* re-cmp.c */
 extern const char *re_cmp(const char *str, const char *regexp);
 /* readable.c */
+extern int nstrtok(const char *buf1, const char *buf2);
+extern char *strtoktolin(const char *buf1, const char *buf2, char *retbuf, size_t size);
 extern int book_overflow(const char *buf1, const char *buf2, size_t booksize);
 extern void init_readable(void);
 extern object *get_random_mon(int level);
+extern char *artifact_msg(int level, char *retbuf, size_t booksize);
+extern char *spellpath_msg(int level, char *retbuf, size_t booksize);
 extern void tailor_readable_ob(object *book, int msg_type);
 extern void free_all_readable(void);
 extern void write_book_archive(void);
 extern const readable_message_type *get_readable_message_type(object *readable);
-extern char *strtoktolin(const char *buf1, const char *buf2, char *retbuf, size_t size);
-extern int nstrtok(const char *buf1, const char *buf2);
 /* recipe.c */
 extern recipelist *get_formulalist(int i);
 extern void init_formulae(void);
@@ -336,7 +349,7 @@ extern void dump_alchemy(void);
 extern archetype *find_treasure_by_name(const treasure *t, const char *name, int depth);
 extern void dump_alchemy_costs(void);
 extern int strtoint(const char *buf);
-extern artifact *locate_recipe_artifact(const recipe *rp, size_t idx);
+extern const artifact *locate_recipe_artifact(const recipe *rp, size_t idx);
 extern recipe *get_random_recipe(recipelist *rpl);
 extern void free_all_recipes(void);
 extern recipe *find_recipe_for_tool(const char *tool, recipe *from);
@@ -368,6 +381,7 @@ extern sstring stringbuffer_finish_shared(StringBuffer *sb);
 extern void stringbuffer_append_string(StringBuffer *sb, const char *str);
 extern void stringbuffer_append_printf(StringBuffer *sb, const char *format, ...);
 extern void stringbuffer_append_stringbuffer(StringBuffer *sb, const StringBuffer *sb2);
+extern size_t stringbuffer_length(StringBuffer *sb);
 /* time.c */
 extern const char *get_periodofday(const int index);
 extern const char *get_month_name(const int index);
@@ -388,14 +402,7 @@ extern void create_treasure(treasurelist *t, object *op, int flag, int difficult
 extern object *generate_treasure(treasurelist *t, int difficulty);
 extern void set_abs_magic(object *op, int magic);
 extern void fix_generated_item(object *op, object *creator, int difficulty, int max_magic, int flags);
-extern artifactlist *find_artifactlist(int type);
-extern void dump_artifacts(void);
 extern void dump_monster_treasure(const char *name);
-extern void init_artifacts(void);
-extern void add_abilities(object *op, object *change);
-extern int legal_artifact_combination(object *op, artifact *art);
-extern void give_artifact_abilities(object *op, object *artifact);
-extern void generate_artifact(object *op, int difficulty);
 extern void free_all_treasures(void);
 /* utils.c */
 extern int random_roll(int min, int max, const object *op, int goodbad);
@@ -406,7 +413,6 @@ extern void decay_objects(mapstruct *m);
 extern materialtype_t *name_to_material(const char *name);
 extern void transmute_materialname(object *op, const object *change);
 extern void set_materialname(object *op, int difficulty, materialtype_t *nmt);
-extern void strip_media_tag(char *message);
 extern const char *strrstr(const char *haystack, const char *needle);
 extern void strip_endline(char *buf);
 extern void replace(const char *src, const char *key, const char *replacement, char *result, size_t resultsize);
@@ -416,6 +422,8 @@ extern int get_randomized_dir(int dir);
 extern int adjust_dir(int dir, int destination_dir);
 extern void replace_unprintable_chars(char *buf);
 extern size_t split_string(char *str, char *array[], size_t array_size, char sep);
+extern StringBuffer *describe_spellpath_attenuation(const char *path, int value, StringBuffer *buf);
+extern StringBuffer *describe_attacktype(const char *attack, int value, StringBuffer *buf);
 /* loader.c */
 extern int lex_load(int *depth, object **items, int maxdepth, int map_flags, int linemode);
 extern void yyrestart(FILE *input_file);
@@ -433,7 +441,7 @@ extern void yyset_debug(int bdebug);
 extern int yylex_destroy(void);
 extern void yyfree(void *ptr);
 extern int load_object(FILE *fp, object *op, int bufstate, int map_flags);
-extern int set_variable(object *op, char *buf);
+extern int set_variable(object *op, const char *buf);
 extern void init_vars(void);
 extern void free_loader(void);
 extern void get_ob_diff(StringBuffer *sb, const object *op, const object *op2);
