@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: object.h 12734 2010-03-25 22:23:45Z ryo_saeba $";
+ *   "$Id: object.h 14031 2010-10-24 13:46:33Z ryo_saeba $";
  */
 
 /*
@@ -182,7 +182,7 @@ typedef struct obj {
     float       speed_left;     /**< How much speed is left to spend this round */
     float       weapon_speed;   /**< The overall speed of this object */
     float       weapon_speed_left; /**< How much speed is left to spend this round */
-    New_Face    *face;          /**< Face with colors */
+    const New_Face    *face;    /**< Face with colors */
     uint32      nrof;           /**< How many of the objects */
     sint8       direction;      /**< Means the object is moving that way. */
     sint8       facing;         /**< Object is oriented/facing that way. */
