@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h 12801 2010-04-03 18:06:12Z akirschbaum $";
+ *   "$Id: player.h 14248 2011-01-24 06:00:53Z mwedel $";
  */
 
 /*
@@ -230,4 +230,25 @@ typedef struct pl {
     uint32      ticks_played;        /**< How many ticks this player has played */
 } player;
 
+/**
+ * @defgroup FIND_PLAYER_xxx Flags to search players through find_player().
+ * Those flag control what player should be returned.
+ */
+/*@{*/
+
+#define FIND_PLAYER_PARTIAL_NAME    0x1
+#define FIND_PLAYER_NO_HIDDEN_DM    0x2
+
+/*@}*/
+
+/**
+ * @defgroup ADD_PLAYER_xxx Flags - flags passed to add_player()
+ * to control behavior
+ */
+/*@{*/
+#define ADD_PLAYER_NEW              0x1 /** Name/password provided, so skip to roll stats */
+#define ADD_PLAYER_NO_MAP           0x2 /** Do not set the first map */
+#define ADD_PLAYER_NO_STATS_ROLL    0x4 /** Stats provided from client */
+/*@}*/
+
 #endif /* PLAYER_H */
