@@ -7,11 +7,11 @@ void send_image_info(socket_struct *ns, char *params);
 void send_image_sums(socket_struct *ns, char *params);
 /* info.c */
 void print_ext_msg(socket_struct *ns, int color, uint8 type, uint8 subtype, const char *message);
-void draw_ext_info(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *message, const char *oldmessage);
-void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *new_format, const char *old_format, ...);
-void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, const char *str1, const char *str2);
-void ext_info_map_except(int color, const mapstruct *map, const object *op, uint8 type, uint8 subtype, const char *str1, const char *str2);
-void ext_info_map_except2(int color, const mapstruct *map, const object *op1, const object *op2, int type, int subtype, const char *str1, const char *str2);
+void draw_ext_info(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *message);
+void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *format, ...);
+void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, const char *str1);
+void ext_info_map_except(int color, const mapstruct *map, const object *op, uint8 type, uint8 subtype, const char *str1);
+void ext_info_map_except2(int color, const mapstruct *map, const object *op1, const object *op2, int type, int subtype, const char *str1);
 void rangetostring(const object *pl, char *obuf, size_t len);
 void set_title(const object *pl, char *buf, size_t len);
 void magic_mapping_mark(object *pl, char *map_mark, int strength);
@@ -54,6 +54,7 @@ size_t SockList_Avail(const SockList *sl);
 void SockList_AddString(SockList *sl, const char *data);
 void SockList_AddData(SockList *sl, const void *data, size_t len);
 void SockList_AddLen8Data(SockList *sl, const void *data, size_t len);
+void SockList_AddLen16Data(SockList *sl, const void *data, size_t len);
 void SockList_AddPrintf(SockList *sl, const char *format, ...);
 void SockList_AddStringBuffer(SockList *sl, StringBuffer *sb);
 void SockList_NullTerminate(SockList *sl);
@@ -108,6 +109,8 @@ void account_add_player_cmd(char *buf, int len, socket_struct *ns);
 void account_play_cmd(char *buf, int len, socket_struct *ns);
 void send_account_players(socket_struct *ns);
 void create_player_cmd(char *buf, int len, socket_struct *ns);
+/* requestinfo.c */
+void send_new_char_info(socket_struct *ns);
 /* sounds.c */
 void play_sound_player_only(player *pl, sint8 sound_type, object *emitter, int dir, const char *action);
 void play_sound_map(sint8 sound_type, object *emitter, int dir, const char *action);
