@@ -1,28 +1,30 @@
 /* account.c */
+void clear_accounts(void);
 void account_load_entries(void);
 void accounts_save(void);
-const char *account_exists(char *account_name);
-int account_check_name_password(char *account_name, char *account_password);
+const char *account_exists(const char *account_name);
+int account_check_name_password(const char *account_name, const char *account_password);
 int account_check_string(const char *str);
-int account_add_account(char *account_name, char *account_password);
-int account_add_player_to_account(char *account_name, char *player_name);
+int account_add_account(const char *account_name, const char *account_password);
+int account_add_player_to_account(const char *account_name, const char *player_name);
 int account_remove_player_from_account(const char *account_name, const char *player_name);
-char **account_get_players_for_account(char *account_name);
+char **account_get_players_for_account(const char *account_name);
 const char *account_get_account_for_char(const char *charname);
-int account_is_logged_in(const char *name);
-socket_struct *account_get_logged_in_init_socket(const char *name);
 player *account_get_logged_in_player(const char *name);
+socket_struct *account_get_logged_in_init_socket(const char *name);
+int account_is_logged_in(const char *name);
 /* account_char.c */
 Account_Char *account_char_load(const char *account_name);
 void account_char_save(const char *account, Account_Char *chars);
 Account_Char *account_char_add(Account_Char *chars, player *pl);
 Account_Char *account_char_remove(Account_Char *chars, const char *pl_name);
 void account_char_free(Account_Char *chars);
+void account_char_init(void);
 /* alchemy.c */
 int use_alchemy(object *op);
 /* apply.c */
 int transport_can_hold(const object *transport, const object *op, int nrof);
-int should_director_abort(object *op, object *victim);
+int should_director_abort(const object *op, const object *victim);
 void apply_handle_yield(object *tmp);
 int set_object_face_main(object *op);
 int apply_container(object *op, object *sack);
@@ -37,7 +39,7 @@ int apply_special(object *who, object *op, int aflags);
 int apply_auto(object *op);
 void apply_auto_fix(mapstruct *m);
 void scroll_failure(object *op, int failure, int power);
-void apply_changes_to_player(object *pl, object *change);
+void apply_changes_to_player(object *pl, object *change, int limit_stats);
 void legacy_apply_container(object *op, object *sack);
 /* attack.c */
 void save_throw_object(object *op, uint32 type, object *originator);
@@ -55,240 +57,236 @@ int checkbanned(const char *login, const char *host);
 void apply_builder_remove(object *pl, int dir);
 void apply_map_builder(object *pl, int dir);
 /* c_chat.c */
-int command_say(object *op, char *params);
-int command_me(object *op, char *params);
-int command_cointoss(object *op, char *params);
-int command_orcknuckle(object *op, char *params);
-int command_shout(object *op, char *params);
-int command_chat(object *op, char *params);
-int command_tell(object *op, char *params);
-int command_dmtell(object *op, char *params);
-int command_reply(object *op, char *params);
-int command_nod(object *op, char *params);
-int command_dance(object *op, char *params);
-int command_kiss(object *op, char *params);
-int command_bounce(object *op, char *params);
-int command_smile(object *op, char *params);
-int command_cackle(object *op, char *params);
-int command_laugh(object *op, char *params);
-int command_giggle(object *op, char *params);
-int command_shake(object *op, char *params);
-int command_puke(object *op, char *params);
-int command_growl(object *op, char *params);
-int command_scream(object *op, char *params);
-int command_sigh(object *op, char *params);
-int command_sulk(object *op, char *params);
-int command_hug(object *op, char *params);
-int command_cry(object *op, char *params);
-int command_poke(object *op, char *params);
-int command_accuse(object *op, char *params);
-int command_grin(object *op, char *params);
-int command_bow(object *op, char *params);
-int command_clap(object *op, char *params);
-int command_blush(object *op, char *params);
-int command_burp(object *op, char *params);
-int command_chuckle(object *op, char *params);
-int command_cough(object *op, char *params);
-int command_flip(object *op, char *params);
-int command_frown(object *op, char *params);
-int command_gasp(object *op, char *params);
-int command_glare(object *op, char *params);
-int command_groan(object *op, char *params);
-int command_hiccup(object *op, char *params);
-int command_lick(object *op, char *params);
-int command_pout(object *op, char *params);
-int command_shiver(object *op, char *params);
-int command_shrug(object *op, char *params);
-int command_slap(object *op, char *params);
-int command_smirk(object *op, char *params);
-int command_snap(object *op, char *params);
-int command_sneeze(object *op, char *params);
-int command_snicker(object *op, char *params);
-int command_sniff(object *op, char *params);
-int command_snore(object *op, char *params);
-int command_spit(object *op, char *params);
-int command_strut(object *op, char *params);
-int command_thank(object *op, char *params);
-int command_twiddle(object *op, char *params);
-int command_wave(object *op, char *params);
-int command_whistle(object *op, char *params);
-int command_wink(object *op, char *params);
-int command_yawn(object *op, char *params);
-int command_beg(object *op, char *params);
-int command_bleed(object *op, char *params);
-int command_cringe(object *op, char *params);
-int command_think(object *op, char *params);
+void command_say(object *op, const char *params);
+void command_me(object *op, const char *params);
+void command_cointoss(object *op, const char *params);
+void command_orcknuckle(object *op, const char *params);
+void command_shout(object *op, const char *params);
+void command_chat(object *op, const char *params);
+void command_tell(object *op, const char *params);
+void command_dmtell(object *op, const char *params);
+void command_reply(object *op, const char *params);
+void command_nod(object *op, const char *params);
+void command_dance(object *op, const char *params);
+void command_kiss(object *op, const char *params);
+void command_bounce(object *op, const char *params);
+void command_smile(object *op, const char *params);
+void command_cackle(object *op, const char *params);
+void command_laugh(object *op, const char *params);
+void command_giggle(object *op, const char *params);
+void command_shake(object *op, const char *params);
+void command_puke(object *op, const char *params);
+void command_growl(object *op, const char *params);
+void command_scream(object *op, const char *params);
+void command_sigh(object *op, const char *params);
+void command_sulk(object *op, const char *params);
+void command_hug(object *op, const char *params);
+void command_cry(object *op, const char *params);
+void command_poke(object *op, const char *params);
+void command_accuse(object *op, const char *params);
+void command_grin(object *op, const char *params);
+void command_bow(object *op, const char *params);
+void command_clap(object *op, const char *params);
+void command_blush(object *op, const char *params);
+void command_burp(object *op, const char *params);
+void command_chuckle(object *op, const char *params);
+void command_cough(object *op, const char *params);
+void command_flip(object *op, const char *params);
+void command_frown(object *op, const char *params);
+void command_gasp(object *op, const char *params);
+void command_glare(object *op, const char *params);
+void command_groan(object *op, const char *params);
+void command_hiccup(object *op, const char *params);
+void command_lick(object *op, const char *params);
+void command_pout(object *op, const char *params);
+void command_shiver(object *op, const char *params);
+void command_shrug(object *op, const char *params);
+void command_slap(object *op, const char *params);
+void command_smirk(object *op, const char *params);
+void command_snap(object *op, const char *params);
+void command_sneeze(object *op, const char *params);
+void command_snicker(object *op, const char *params);
+void command_sniff(object *op, const char *params);
+void command_snore(object *op, const char *params);
+void command_spit(object *op, const char *params);
+void command_strut(object *op, const char *params);
+void command_thank(object *op, const char *params);
+void command_twiddle(object *op, const char *params);
+void command_wave(object *op, const char *params);
+void command_whistle(object *op, const char *params);
+void command_wink(object *op, const char *params);
+void command_yawn(object *op, const char *params);
+void command_beg(object *op, const char *params);
+void command_bleed(object *op, const char *params);
+void command_cringe(object *op, const char *params);
+void command_think(object *op, const char *params);
 /* c_misc.c */
-void map_info(object *op, char *search);
-int command_language(object *op, char *params);
-int command_body(object *op, char *params);
-int command_motd(object *op, char *params);
-int command_rules(object *op, char *params);
-int command_news(object *op, char *params);
+void map_info(object *op, const char *search);
+void command_language(object *op, const char *params);
+void command_body(object *op, const char *params);
+void command_motd(object *op, const char *params);
+void command_rules(object *op, const char *params);
+void command_news(object *op, const char *params);
 void malloc_info(object *op);
 void current_region_info(object *op);
 void current_map_info(object *op);
-int command_whereabouts(object *op, char *params);
+void command_whereabouts(object *op, const char *params);
 void list_players(object *op, region *reg, partylist *party);
-int command_who(object *op, char *params);
+void command_who(object *op, const char *params);
 void display_who_entry(object *op, player *pl, const char *format);
 void get_who_escape_code_value(char *return_val, int size, const char letter, player *pl);
-int command_afk(object *op, char *params);
-int command_malloc(object *op, char *params);
-int command_mapinfo(object *op, char *params);
-int command_whereami(object *op, char *params);
-int command_maps(object *op, char *params);
-int command_strings(object *op, char *params);
-int command_time(object *op, char *params);
-int command_archs(object *op, char *params);
-int command_hiscore(object *op, char *params);
-int command_debug(object *op, char *params);
-int command_dumpbelow(object *op, char *params);
-int command_wizpass(object *op, char *params);
-int command_wizcast(object *op, char *params);
-int command_dumpallobjects(object *op, char *params);
-int command_dumpfriendlyobjects(object *op, char *params);
-int command_dumpallarchetypes(object *op, char *params);
-int command_ssdumptable(object *op, char *params);
-int command_dumpmap(object *op, char *params);
-int command_dumpallmaps(object *op, char *params);
-int command_printlos(object *op, char *params);
-int command_version(object *op, char *params);
-int command_listen(object *op, char *params);
-int command_statistics(object *pl, char *params);
-int command_fix_me(object *op, char *params);
-int command_players(object *op, char *params);
-int command_applymode(object *op, char *params);
-int command_bowmode(object *op, char *params);
-int command_unarmed_skill(object *op, char *params);
-int command_petmode(object *op, char *params);
-int command_showpets(object *op, char *params);
-int command_usekeys(object *op, char *params);
-int command_resistances(object *op, char *params);
-int command_help(object *op, char *params);
+void command_afk(object *op, const char *params);
+void command_malloc(object *op, const char *params);
+void command_mapinfo(object *op, const char *params);
+void command_whereami(object *op, const char *params);
+void command_maps(object *op, const char *params);
+void command_strings(object *op, const char *params);
+void command_time(object *op, const char *params);
+void command_archs(object *op, const char *params);
+void command_hiscore(object *op, const char *params);
+void command_debug(object *op, const char *params);
+void command_wizpass(object *op, const char *params);
+void command_wizcast(object *op, const char *params);
+void command_dumpallobjects(object *op, const char *params);
+void command_dumpfriendlyobjects(object *op, const char *params);
+void command_dumpallarchetypes(object *op, const char *params);
+void command_ssdumptable(object *op, const char *params);
+void command_dumpmap(object *op, const char *params);
+void command_dumpallmaps(object *op, const char *params);
+void command_printlos(object *op, const char *params);
+void command_version(object *op, const char *params);
+void command_listen(object *op, const char *params);
+void command_statistics(object *pl, const char *params);
+void command_fix_me(object *op, const char *params);
+void command_players(object *op, const char *params);
+void command_applymode(object *op, const char *params);
+void command_bowmode(object *op, const char *params);
+void command_unarmed_skill(object *op, const char *params);
+void command_petmode(object *op, const char *params);
+void command_showpets(object *op, const char *params);
+void command_usekeys(object *op, const char *params);
+void command_resistances(object *op, const char *params);
+void command_help(object *op, const char *params);
 int onoff_value(const char *line);
-int command_quit(object *op, char *params);
-int command_sound(object *op, char *params);
+void command_quit(object *op, const char *params);
+void command_sound(object *op, const char *params);
 void receive_player_name(object *op);
 void receive_player_password(object *op);
-int command_title(object *op, char *params);
-int command_save(object *op, char *params);
-int command_peaceful(object *op, char *params);
-int command_wimpy(object *op, char *params);
-int command_brace(object *op, char *params);
-int command_kill_pets(object *op, char *params);
-int command_passwd(object *pl, char *params);
-int do_harvest(object *pl, int dir, object *skill);
+void command_title(object *op, const char *params);
+void command_save(object *op, const char *params);
+void command_peaceful(object *op, const char *params);
+void command_wimpy(object *op, const char *params);
+void command_brace(object *op, const char *params);
+void command_kill_pets(object *op, const char *params);
+void command_passwd(object *pl, const char *params);
+void do_harvest(object *pl, int dir, object *skill);
 /* c_move.c */
-int command_east(object *op, char *params);
-int command_north(object *op, char *params);
-int command_northeast(object *op, char *params);
-int command_northwest(object *op, char *params);
-int command_south(object *op, char *params);
-int command_southeast(object *op, char *params);
-int command_southwest(object *op, char *params);
-int command_west(object *op, char *params);
-int command_stay(object *op, char *params);
+void command_east(object *op, const char *params);
+void command_north(object *op, const char *params);
+void command_northeast(object *op, const char *params);
+void command_northwest(object *op, const char *params);
+void command_south(object *op, const char *params);
+void command_southeast(object *op, const char *params);
+void command_southwest(object *op, const char *params);
+void command_west(object *op, const char *params);
+void command_stay(object *op, const char *params);
 /* c_new.c */
-int execute_newserver_command(object *pl, char *command);
-int command_run(object *op, char *params);
-int command_run_stop(object *op, char *params);
-int command_fire(object *op, char *params);
-int command_fire_stop(object *op, char *params);
+void execute_newserver_command(object *pl, char *command);
+void command_run(object *op, const char *params);
+void command_run_stop(object *op, const char *params);
+void command_fire(object *op, const char *params);
+void command_fire_stop(object *op, const char *params);
 /* c_object.c */
-int command_uskill(object *pl, char *params);
-int command_rskill(object *pl, char *params);
-int command_search(object *op, char *params);
-int command_disarm(object *op, char *params);
-int command_throw(object *op, char *params);
-int command_apply(object *op, char *params);
+void command_uskill(object *pl, const char *params);
+void command_rskill(object *pl, const char *params);
+void command_search(object *op, const char *params);
+void command_disarm(object *op, const char *params);
+void command_throw(object *op, const char *params);
+void command_apply(object *op, const char *params);
 int sack_can_hold(const object *pl, const object *sack, const object *op, uint32 nrof);
 void pick_up(object *op, object *alt);
-int command_take(object *op, char *params);
+void command_take(object *op, const char *params);
 void put_object_in_sack(object *op, object *sack, object *tmp, uint32 nrof);
 object *drop_object(object *op, object *tmp, uint32 nrof);
 void drop(object *op, object *tmp);
-int command_dropall(object *op, char *params);
-int command_drop(object *op, char *params);
-int command_empty(object *op, char *params);
-int command_examine(object *op, char *params);
+void command_dropall(object *op, const char *params);
+void command_drop(object *op, const char *params);
+void command_empty(object *op, const char *params);
+void command_examine(object *op, const char *params);
 object *find_marked_object(object *op);
-int command_mark(object *op, char *params);
+void command_mark(object *op, const char *params);
 void examine_monster(object *op, object *tmp);
 void examine(object *op, object *tmp);
 void inventory(object *op, object *inv);
-int command_pickup(object *op, char *params);
-int command_search_items(object *op, char *params);
-int command_rename_item(object *op, char *params);
-int command_lock_item(object *op, char *params);
-int command_use(object *op, char *params);
+void command_pickup(object *op, const char *params);
+void command_search_items(object *op, const char *params);
+void command_rename_item(object *op, const char *params);
+void command_lock_item(object *op, const char *params);
+void command_use(object *op, const char *params);
 /* c_party.c */
 int confirm_party_password(object *op);
 void receive_party_password(object *op);
-int command_gsay(object *op, char *params);
-int command_party(object *op, char *params);
-int command_party_rejoin(object *op, char *params);
+void command_gsay(object *op, const char *params);
+void command_party(object *op, const char *params);
+void command_party_rejoin(object *op, const char *params);
 /* c_range.c */
-int command_invoke(object *op, char *params);
-int command_cast(object *op, char *params);
-int command_prepare(object *op, char *params);
-int command_cast_spell(object *op, char *params, char command);
+void command_invoke(object *op, const char *params);
+void command_cast(object *op, const char *params);
+void command_prepare(object *op, const char *params);
+void command_cast_spell(object *op, const char *params, char command);
 int legal_range(object *op, int r);
 void change_spell(object *op, char k);
-int command_rotateshoottype(object *op, char *params);
+void command_rotateshoottype(object *op, const char *params);
 /* c_wiz.c */
-int command_loadtest(object *op, char *params);
-void do_wizard_hide(object *op, int silent_dm);
-int command_hide(object *op, char *params);
-int command_setgod(object *op, char *params);
-int command_banish(object *op, char *params);
-int command_kick(object *op, char *params);
-int command_overlay_save(object *op, char *params);
-int command_overlay_reset(object *op, char *params);
-int command_toggle_shout(object *op, char *params);
-int command_shutdown(object *op, char *params);
-int command_goto(object *op, char *params);
-int command_freeze(object *op, char *params);
-int command_arrest(object *op, char *params);
-int command_summon(object *op, char *params);
-int command_teleport(object *op, char *params);
-int command_create(object *op, char *params);
-int command_inventory(object *op, char *params);
-int command_skills(object *op, char *params);
-int command_dump(object *op, char *params);
-int command_mon_aggr(object *op, char *params);
-int command_possess(object *op, char *params);
-int command_patch(object *op, char *params);
-int command_remove(object *op, char *params);
-int command_free(object *op, char *params);
-int command_addexp(object *op, char *params);
-int command_speed(object *op, char *params);
-int command_stats(object *op, char *params);
-int command_abil(object *op, char *params);
-int command_reset(object *op, char *params);
-int command_nowiz(object *op, char *params);
-int do_wizard_dm(object *op, char *params, int silent);
-int command_dm(object *op, char *params);
-int command_invisible(object *op, char *params);
-int command_learn_spell(object *op, char *params);
-int command_learn_special_prayer(object *op, char *params);
-int command_forget_spell(object *op, char *params);
-int command_listplugins(object *op, char *params);
-int command_loadplugin(object *op, char *params);
-int command_unloadplugin(object *op, char *params);
-int command_dmhide(object *op, char *params);
-void dm_stack_pop(player *pl);
-object *dm_stack_peek(player *pl);
-void dm_stack_push(player *pl, tag_t item);
-object *get_dm_object(player *pl, char **params, int *from);
-int command_stack_pop(object *op, char *params);
-int command_stack_push(object *op, char *params);
-int command_stack_list(object *op, char *params);
-int command_stack_clear(object *op, char *params);
-int command_diff(object *op, char *params);
-int command_insert_into(object *op, char *params);
-int command_style_map_info(object *op, char *params);
-int command_follow(object *op, char *params);
+void command_loadtest(object *op, const char *params);
+void command_hide(object *op, const char *params);
+void command_setgod(object *op, const char *params);
+void command_banish(object *op, const char *params);
+void command_kick(object *op, const char *params);
+void command_overlay_save(object *op, const char *params);
+void command_overlay_reset(object *op, const char *params);
+void command_toggle_shout(object *op, const char *params);
+void command_shutdown(object *op, const char *params);
+void command_goto(object *op, const char *params);
+void command_freeze(object *op, const char *params);
+void command_arrest(object *op, const char *params);
+void command_summon(object *op, const char *params);
+void command_teleport(object *op, const char *params);
+void command_create(object *op, const char *params);
+void command_inventory(object *op, const char *params);
+void command_skills(object *op, const char *params);
+void command_dump(object *op, const char *params);
+void command_mon_aggr(object *op, const char *params);
+void command_possess(object *op, const char *params);
+void command_patch(object *op, const char *params);
+void command_remove(object *op, const char *params);
+void command_free(object *op, const char *params);
+void command_addexp(object *op, const char *params);
+void command_speed(object *op, const char *params);
+void command_stats(object *op, const char *params);
+void command_abil(object *op, const char *params);
+void command_reset(object *op, const char *params);
+void command_nowiz(object *op, const char *params);
+void command_dm(object *op, const char *params);
+void command_invisible(object *op, const char *params);
+void command_learn_spell(object *op, const char *params);
+void command_learn_special_prayer(object *op, const char *params);
+void command_forget_spell(object *op, const char *params);
+void command_listplugins(object *op, const char *params);
+void command_loadplugin(object *op, const char *params);
+void command_unloadplugin(object *op, const char *params);
+void command_dmhide(object *op, const char *params);
+void command_stack_pop(object *op, const char *params);
+void command_stack_push(object *op, const char *params);
+void command_stack_list(object *op, const char *params);
+void command_stack_clear(object *op, const char *params);
+void command_diff(object *op, const char *params);
+void command_insert_into(object *op, const char *params);
+void command_style_map_info(object *op, const char *params);
+void command_follow(object *op, const char *params);
+void command_purge_quest(object *op, const char *param);
+void command_purge_quest_definitions(object *op, const char *param);
+void command_dumpbelow(object *op, const char *params);
 /* commands.c */
 void init_commands(void);
 /* daemon.c */
@@ -316,8 +314,9 @@ void free_server(void);
 racelink *find_racelink(const char *name);
 /* knowledge.c */
 void knowledge_read(player *pl, object *book);
-int command_knowledge(object *pl, char *params);
+void command_knowledge(object *pl, const char *params);
 void free_knowledge(void);
+int knowledge_player_knows(const player *pl, const char *knowledge);
 /* login.c */
 void emergency_save(int flag);
 void delete_character(const char *name);
@@ -336,6 +335,7 @@ void monster_check_apply(object *mon, object *item);
 void monster_npc_call_help(object *op);
 void monster_check_earthwalls(object *op, mapstruct *m, int x, int y);
 void monster_check_doors(object *op, mapstruct *m, int x, int y);
+void monster_do_say(const mapstruct *map, const char *message);
 void monster_communicate(object *op, const char *txt);
 void monster_npc_say(object *npc, const char *cp);
 object *monster_find_throw_ob(object *op);
@@ -380,15 +380,16 @@ int pets_summon_golem(object *op, object *caster, int dir, object *spob);
 int pets_summon_object(object *op, object *caster, object *spell_ob, int dir, const char *stringarg);
 int pets_should_arena_attack(object *pet, object *owner, object *target);
 /* player.c */
-void set_first_map(object *op);
-player *get_player(player *p);
+player *find_player_options(const char *plname, int options, const mapstruct *map);
 player *find_player(const char *plname);
 player *find_player_partial_name(const char *plname);
-player* find_player_socket(const socket_struct *ns);
+player *find_player_socket(const socket_struct *ns);
 void display_motd(const object *op);
 void send_rules(const object *op);
 void send_news(const object *op);
 int playername_ok(const char *cp);
+player *get_player(player *p);
+void set_first_map(object *op);
 player *add_player(socket_struct *ns, int new_char);
 object *get_nearest_player(object *mon);
 int path_to_player(object *mon, object *pl, unsigned mindiff);
@@ -427,120 +428,30 @@ int player_can_view(object *pl, object *op);
 int op_on_battleground(object *op, int *x, int *y, archetype **trophy);
 void dragon_ability_gain(object *who, int atnr, int level);
 void player_unready_range_ob(player *pl, object *ob);
+void player_set_state(player *pl, uint8 state);
+int apply_race_and_class(object *op, archetype *race, archetype *class);
 /* plugins.c */
 int user_event(object *op, object *activator, object *third, const char *message, int fix);
 int execute_event(object *op, int eventcode, object *activator, object *third, const char *message, int fix);
+int plugin_event_say(object *npc, talk_info *talk);
 int execute_global_event(int eventcode, ...);
 int plugins_init_plugin(const char *libfile);
-void *cfapi_get_hooks(int *type, ...);
+void cfapi_get_hooks(int *type, ...);
 int plugins_remove_plugin(const char *id);
 crossfire_plugin *plugins_find_plugin(const char *id);
 void plugins_display_list(object *op);
-void *cfapi_system_find_animation(int *type, ...);
-void *cfapi_system_find_face(int *type, ...);
-void *cfapi_system_strdup_local(int *type, ...);
-void *cfapi_system_register_global_event(int *type, ...);
-void *cfapi_system_unregister_global_event(int *type, ...);
-void *cfapi_system_add_string(int *type, ...);
-void *cfapi_system_remove_string(int *type, ...);
-void *cfapi_system_find_string(int *type, ...);
-void *cfapi_system_check_path(int *type, ...);
-void *cfapi_system_re_cmp(int *type, ...);
-void *cfapi_system_directory(int *type, ...);
-void *cfapi_get_time(int *type, ...);
-void *cfapi_get_season_name(int *type, ...);
-void *cfapi_get_weekday_name(int *type, ...);
-void *cfapi_get_month_name(int *type, ...);
-void *cfapi_get_periodofday_name(int *type, ...);
-void *cfapi_timer_create(int *type, ...);
-void *cfapi_timer_destroy(int *type, ...);
-void *cfapi_log(int *type, ...);
-void *cfapi_map_get_map(int *type, ...);
-void *cfapi_map_has_been_loaded(int *type, ...);
-void *cfapi_map_create_path(int *type, ...);
-void *cfapi_map_get_map_property(int *type, ...);
-void *cfapi_map_set_map_property(int *type, ...);
-void *cfapi_map_out_of_map(int *type, ...);
-void *cfapi_map_update_position(int *type, ...);
-void *cfapi_map_delete_map(int *type, ...);
-void *cfapi_map_message(int *type, ...);
-void *cfapi_map_get_object_at(int *type, ...);
-void *cfapi_map_find_by_archetype_name(int *type, ...);
-void *cfapi_map_change_light(int *type, ...);
-void *cfapi_object_move(int *type, ...);
-void *cfapi_object_get_key(int *type, ...);
-void *cfapi_object_set_key(int *type, ...);
-void *cfapi_object_get_property(int *type, ...);
-void *cfapi_object_set_property(int *type, ...);
-void *cfapi_object_apply_below(int *type, ...);
-void *cfapi_object_apply(int *type, ...);
-void *cfapi_object_identify(int *type, ...);
-void *cfapi_object_describe(int *type, ...);
-void *cfapi_object_drain(int *type, ...);
-void *cfapi_object_remove_depletion(int *type, ...);
-void *cfapi_object_fix(int *type, ...);
-void *cfapi_object_give_skill(int *type, ...);
-void *cfapi_object_transmute(int *type, ...);
-void *cfapi_object_remove(int *type, ...);
-void *cfapi_object_delete(int *type, ...);
-void *cfapi_object_clone(int *type, ...);
-void *cfapi_object_find(int *type, ...);
-void *cfapi_object_create(int *type, ...);
-void *cfapi_object_insert(int *type, ...);
-void *cfapi_object_split(int *type, ...);
-void *cfapi_object_merge(int *type, ...);
-void *cfapi_object_distance(int *type, ...);
-void *cfapi_object_update(int *type, ...);
-void *cfapi_object_clear(int *type, ...);
-void *cfapi_object_reset(int *type, ...);
-void *cfapi_object_check_inventory(int *type, ...);
-void *cfapi_object_clean_object(int *type, ...);
-void *cfapi_object_on_same_map(int *type, ...);
-void *cfapi_object_spring_trap(int *type, ...);
-void *cfapi_object_check_trigger(int *type, ...);
-void *cfapi_map_trigger_connected(int *type, ...);
-void *cfapi_object_query_cost(int *type, ...);
-void *cfapi_object_query_cost_string(int *type, ...);
-void *cfapi_object_query_money(int *type, ...);
-void *cfapi_object_cast(int *type, ...);
-void *cfapi_object_learn_spell(int *type, ...);
-void *cfapi_object_forget_spell(int *type, ...);
-void *cfapi_object_check_spell(int *type, ...);
-void *cfapi_object_pay_amount(int *type, ...);
-void *cfapi_object_pay_item(int *type, ...);
-void *cfapi_object_transfer(int *type, ...);
-void *cfapi_object_find_archetype_inside(int *type, ...);
-void *cfapi_object_drop(int *type, ...);
-void *cfapi_object_change_abil(int *type, ...);
-void *cfapi_object_say(int *type, ...);
-void *cfapi_player_find(int *type, ...);
-void *cfapi_player_message(int *type, ...);
-void *cfapi_object_change_exp(int *type, ...);
-void *cfapi_player_can_pay(int *type, ...);
-void *cfapi_object_teleport(int *type, ...);
-void *cfapi_object_pickup(int *type, ...);
-void *cfapi_object_find_by_arch_name(int *type, ...);
-void *cfapi_object_find_by_name(int *type, ...);
-void *cfapi_archetype_get_property(int *type, ...);
-void *cfapi_party_get_property(int *type, ...);
-void *cfapi_region_get_property(int *type, ...);
-void *cfapi_friendlylist_get_next(int *type, ...);
-void *cfapi_set_random_map_variable(int *type, ...);
-void *cfapi_generate_random_map(int *type, ...);
-void *cfapi_object_user_event(int *type, ...);
-void *cfapi_cost_string_from_value(int *type, ...);
-void *cfapi_player_quest(int *type, ...);
-command_array_struct *find_plugin_command(char *cmd, object *op);
+command_array_struct *find_plugin_command(const char *cmd, command_array_struct *command);
 int initPlugins(void);
 void cleanupPlugins(void);
 /* quest.c */
 int quest_get_player_state(player *pl, sstring quest_code);
 void quest_start(player *pl, sstring quest_code, int state);
-void quest_end(player *pl, sstring quest_code);
 void quest_set_player_state(player *pl, sstring quest_code, int state);
 int quest_was_completed(player *pl, sstring quest_code);
-int command_quest(object *op, char *params);
+void command_quest(object *op, const char *params);
+void dump_quests(void);
 void free_quest(void);
+void free_quest_definitions(void);
 /* resurrection.c */
 int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, const char *arg);
 void dead_player(object *op);
@@ -554,7 +465,6 @@ int trap_disarm(object *disarmer, object *trap, int risk, object *skill);
 void trap_adjust(object *trap, int difficulty);
 /* shop.c */
 uint64 query_cost(const object *tmp, object *who, int flag);
-StringBuffer *query_cost_string(const object *tmp, object *who, int flag, StringBuffer *buf);
 uint64 query_money(const object *op);
 int pay_for_amount(uint64 to_pay, object *pl);
 int pay_for_item(object *op, object *pl);
@@ -565,12 +475,16 @@ double shopkeeper_approval(const mapstruct *map, const object *player);
 int describe_shop(const object *op);
 int is_in_shop(object *ob);
 int coords_in_shop(mapstruct *map, int x, int y);
-StringBuffer *cost_string_from_value(uint64 cost, StringBuffer *buf);
+StringBuffer *query_cost_string(const object *tmp, object *who, int flag, StringBuffer *buf);
+StringBuffer *cost_string_from_value(uint64 cost, int largest_coin, StringBuffer *buf);
 /* skills.c */
 int steal(object *op, int dir, object *skill);
 int pick_lock(object *pl, int dir, object *skill);
 int hide(object *op, object *skill);
 int jump(object *pl, int dir, object *skill);
+int detect_curse_on_item(object *pl, object *tmp, object *skill);
+int detect_magic_on_item(object *pl, object *tmp, object *skill);
+int identify_object_with_skill(object *tmp, object *pl, object *skill, int print_on_success);
 int skill_ident(object *pl, object *skill);
 int use_oratory(object *pl, int dir, object *skill);
 int singing(object *pl, int dir, object *skill);
@@ -588,11 +502,11 @@ object *find_skill_by_number(object *who, int skillno);
 int change_skill(object *who, object *new_skill, int flag);
 void clear_skill(object *who);
 int do_skill(object *op, object *part, object *skill, int dir, const char *string);
-sint64 calc_skill_exp(object *who, object *op, object *skill);
+sint64 calc_skill_exp(const object *who, const object *op, const object *skill);
 int learn_skill(object *pl, object *scroll);
 void show_skills(object *op, const char *search);
 int use_skill(object *op, const char *string);
-int skill_attack(object *tmp, object *pl, int dir, const char *string, object *skill);
+void skill_attack(object *tmp, object *pl, int dir, const char *string, object *skill);
 /* spell_attack.c */
 int fire_bolt(object *op, object *caster, int dir, object *spob);
 void explode_bullet(object *op);
@@ -703,8 +617,8 @@ int similar_direction(int a, int b);
 /* server.c */
 void version(object *op);
 void start_info(object *op);
-char *crypt_string(char *str, char *salt);
-int check_password(char *typed, char *crypted);
+char *crypt_string(const char *str, const char *salt);
+int check_password(const char *typed, const char *crypted);
 void enter_player_savebed(object *op);
 void set_map_timeout(mapstruct *oldmap);
 void enter_exit(object *op, object *exit_ob);
