@@ -15,23 +15,33 @@ use FileHandle;
 die("No arch directory - will not rebuild $mode image file") if (! -e "arch");
 
 $archive = 0;
+$src_path = ".";
 $TMPDIR="/tmp";
 
 # What we will call the collection of images.
 $ARCHNAME="crossfire-images";
 $DESTDIR="$TMPDIR/$ARCHNAME";
 
-# Maximum expected file 
+# Maximum expected file
 $MAXFILESIZE=100000;
 
-if ($ARGV[0] eq "-archive") {
-	$archive =1;
-	print "Will generate appropriate files for image archive\n";
-	die("$DESTDIR already exists - remove if you really want to remake the images") if (-d $DESTDIR);
-	die("$0: unable to mkdir $DESTDIR: $1\n") if (!mkdir($DESTDIR, 0755));
+if ($ARGV[0] eq "-src") {
+    $src_path = $ARGV[1];
+    print "Using $ARGV[1] as source directory\n";
+    die("$src_path doesn't exist.") if (! -d $src_path);
+    shift;
+    shift;
 }
 
-open(IMAGEINFO,"image_info") || die("Can't open image_info file: $!\n");
+if ($ARGV[0] eq "-archive") {
+    $archive =1;
+    print "Will generate appropriate files for image archive\n";
+    die("$DESTDIR already exists - remove if you really want to remake the images") if (-d $DESTDIR);
+    die("$0: unable to mkdir $DESTDIR: $1\n") if (!mkdir($DESTDIR, 0755));
+}
+
+open(IMAGEINFO,"$src_path/image_info") || die("Can't open image_info file: $!\n");
+binmode(IMAGEINFO);
 while (<IMAGEINFO>) {
     # Ignore lines that start with comments or just empty lines
     next if /^#/;
@@ -55,10 +65,11 @@ for ($count=0; $count<=$#extension; $count++) {
     $fh = $ESRV[$count];
     open($fh, ">crossfire.$count") ||
 	die("Can't open crossfire.$count for write: $!\n");
-	binmode( $fh );
+    binmode( $fh );
 }
 
 open(BMAPS,"bmaps.paths") || die("Can't open bmaps.paths: $!\n");
+binmode(BMAPS);
 $_ = <BMAPS>;
 while(<BMAPS>) {
     chop;
@@ -72,7 +83,7 @@ while(<BMAPS>) {
     $file1 = $3;
 
     print "$num $file\n" if ($num % 500) == 0 ;
-    # This probably isn't the most efficient way to do this if a 
+    # This probably isn't the most efficient way to do this if a
     # large number of images are added, as we try to open each
     # instance.
     # OTOH, we are doing one directory
@@ -116,7 +127,7 @@ while(<BMAPS>) {
 	    # set 0 should have all the sets
 	    print "Error: Image $filename not found for set 0!\n";
 	}
-	
+
     }
 }
 for ($count=0; $count<=$#extension; $count++) {
@@ -125,7 +136,8 @@ for ($count=0; $count<=$#extension; $count++) {
 close(BMAPS);
 
 if ($archive) {
-    open(OUT,">$DESTDIR/bmaps.client") || die("Can not open $DESTDIR/bmaps.paths\n");
+    open(OUT,">$DESTDIR/bmaps.client") || die("Can not open $DESTDIR/bmaps.client\n");
+	binmode(OUT);
     print OUT sort @csums;
     close(OUT);
     open(OUT,">$DESTDIR/README") || die("Can not open $DESTDIR/README\n");
