@@ -143,7 +143,7 @@ static long int initvisible(const char *name, char *parameters, struct CFmovemen
 
     if (get_boolean(parameters, &result))
         return result;
-    cf_log(llevDebug, "CFAnim: Error in animation - possible values for 'invisible' are 'yes' and 'no'\n");
+    cf_log(llevError, "CFAnim: Error in animation - possible values for 'invisible' are 'yes' and 'no'\n");
     return -1;
 }
 
@@ -159,7 +159,7 @@ static long int initwizard(const char *name, char *parameters, struct CFmovement
 
     if (get_boolean(parameters, &result))
         return result;
-    cf_log(llevDebug, "CFAnim: Error in animation - possible values for 'wizard' are 'yes' and 'no'\n");
+    cf_log(llevError, "CFAnim: Error in animation - possible values for 'wizard' are 'yes' and 'no'\n");
     return -1;
 }
 
@@ -185,7 +185,7 @@ static anim_move_result runsay(struct CFanimation_struct *animation, long int id
         cf_object_say(animation->victim, parameters);
         free(parameters);
     } else
-        cf_log(llevDebug, "CFAnim: Error in animation: nothing to say with say function\n");
+        cf_log(llevError, "CFAnim: Error in animation: nothing to say with say function\n");
     return mr_finished;
 }
 
@@ -283,7 +283,7 @@ static long int initghosted(const char *name, char *parameters, struct CFmovemen
 
     if (get_boolean(parameters, &result))
         return result;
-    cf_log(llevDebug, "CFAnim: Error in animation: possible values for 'ghosted' are 'yes' and 'no'\n");
+    cf_log(llevError, "CFAnim: Error in animation: possible values for 'ghosted' are 'yes' and 'no'\n");
     return -1;
 }
 
@@ -332,7 +332,7 @@ static long int initteleport(const char *name, char *parameters, struct CFmoveme
     move_entity->parameters = NULL;
     cf_log(llevDebug, ".(%s)\n", parameters);
     if (!parameters) {
-        cf_log(llevDebug, "CFAnim: Error - no parameters for teleport\n");
+        cf_log(llevError, "CFAnim: Error - no parameters for teleport\n");
         return 0;
     }
     mapname = strstr(parameters, " ");
@@ -344,7 +344,7 @@ static long int initteleport(const char *name, char *parameters, struct CFmoveme
     mapname++;
     parameters = mapname;
     if (!parameters) {
-        cf_log(llevDebug, "CFAnim: Error - not enough parameters for teleport\n");
+        cf_log(llevError, "CFAnim: Error - not enough parameters for teleport\n");
         return 0;
     }
     cf_log(llevDebug, ".(%s)\n", parameters);
@@ -584,7 +584,7 @@ static CFmovement *parse_animation_block(char *buffer, size_t buffer_size, FILE
         animationhook = get_command(name);
         if (parent->verbose) {
             if (!animationhook)
-                cf_log(llevDebug, "CFAnim: %s - Unknown animation command\n", name);
+                cf_log(llevError, "CFAnim: %s - Unknown animation command\n", name);
             else
                 cf_log(llevDebug, "CFAnim: Parsed %s -> %p\n", name, animationhook);
         }
@@ -671,6 +671,21 @@ static int get_boolean(const char *strg, int *bl) {
     return 1;
 }
 
+/**
+ * Is specified object currently being animated?
+ * @param ob object to search for.
+ * @return 1 if ob is part of animation, 0 else.
+ */
+static int is_animated_object(const object *ob) {
+    CFanimation *current;
+
+    for (current = first_animation; current; current = current->nextanimation)
+        if (current->victim == ob) {
+            return 1;
+        }
+    return 0;
+}
+
 /**
  * Is specified player currently victim of a paralysing animation?
  * @param pl player to search for.
@@ -679,7 +694,7 @@ static int get_boolean(const char *strg, int *bl) {
 static int is_animated_player(object *pl) {
     CFanimation *current;
 
-    for (current = first_animation; current; current++)
+    for (current = first_animation; current; current = current->nextanimation)
         if ((current->victim == pl) && (current->paralyze)) {
             if (current->verbose)
                 cf_log(llevDebug, "CFAnim: Getting a command for a paralyzed player %s.\n", pl->name);
@@ -701,8 +716,10 @@ static CFanimation *create_animation(void) {
         return NULL;
     new->name = NULL;
     new->victim = NULL;
+    new->event = NULL;
     new->nextmovement = NULL;
     new->nextanimation = NULL;
+    new->delete_end = 0;
     for (current = first_animation; (current && current->nextanimation); current = current->nextanimation)
         ;
     if (!current)
@@ -762,6 +779,7 @@ static int start_animation(object *who, object *activator, object *event, const
     object  *victim = NULL;
     int     unique = 0;
     int     always_delete = 0;
+    int     delete_end = 0;
     int     parallel = 0;
     int     paralyzed = 1;
     int     invisible = 0;
@@ -789,12 +807,12 @@ static int start_animation(object *who, object *activator, object *event, const
         if (!strcmp(buffer, "\n"))
             continue;
         errors_found = 1;
-        cf_log(llevDebug, "CFAnim: '%s' has an invalid syntax.\n", buffer);
+        cf_log(llevError, "CFAnim: '%s' has an invalid syntax.\n", buffer);
     }
     if (feof(fichier))
         return 0;
     if (strncmp(buffer, "[Config]", 8)) {
-        cf_log(llevDebug, "CFAnim: Fatal error in %s: [Config] must be the first group defined.\n", file);
+        cf_log(llevError, "CFAnim: Fatal error in %s: [Config] must be the first group defined.\n", file);
         return 0;
     }
     while (fgets(buffer, HUGE_BUF, fichier)) {
@@ -833,13 +851,13 @@ static int start_animation(object *who, object *activator, object *event, const
                 else if (!strcmp(value, "who_owner"))
                     if (!who) {
                         errors_found = 1;
-                        cf_log(llevDebug, "CFAnim: Warning: object \"who\" doesn't exist and you're victimized it's owner\n");
+                        cf_log(llevError, "CFAnim: Warning: object \"who\" doesn't exist and you're victimized it's owner\n");
                     } else
                         victim = who->env;
                 else if (!strcmp(value, "activator_owner"))
                     if (!activator) {
                         errors_found = 1;
-                        cf_log(llevDebug, "CFAnim: Warning: object \"activator\" doesn't exist and you're victimized it's owner\n");
+                        cf_log(llevError, "CFAnim: Warning: object \"activator\" doesn't exist and you're victimized it's owner\n");
                     } else
                         victim = activator->env;
                 else if (victimtype == 3) {
@@ -852,6 +870,9 @@ static int start_animation(object *who, object *activator, object *event, const
             } else if (!strcmp(variable, "always_delete")) {
                 if (!get_boolean(value, &always_delete))
                     errors_found = 1;
+            } else if (!strcmp(variable, "delete_event_end")) {
+                if (!get_boolean(value, &delete_end))
+                    errors_found = 1;
             } else if (!strcmp(variable, "parallel")) {
                 if (!get_boolean(value, &parallel))
                     errors_found = 1;
@@ -892,30 +913,31 @@ static int start_animation(object *who, object *activator, object *event, const
     if (buffer[0] == '\0') {
         if (animationitem)
             cf_free_string(animationitem);
-        cf_log(llevDebug, "CFAnim: Errors occurred during the parsing of %s\n", file);
+        cf_log(llevError, "CFAnim: Errors occurred during the parsing of %s\n", file);
         return 0;
     }
     if (!victim) {
         if (animationitem)
             cf_free_string(animationitem);
-        cf_log(llevDebug,  "CFAnim: Fatal error - victim is NULL");
+        cf_log(llevError,  "CFAnim: Fatal error - victim is NULL");
         return 0;
     }
     if (!(current_anim = create_animation())) {
         if (animationitem)
             cf_free_string(animationitem);
-        cf_log(llevDebug, "CFAnim: Fatal error - Not enough memory.\n");
+        cf_log(llevError, "CFAnim: Fatal error - Not enough memory.\n");
         return 0;
     }
     if (always_delete) {
         /*if (verbose) printf("CFAnim: Freeing event nr. %d for %s.\n", current_event, who->name);*/
         cf_object_remove(event);
+        event = NULL;
     }
     if (((victim->type == PLAYER) && (victimtype == 1))
     || ((victim->type != PLAYER) && (victimtype == 0))
     || (errors_found && !errors_allowed)) {
         if (verbose)
-            cf_log(llevDebug, "CFAnim: No correct victim found or errors found, aborting.\n");
+            cf_log(llevError, "CFAnim: No correct victim found or errors found, aborting.\n");
         if (animationitem)
             cf_free_string(animationitem);
         return 0;
@@ -923,13 +945,16 @@ static int start_animation(object *who, object *activator, object *event, const
     if (unique && !always_delete) {
         /*if (verbose) printf("CFAnim: Freeing event nr. %d for %s.\n", current_event, who->name);*/
         cf_object_remove(event);
+        event = NULL;
     }
     current_anim->name = name;
     current_anim->victim = victim;
+    current_anim->event = event;
     current_anim->paralyze = paralyzed;
     current_anim->invisible = invisible;
     current_anim->wizard = wizard;
     current_anim->unique = unique;
+    current_anim->delete_end = delete_end;
     current_anim->ghosted = 0;
     current_anim->corpse = NULL;
     current_anim->time_representation = timetype;
@@ -943,7 +968,7 @@ static int start_animation(object *who, object *activator, object *event, const
                 if (buffer[0] == '[')
                     break;
             if (value == NULL) {
-                cf_log(llevDebug, "CFAnim: no matching animation %s in file.\n", animationitem);
+                cf_log(llevError, "CFAnim: no matching animation %s in file.\n", animationitem);
                 cf_free_string(animationitem);
                 return 0;
             }
@@ -1044,6 +1069,8 @@ static void animate(void) {
             else {
                 previous_anim->nextanimation = next;
             }
+            if (current->delete_end && current->event != NULL)
+                cf_object_remove(current->event);
             free(current->name);
             free(current);
             current = next;
@@ -1105,9 +1132,9 @@ CF_PLUGIN int postInitPlugin(void) {
     return 0;
 }
 
-CF_PLUGIN void *cfanim_globalEventListener(int *type, ...) {
+CF_PLUGIN int cfanim_globalEventListener(int *type, ...) {
     va_list args;
-    static int rv = 0;
+    int rv = 0;
     int event_code;
 
     va_start(args, type);
@@ -1118,14 +1145,15 @@ CF_PLUGIN void *cfanim_globalEventListener(int *type, ...) {
 
     va_end(args);
 
-    return &rv;
+    return rv;
 }
 
-CF_PLUGIN void *eventListener(int *type, ...) {
-    static int rv = 0;
+CF_PLUGIN int eventListener(int *type, ...) {
+    int rv = 0;
     va_list args;
     char *buf, message[MAX_BUF], script[MAX_BUF];
     object *who, *activator, *third, *event;
+    int query;
 
     va_start(args, type);
 
@@ -1139,9 +1167,14 @@ CF_PLUGIN void *eventListener(int *type, ...) {
     else
         message[0] = '\0';
 
-    va_arg(args, int); /* 'fix', ignored */
+    query = va_arg(args, int); /* 'fix', ignored */
     event = va_arg(args, object *);
 
+    if (query == 1 && strcmp(message, "query_object_is_animated") == 0) {
+        rv = is_animated_object(who);
+        return rv;
+    }
+
     /** @todo build from current map's path, probably */
     cf_get_maps_directory(event->slaying, script, sizeof(script));
     va_end(args);
@@ -1156,7 +1189,7 @@ CF_PLUGIN void *eventListener(int *type, ...) {
 
     rv = start_animation(who, activator, event, script, message);
 
-    return &rv;
+    return rv;
 }
 
 CF_PLUGIN int   closePlugin(void) {
