@@ -120,6 +120,10 @@ static PyObject *getSeasonName(PyObject *self, PyObject *args);
 static PyObject *getMonthName(PyObject *self, PyObject *args);
 static PyObject *getWeekdayName(PyObject *self, PyObject *args);
 static PyObject *getPeriodofdayName(PyObject *self, PyObject *args);
+static PyObject *addReply(PyObject *self, PyObject *args);
+static PyObject *setPlayerMessage(PyObject *self, PyObject *args);
+static PyObject *npcSay(PyObject *self, PyObject *args);
+static PyObject *costStringFromValue(PyObject *self, PyObject *args);
 
 /** Set up an Python exception object. */
 static void set_exception(const char *fmt, ...) {
@@ -178,6 +182,10 @@ static PyMethodDef CFPythonMethods[] = {
     { "GetMonthName",        getMonthName,           METH_VARARGS, NULL },
     { "GetWeekdayName",      getWeekdayName,         METH_VARARGS, NULL },
     { "GetPeriodofdayName",  getPeriodofdayName,     METH_VARARGS, NULL },
+    { "AddReply",            addReply,               METH_VARARGS, NULL },
+    { "SetPlayerMessage",    setPlayerMessage,       METH_VARARGS, NULL },
+    { "NPCSay",              npcSay,                 METH_VARARGS, NULL },
+    { "CostStringFromValue", costStringFromValue,    METH_VARARGS, NULL },
     { NULL, NULL, 0, NULL }
 };
 
@@ -646,6 +654,96 @@ static PyObject *getPeriodofdayName(PyObject *self, PyObject *args) {
     return Py_BuildValue("s", cf_get_periodofday_name(i));
 }
 
+static PyObject *addReply(PyObject *self, PyObject *args) {
+    char *word, *reply;
+    talk_info *talk;
+
+    if (current_context->talk == NULL) {
+        set_exception("not in a dialog context");
+        return NULL;
+    }
+    talk = current_context->talk;
+
+    if (!PyArg_ParseTuple(args, "ss", &word, &reply)) {
+        return NULL;
+    }
+
+    if (talk->replies_count == MAX_REPLIES) {
+        set_exception("too many replies");
+        return NULL;
+    }
+
+    talk->replies_words[talk->replies_count] = cf_add_string(word);
+    talk->replies[talk->replies_count] = cf_add_string(reply);
+    talk->replies_count++;
+    Py_INCREF(Py_None);
+    return Py_None;
+
+}
+
+static PyObject *setPlayerMessage(PyObject *self, PyObject *args) {
+    char *message;
+    int type = rt_reply;
+
+    if (current_context->talk == NULL) {
+        set_exception("not in a dialog context");
+        return NULL;
+    }
+
+    if (!PyArg_ParseTuple(args, "s|i", &message, &type)) {
+        return NULL;
+    }
+
+    if (current_context->talk->message != NULL)
+        cf_free_string(current_context->talk->message);
+    current_context->talk->message = cf_add_string(message);
+    current_context->talk->message_type = type;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *npcSay(PyObject *self, PyObject *args) {
+    Crossfire_Object *npc = NULL;
+    char *message, buf[2048];
+
+    if (!PyArg_ParseTuple(args, "O!s", &Crossfire_ObjectType, &npc, &message))
+        return NULL;
+
+    if (current_context->talk == NULL) {
+        set_exception("not in a dialog context");
+        return NULL;
+    }
+
+    if (current_context->talk->npc_msg_count == MAX_NPC) {
+        set_exception("too many NPCs");
+        return NULL;
+    }
+
+    if (strlen(message) >= sizeof(buf) - 1)
+        cf_log(llevError, "warning, too long message in npcSay, will be truncated");
+    /** @todo fix by wrapping monster_format_say() (or the whole talk structure methods) */
+    snprintf(buf, sizeof(buf), "%s says: %s", npc->obj->name, message);
+
+    current_context->talk->npc_msgs[current_context->talk->npc_msg_count] = cf_add_string(buf);
+    current_context->talk->npc_msg_count++;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *costStringFromValue(PyObject *self, PyObject *args) {
+    uint64 value;
+    char buf[2048];
+    int largest_coin = 0;
+
+    if (!PyArg_ParseTuple(args, "L|i", &value, &largest_coin))
+        return NULL;
+
+    cf_cost_string_from_value(value, largest_coin, buf, sizeof(buf));
+    return Py_BuildValue("s", buf);
+}
+
 static void initContextStack(void) {
     current_context = NULL;
     context_stack = NULL;
@@ -1005,7 +1103,7 @@ static void initConstants(PyObject *module) {
         { "DIRECTOR", DIRECTOR },
         { "GIRDLE", GIRDLE },
         { "FORCE", FORCE },
-        { "POTION_EFFECT", POTION_EFFECT },
+        { "POTION_RESIST_EFFECT", POTION_RESIST_EFFECT },
         { "EVENT_CONNECTOR", EVENT_CONNECTOR },
         { "CLOSE_CON", CLOSE_CON },
         { "CONTAINER", CONTAINER },
@@ -1171,6 +1269,13 @@ static void initConstants(PyObject *module) {
         { NULL, 0 }
     };
 
+    static const CFConstant cstReplyTypes[] = {
+        { "SAY", rt_say },
+        { "REPLY", rt_reply },
+        { "QUESTION", rt_question },
+        { NULL, 0 }
+    };
+
     addConstants(module, "Direction", cstDirection);
     addConstants(module, "Type", cstType);
     addConstants(module, "Move", cstMove);
@@ -1180,6 +1285,7 @@ static void initConstants(PyObject *module) {
     addConstants(module, "AttackTypeNumber", cstAttackTypeNumber);
     addConstants(module, "EventType", cstEventType);
     addSimpleConstants(module, "Time", cstTime);
+    addSimpleConstants(module, "ReplyType", cstReplyTypes);
 }
 
 /*
@@ -1346,16 +1452,13 @@ CF_PLUGIN void *getPluginProperty(int *type, ...) {
     return NULL;
 }
 
-CF_PLUGIN int cfpython_runPluginCommand(object *op, char *params) {
+CF_PLUGIN void cfpython_runPluginCommand(object *op, const char *params) {
     char buf[1024], path[1024];
     CFPContext *context;
-    static int rv = 0;
-
-    rv = 0;
 
     if (current_command < 0) {
         cf_log(llevError, "Illegal call of cfpython_runPluginCommand, call find_plugin_command first.\n");
-        return 1;
+        return;
     }
     snprintf(buf, sizeof(buf), "%s.py", cf_get_maps_directory(CustomCommand[current_command].script, path, sizeof(path)));
 
@@ -1376,14 +1479,12 @@ CF_PLUGIN int cfpython_runPluginCommand(object *op, char *params) {
     current_command = -999;
     if (!do_script(context, 0)) {
         freeContext(context);
-        return rv;
+        return;
     }
 
     context = popContext();
-    rv = context->returnvalue;
     freeContext(context);
 /*    printf("Execution complete"); */
-    return rv;
 }
 
 CF_PLUGIN int postInitPlugin(void) {
@@ -1420,17 +1521,15 @@ CF_PLUGIN int postInitPlugin(void) {
     return 0;
 }
 
-CF_PLUGIN void *cfpython_globalEventListener(int *type, ...) {
+CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
     va_list args;
-    static int rv = 0;
+    int rv = 0;
     CFPContext *context;
     char *buf;
     player *pl;
     object *op;
     context = malloc(sizeof(CFPContext));
 
-    rv = 0;
-
     va_start(args, type);
     context->event_code = va_arg(args, int);
 
@@ -1440,6 +1539,7 @@ CF_PLUGIN void *cfpython_globalEventListener(int *type, ...) {
     context->activator   = NULL;
     context->third       = NULL;
     context->event       = NULL;
+    context->talk        = NULL;
     rv = context->returnvalue = 0;
     cf_get_maps_directory("python/events/python_event.py", context->script, sizeof(context->script));
     strcpy(context->options, "");
@@ -1567,7 +1667,7 @@ CF_PLUGIN void *cfpython_globalEventListener(int *type, ...) {
 
     if (!do_script(context, 1)) {
         freeContext(context);
-        return &rv;
+        return rv;
     }
 
     context = popContext();
@@ -1579,18 +1679,16 @@ CF_PLUGIN void *cfpython_globalEventListener(int *type, ...) {
 
     freeContext(context);
 
-    return &rv;
+    return rv;
 }
 
-CF_PLUGIN void *eventListener(int *type, ...) {
-    static int rv = 0;
+CF_PLUGIN int eventListener(int *type, ...) {
+    int rv = 0;
     va_list args;
     char *buf;
     CFPContext *context;
     object *event;
 
-    rv = 0;
-
     context = malloc(sizeof(CFPContext));
 
     context->message[0] = 0;
@@ -1605,6 +1703,7 @@ CF_PLUGIN void *eventListener(int *type, ...) {
         snprintf(context->message, sizeof(context->message), "%s", buf);
     context->fix         = va_arg(args, int);
     event = va_arg(args, object *);
+    context->talk = va_arg(args, talk_info *);
     context->event_code  = event->subtype;
     context->event       = Crossfire_Object_wrap(event);
     cf_get_maps_directory(event->slaying, context->script, sizeof(context->script));
@@ -1615,13 +1714,13 @@ CF_PLUGIN void *eventListener(int *type, ...) {
 
     if (!do_script(context, 0)) {
         freeContext(context);
-        return &rv;
+        return rv;
     }
 
     context = popContext();
     rv = context->returnvalue;
     freeContext(context);
-    return &rv;
+    return rv;
 }
 
 CF_PLUGIN int   closePlugin(void) {
