@@ -52,8 +52,9 @@ static void free_object_assoc(object *key) {
 }
 
 static PyObject *Player_GetTitle(Crossfire_Object *whoptr, void *closure) {
+    char title[MAX_NAME];
     EXISTCHECK(whoptr);
-    return Py_BuildValue("s", cf_player_get_title(whoptr->obj));
+    return Py_BuildValue("s", cf_player_get_title(whoptr->obj, title, MAX_NAME));
 }
 
 static int Player_SetTitle(Crossfire_Object *whoptr, PyObject *value, void *closure) {
@@ -108,6 +109,16 @@ static PyObject *Crossfire_Player_Message(Crossfire_Player *who, PyObject *args)
     return Py_None;
 }
 
+static PyObject *Player_KnowledgeKnown(Crossfire_Player *who, PyObject *args) {
+    const char *knowledge;
+
+    EXISTCHECK(who);
+    if (!PyArg_ParseTuple(args, "s", &knowledge))
+        return NULL;
+
+    return Py_BuildValue("i", cf_player_knowledge_has(who->obj, knowledge));
+}
+
 static PyObject *Player_GetParty(Crossfire_Player *whoptr, void *closure) {
     EXISTCHECK(whoptr);
     return Crossfire_Party_wrap(cf_player_get_party(whoptr->obj));
@@ -511,13 +522,15 @@ static PyObject *Object_GetLevel(Crossfire_Object *whoptr, void *closure) {
 }
 
 static PyObject *Object_GetFace(Crossfire_Object *whoptr, void *closure) {
+    char buf[200];
     EXISTCHECK(whoptr);
-    return Py_BuildValue("i", cf_object_get_int_property(whoptr->obj, CFAPI_OBJECT_PROP_FACE));
+    return Py_BuildValue("s", cf_object_get_string_property(whoptr->obj, CFAPI_OBJECT_PROP_FACE, buf, sizeof(buf)));
 }
 
 static PyObject *Object_GetAnim(Crossfire_Object *whoptr, void *closure) {
+    char buf[200];
     EXISTCHECK(whoptr);
-    return Py_BuildValue("i", cf_object_get_int_property(whoptr->obj, CFAPI_OBJECT_PROP_ANIMATION));
+    return Py_BuildValue("s", cf_object_get_string_property(whoptr->obj, CFAPI_OBJECT_PROP_ANIMATION, buf, sizeof(buf)));
 }
 
 static PyObject *Object_GetAnimSpeed(Crossfire_Object *whoptr, void *closure) {
@@ -894,7 +907,10 @@ static PyObject *Object_GetAnimated(Crossfire_Object *whoptr, void *closure) {
     EXISTCHECK(whoptr);
     return Py_BuildValue("i", cf_object_get_flag(whoptr->obj, FLAG_ANIMATE));
 }
-
+static PyObject *Object_GetMaterial(Crossfire_Object *whoptr, void *closure) {
+    EXISTCHECK(whoptr);
+    return Py_BuildValue("{s:s,s:i}", "Name", cf_object_get_sstring_property(whoptr->obj, CFAPI_OBJECT_PROP_MATERIAL_NAME), "Number", cf_object_get_int_property(whoptr->obj, CFAPI_OBJECT_PROP_MATERIAL));
+}
 /** Setters */
 static int Object_SetMessage(Crossfire_Object *whoptr, PyObject *value, void *closure) {
     char *val;
@@ -1399,38 +1415,31 @@ static int Object_SetLastEat(Crossfire_Object *whoptr, PyObject *value, void *cl
 }
 
 static int Object_SetFace(Crossfire_Object *whoptr, PyObject *value, void *closure) {
-    char *txt;
-    int face;
+    char *face;
 
     EXISTCHECK_INT(whoptr);
-    if (!PyArg_Parse(value, "s", &txt))
+    if (!PyArg_Parse(value, "s", &face))
         return -1;
 
-    face = cf_find_face(txt, -1);
-    if (face == -1) {
+    if (!cf_object_set_face(whoptr->obj, face)) {
         PyErr_SetString(PyExc_TypeError, "Unknown face.");
         return -1;
     }
-
-    cf_object_set_int_property(whoptr->obj, CFAPI_OBJECT_PROP_FACE, face);
     return 0;
 }
 
 static int Object_SetAnim(Crossfire_Object *whoptr, PyObject *value, void *closure) {
-    char *txt;
-    int anim;
+    char *anim;
 
     EXISTCHECK_INT(whoptr);
-    if (!PyArg_Parse(value, "s", &txt))
+    if (!PyArg_Parse(value, "s", &anim))
         return -1;
 
-    anim = cf_find_animation(txt);
-    if (anim == 0) {
+    if (!cf_object_set_animation(whoptr->obj, anim)) {
         PyErr_SetString(PyExc_TypeError, "Unknown animation.");
         return -1;
     }
 
-    cf_object_set_int_property(whoptr->obj, CFAPI_OBJECT_PROP_ANIMATION, anim);
     return 0;
 }
 
@@ -2037,7 +2046,7 @@ static PyObject *Crossfire_Object_Take(Crossfire_Object *who, PyObject *args) {
     EXISTCHECK(who);
     TYPEEXISTCHECK(whoptr);
 
-    cf_object_pickup(whoptr->obj, who->obj);
+    cf_object_pickup(who->obj, whoptr->obj);
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -2088,14 +2097,35 @@ static PyObject *Crossfire_Object_CheckTrigger(Crossfire_Object *who, PyObject *
 }
 
 static PyObject *Crossfire_Object_Say(Crossfire_Object *who, PyObject *args) {
-    char *message;
+    char *message, buf[2048];
 
     EXISTCHECK(who);
     if (!PyArg_ParseTuple(args, "s", &message))
         return NULL;
-    cf_object_say(who->obj, message);
+
+    /* compatibility */
+    if (current_context->talk == NULL) {
+        cf_object_say(who->obj, message);
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
+    if (current_context->talk->npc_msg_count == MAX_NPC) {
+        PyErr_SetString(PyExc_ValueError, "too many NPCs");
+        return NULL;
+    }
+
+    if (strlen(message) >= sizeof(buf) - 1)
+        cf_log(llevError, "warning, too long message in npcSay, will be truncated");
+    /** @todo fix by wrapping monster_format_say() (or the whole talk structure methods) */
+    snprintf(buf, sizeof(buf), "%s says: %s", who->obj->name, message);
+
+    current_context->talk->npc_msgs[current_context->talk->npc_msg_count] = cf_add_string(buf);
+    current_context->talk->npc_msg_count++;
+
     Py_INCREF(Py_None);
     return Py_None;
+
 }
 
 static PyObject *Crossfire_Object_Reposition(Crossfire_Object *who, PyObject *args) {
