@@ -30,48 +30,75 @@
 #include <cfpython.h>
 #include <cfpython_region_private.h>
 
-static PyObject* Crossfire_Region_GetName(Crossfire_Region* regionptr, void* closure)
-{
-	return Py_BuildValue("s",cf_region_get_name(regionptr->reg));
+static PyObject *Crossfire_Region_GetName(Crossfire_Region *regionptr, void *closure) {
+    return Py_BuildValue("s", cf_region_get_name(regionptr->reg));
 }
 
-static PyObject* Crossfire_Region_GetLongname(Crossfire_Region* regionptr, void* closure)
-{
-	return Py_BuildValue("s",cf_region_get_longname(regionptr->reg));
+static PyObject *Crossfire_Region_GetLongname(Crossfire_Region *regionptr, void *closure) {
+    return Py_BuildValue("s", cf_region_get_longname(regionptr->reg));
 }
 
-static PyObject* Crossfire_Region_GetMessage(Crossfire_Region* regionptr, void* closure)
-{
-	return Py_BuildValue("s",cf_region_get_message(regionptr->reg));
+static PyObject *Crossfire_Region_GetMessage(Crossfire_Region *regionptr, void *closure) {
+    return Py_BuildValue("s", cf_region_get_message(regionptr->reg));
 }
 
-static PyObject* Crossfire_Region_GetNext( Crossfire_Region* party, void* closure )
-{
-	return Crossfire_Region_wrap(cf_region_get_next(party->reg));
+static PyObject *Crossfire_Region_GetNext(Crossfire_Region *party, void *closure) {
+    return Crossfire_Region_wrap(cf_region_get_next(party->reg));
 }
 
-static PyObject* Crossfire_Region_GetParent( Crossfire_Region* party, PyObject* args )
-{
-	return Crossfire_Region_wrap(cf_region_get_parent(party->reg));
+static PyObject *Crossfire_Region_GetParent(Crossfire_Region *party, PyObject *args) {
+    return Crossfire_Region_wrap(cf_region_get_parent(party->reg));
 }
 
-PyObject *Crossfire_Region_wrap(region *what)
-{
+PyObject *Crossfire_Region_wrap(region *what) {
     Crossfire_Region *wrapper;
 
     /* return None if no object was to be wrapped */
-    if(what == NULL) {
+    if (what == NULL) {
         Py_INCREF(Py_None);
         return Py_None;
     }
 
     wrapper = PyObject_NEW(Crossfire_Region, &Crossfire_RegionType);
-    if(wrapper != NULL)
+    if (wrapper != NULL)
         wrapper->reg = what;
     return (PyObject *)wrapper;
 }
 
-static int Crossfire_Region_InternalCompare(Crossfire_Region* left, Crossfire_Region* right)
-{
-	return (left->reg < right->reg ? -1 : ( left->reg == right->reg ? 0 : 1 ) );
+static int Crossfire_Region_InternalCompare(Crossfire_Region *left, Crossfire_Region *right) {
+    return (left->reg < right->reg ? -1 : (left->reg == right->reg ? 0 : 1));
+}
+
+static PyObject *Crossfire_Region_RichCompare(Crossfire_Region *left, Crossfire_Region *right, int op) {
+    int result;
+    if (!left
+        || !right
+        || !PyObject_TypeCheck((PyObject*)left, &Crossfire_RegionType)
+        || !PyObject_TypeCheck((PyObject*)right, &Crossfire_RegionType)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+    result = Crossfire_Region_InternalCompare(left, right);
+    /* Based on how Python 3.0 (GPL compatible) implements it for internal types: */
+    switch (op) {
+        case Py_EQ:
+            result = (result == 0);
+            break;
+        case Py_NE:
+            result = (result != 0);
+            break;
+        case Py_LE:
+            result = (result <= 0);
+            break;
+        case Py_GE:
+            result = (result >= 0);
+            break;
+        case Py_LT:
+            result = (result == -1);
+            break;
+        case Py_GT:
+            result = (result == 1);
+            break;
+    }
+    return PyBool_FromLong(result);
 }
