@@ -93,14 +93,15 @@
 
 #undef MODULEAPI
 #ifdef WIN32
-#ifdef PYTHON_PLUGIN_EXPORTS
-#define MODULEAPI __declspec(dllexport)
+# ifdef PYTHON_PLUGIN_EXPORTS
+#  define MODULEAPI __declspec(dllexport)
+# else
+#  define MODULEAPI __declspec(dllimport)
+# endif
+#elif HAVE_VISIBILITY
+# define MODULEAPI __attribute__((visibility("default")))
 #else
-#define MODULEAPI __declspec(dllimport)
-#endif
-
-#else
-#define MODULEAPI
+# define MODULEAPI
 #endif
 
 #define PLUGIN_NAME    "Python"
@@ -126,6 +127,7 @@ typedef struct _cfpcontext {
     char        options[1024];
     int         returnvalue;
     int         parms[5];
+    struct talk_info    *talk;
 } CFPContext;
 
 extern f_plug_api gethook;
