@@ -280,6 +280,7 @@ static PyObject *Crossfire_Object_Move(Crossfire_Object *who, PyObject *args);
 static PyObject *Crossfire_Object_ChangeAbil(Crossfire_Object *who, PyObject *args);
 static PyObject *Crossfire_Object_Event(Crossfire_Object *who, PyObject *args);
 static PyObject *Crossfire_Object_RemoveDepletion(Crossfire_Object *who, PyObject *args);
+static PyObject *Object_GetMaterial(Crossfire_Object *whoptr, void *closure);
 
 static int Crossfire_Object_InternalCompare(Crossfire_Object *left, Crossfire_Object *right);
 static PyObject *Crossfire_Object_RichCompare(Crossfire_Object *left, Crossfire_Object *right, int op);
@@ -419,6 +420,7 @@ static PyGetSetDef Object_getseters[] = {
     { "Animated",       (getter)Object_GetAnimated,     (setter)Object_SetAnimated, NULL, NULL },
     { "NoDamage",       (getter)Object_GetNoDamage,     (setter)Object_SetNoDamage, NULL, NULL },
     { "RandomMovement", (getter)Object_GetRandomMovement, (setter)Object_SetRandomMovement, NULL, NULL },
+    { "Material",       (getter)Object_GetMaterial,     NULL, NULL, NULL },
     { NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -611,6 +613,7 @@ static PyObject *Player_QuestStart(Crossfire_Player *whoptr, PyObject *args);
 static PyObject *Player_QuestGetState(Crossfire_Player *whoptr, PyObject *args);
 static PyObject *Player_QuestSetState(Crossfire_Player *whoptr, PyObject *args);
 static PyObject *Player_QuestWasCompleted(Crossfire_Player *whoptr, PyObject *args);
+static PyObject *Player_KnowledgeKnown(Crossfire_Player *whoptr, PyObject *args);
 
 static PyGetSetDef Player_getseters[] = {
     { "Title",         (getter)Player_GetTitle,         (setter)Player_SetTitle, NULL, NULL },
@@ -631,6 +634,7 @@ static PyMethodDef PlayerMethods[] = {
     { "QuestGetState",  (PyCFunction)Player_QuestGetState,     METH_VARARGS,  NULL },
     { "QuestSetState",  (PyCFunction)Player_QuestSetState,     METH_VARARGS,  NULL },
     { "QuestWasCompleted",  (PyCFunction)Player_QuestWasCompleted, METH_VARARGS,  NULL },
+    { "KnowledgeKnown",  (PyCFunction)Player_KnowledgeKnown, METH_VARARGS,  NULL },
     { NULL, NULL, 0, NULL }
 };
 
