@@ -31,6 +31,8 @@
 
 #ifdef WIN32
 #define CF_PLUGIN __declspec(dllexport)
+#elif HAVE_VISIBILITY
+#define CF_PLUGIN __attribute__((visibility("default")))
 #else
 #define CF_PLUGIN
 #endif
@@ -40,7 +42,7 @@
 #define PLUGIN_ERROR_INT        0x0FFFFF
 
 extern int cf_init_plugin(f_plug_api getHooks);
-extern void cf_system_register_global_event(int event, const char *name, f_plug_api hook);
+extern void cf_system_register_global_event(int event, const char *name, f_plug_event hook);
 extern void cf_system_unregister_global_event(int event, const char *name);
 
 /* General functions */
@@ -62,7 +64,7 @@ extern const char  *cf_get_season_name(int index);
 extern const char  *cf_get_month_name(int index);
 extern const char  *cf_get_weekday_name(int index);
 extern const char  *cf_get_periodofday_name(int index);
-extern void         cf_cost_string_from_value(uint64 cost, char *buffer, int length);
+extern void         cf_cost_string_from_value(uint64 cost, int largest_coin, char *buffer, int length);
 
 /* Objects */
 extern void         cf_object_set_int_property(object *op, int propcode, int value);
@@ -139,8 +141,10 @@ extern int          cf_object_user_event(object *op, object *activator, object *
 extern int          cf_object_remove_depletion(object *op, int level);
 extern object      *cf_object_find_by_arch_name(const object *who, const char *name);
 extern object      *cf_object_find_by_name(const object *who, const char *name);
-extern object      *cf_object_split(object *op, int nr, char *buf, int size);
+extern object      *cf_object_split(object *orig_ob, uint32 nr, char *err, size_t size);
 extern object      *cf_object_clone(object *op, int clonetype);
+extern int          cf_object_set_face(object *op, const char *face);
+extern int          cf_object_set_animation(object *op, const char *animation);
 /* Maps */
 /*extern void        *cf_map_get_property(mapstruct *map, int propcode);*/
 extern sstring cf_map_get_sstring_property(mapstruct *map, int propcode);
@@ -179,7 +183,7 @@ extern int          cf_random_map_set_variable(RMParms *rp, const char *buf);
 extern mapstruct   *cf_random_map_generate(const char *OutFileName, RMParms *RP, char **use_layout);
 
 /* Players */
-extern sstring      cf_player_get_title(object *op);
+extern char        *cf_player_get_title(object *op, char *title, int size);
 extern void         cf_player_set_title(object *op, const char *title);
 extern sstring      cf_player_get_ip(object *op);
 extern object      *cf_player_get_marked_item(object *op);
@@ -190,6 +194,7 @@ extern int          cf_player_move(player *pl, int dir);
 extern partylist   *cf_player_get_party(object *op);
 extern void         cf_player_set_party(object *op, partylist *party);
 extern int          cf_player_can_pay(object *op);
+extern int          cf_player_knowledge_has(object *op, const char *knowledge);
 
 /* Archetypes */
 extern archetype   *cf_archetype_get_first(void);
