@@ -128,6 +128,7 @@ static f_plug_api cfapiObject_user_event = NULL;
 static f_plug_api cfapiCost_string_from_value = NULL;
 static f_plug_api cfapiPlayer_quest = NULL;
 static f_plug_api cfapiObject_remove_depletion = NULL;
+static f_plug_api cfapiPlayer_knowledge = NULL;
 
 #define GET_HOOK(x, y, z) { \
     getHooks(&z, 1, y, &x); \
@@ -230,6 +231,7 @@ int cf_init_plugin(f_plug_api getHooks) {
     GET_HOOK(cfapiObject_remove_depletion, "cfapi_object_remove_depletion", z);
     GET_HOOK(cfapiObject_find_by_arch_name, "cfapi_object_find_by_arch_name", z);
     GET_HOOK(cfapiObject_find_by_name, "cfapi_object_find_by_name", z);
+    GET_HOOK(cfapiPlayer_knowledge, "cfapi_player_knowledge", z);
     return 1;
 }
 
@@ -443,9 +445,40 @@ char *cf_object_get_string_property(object *op, int propcode, char *buf, int siz
 void cf_object_set_string_property(object *op, int propcode, const char *value) {
     int type;
 
+    /* use cf_object_set_face() for changing the face! */
+    assert(propcode != CFAPI_OBJECT_PROP_FACE);
+    /* use cf_object_set_animation() to change the animation */
+    assert(propcode != CFAPI_OBJECT_PROP_ANIMATION);
+
     cfapiObject_set_property(&type, op, propcode, value);
     assert(type == CFAPI_STRING);
 }
+
+/**
+ * Set the object's face.
+ * @param op who to change the face for.
+ * @param face face to set.
+ * @return 0 if invalid face, non zero is set.
+ */
+int cf_object_set_face(object *op, const char *face) {
+    int type, ret;
+    cfapiObject_set_property(&type, op, CFAPI_OBJECT_PROP_FACE, face, &ret);
+    assert(type == CFAPI_INT);
+    return ret;
+}
+/**
+ * Set the object's animation.
+ * @param op who to change the animation for.
+ * @param animation animation's name to set.
+ * @return 0 if invalid animation, non zero is set.
+ */
+int cf_object_set_animation(object *op, const char *animation) {
+    int type, ret;
+    cfapiObject_set_property(&type, op, CFAPI_OBJECT_PROP_ANIMATION, animation, &ret);
+    assert(type == CFAPI_INT);
+    return ret;
+}
+
 void cf_object_set_object_property(object *op, int propcode, object *value) {
     int type;
 
@@ -636,11 +669,11 @@ object *cf_object_clone(object *op, int clonetype) {
  * Wrapper for object_split().
  * @copydoc object_split().
  */
-object *cf_object_split(object *op, int nr, char *buf, int size) {
+object *cf_object_split(object *orig_ob, uint32 nr, char *err, size_t size) {
     int type;
     object *value;
 
-    cfapiObject_split(&type, op, nr, buf, size, &value);
+    cfapiObject_split(&type, orig_ob, nr, err, size, &value);
 
     if (value == NULL)
     {
@@ -770,13 +803,12 @@ player *cf_player_find(const char *plname) {
     return value;
 }
 
-sstring cf_player_get_title(object *op) {
+char *cf_player_get_title(object *op, char *title, int size) {
     int type;
-    sstring value;
 
-    cfapiObject_get_property(&type, op, CFAPI_PLAYER_PROP_TITLE, &value);
-    assert(type == CFAPI_SSTRING);
-    return value;
+    cfapiObject_get_property(&type, op, CFAPI_PLAYER_PROP_TITLE, title, size);
+    assert(type == CFAPI_STRING);
+    return title;
 }
 
 void cf_player_set_title(object *op, const char *title) {
@@ -831,6 +863,20 @@ int cf_player_can_pay(object *pl) {
     return value;
 }
 
+/**
+ * Wrapper for knowledge_player_has().
+ * @param op who to check knowledge for.
+ * @param knowledge what to check for.
+ * @return 0 if op is not a player or knowledge is not known, 1 else.
+ */
+int cf_player_knowledge_has(object *op, const char *knowledge) {
+    int type, value;
+
+    cfapiPlayer_knowledge(&type, 1, op, knowledge, &value);
+    assert(type == CFAPI_INT);
+    return value;
+}
+
 /**
  * Wrapper for ready_map_name().
  * @copydoc ready_map_name()
@@ -916,10 +962,10 @@ void cf_object_query_cost_string(const object *tmp, object *who, int flag, char
 /**
  * Wrapper for cost_string_from_value modified to take a char* and length instead of a StringBuffer.
  */
-void cf_cost_string_from_value(uint64 cost, char *buffer, int length) {
+void cf_cost_string_from_value(uint64 cost, int largest_coin, char *buffer, int length) {
     int type;
 
-    cfapiCost_string_from_value(&type, cost, buffer, length);
+    cfapiCost_string_from_value(&type, cost, largest_coin, buffer, length);
     assert(type == CFAPI_NONE);
 }
 
@@ -1024,7 +1070,7 @@ object *cf_create_object_by_name(const char *name) {
     return value;
 }
 
-void cf_system_register_global_event(int event, const char *name, f_plug_api hook) {
+void cf_system_register_global_event(int event, const char *name, f_plug_event hook) {
     int type;
 
     cfapiSystem_register_global_event(&type, event, name, hook);
