@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_standalone_c =
- *   "$Id: standalone.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *   "$Id: standalone.c 14102 2010-11-19 06:35:42Z mwedel $";
  */
 
 /*
@@ -105,7 +105,7 @@ int apply_auto(object *op) {
         SET_FLAG(tmp, FLAG_UNPAID);
         object_insert_in_map_at(tmp, op->map, NULL, 0, op->x, op->y);
         CLEAR_FLAG(op, FLAG_AUTO_APPLY);
-        identify(tmp);
+        tmp = identify(tmp);
         break;
 
     case TREASURE:
@@ -164,21 +164,21 @@ void apply_auto_fix(mapstruct *m) {
  * Added as part of glue cleaning.
  * Ryo 2005-07-15
  **/
-void draw_ext_info(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *txt, const char *txt2) {
-    fprintf(logfile, "%s\n", txt);
+void draw_ext_info(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *message) {
+    fprintf(logfile, "%s\n", message);
 }
 
-void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *new_format, const char *old_format, ...) {
+void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *format, ...) {
     va_list ap;
 
-    va_start(ap, old_format);
-    vfprintf(logfile, old_format, ap);
+    va_start(ap, format);
+    vfprintf(logfile, format, ap);
     va_end(ap);
 }
 
 
-void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, const char *str1, const char *str2) {
-    fprintf(logfile, "ext_info_map: %s\n", str2);
+void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, const char *str1) {
+    fprintf(logfile, "ext_info_map: %s\n", str1);
 }
 
 void move_firewall(object *ob) {
