@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_account_c =
- *   "$Id: account.c 12864 2010-04-07 07:15:56Z mwedel $";
+ *   "$Id: account.c 14125 2010-11-24 19:01:48Z akirschbaum $";
  */
 
 /*
@@ -67,7 +67,7 @@
 #include <sproto.h>
 #endif
 
-/* Number of fields in the accounts file.  These are colon seperated */
+/** Number of fields in the accounts file.  These are colon seperated */
 #define NUM_ACCOUNT_FIELDS 6
 
 /**
@@ -82,15 +82,15 @@
  */
 
 typedef struct account_struct {
-    char  *name;                    /** Account name */
-    char  *password;                /** Password for this account */
-    time_t last_login;              /** Last time this account was logged in */
-    int	  num_characters;           /** Number of characters on this account */
+    char  *name;                    /**< Account name */
+    char  *password;                /**< Password for this account */
+    time_t last_login;              /**< Last time this account was logged in */
+    int	  num_characters;           /**< Number of characters on this account */
     char  *character_names[MAX_CHARACTERS_PER_ACCOUNT+1];
-                                    /** character names associated with this account */
-                                    /** +1 added to allow for NULL termination */
-    time_t  created;                /** When character was created */
-    struct account_struct *next;    /** Next in list */
+                                    /**< Character names associated with this account
+                                     +1 added to allow for NULL termination */
+    time_t  created;                /**< When character was created */
+    struct account_struct *next;    /**< Next in list */
 } account_struct;
 
 /**
@@ -134,7 +134,8 @@ void account_load_entries(void)
         /* This may not in fact be a critical error - on a new server,
          * the accounts file may not yet exist.
          */
-        LOG(llevInfo,"Warning: Unable to open %s\n");
+        char err[MAX_BUF];
+        LOG(llevInfo,"Warning: Unable to open %s [%s]\n", fname, strerror_local(errno, err, sizeof(err)));
         return;
     }
     while (fgets(buf, VERY_BIG_BUF, fp)) {
@@ -311,7 +312,7 @@ void accounts_save(void)
  * @return
  * returns official name on match, NULL on no match.
  */
-const char *account_exists(char *account_name)
+const char *account_exists(const char *account_name)
 {
     account_struct *ac;
 
@@ -336,7 +337,7 @@ const char *account_exists(char *account_name)
  * @return
  * 0 if no match/wrong password, 1 if a match is found and password matches.
  */
-int account_check_name_password(char *account_name, char *account_password)
+int account_check_name_password(const char *account_name, const char *account_password)
 {
     account_struct *ac;
 
@@ -408,7 +409,7 @@ int account_check_string(const char *str)
  * @retval 2
  * account already exists.
  */
-int account_add_account(char *account_name, char *account_password)
+int account_add_account(const char *account_name, const char *account_password)
 {
     account_struct *ac;
 
@@ -463,7 +464,7 @@ int account_add_account(char *account_name, char *account_password)
  * number of characters on this account has reached a maximum.
  */
 
-int account_add_player_to_account(char *account_name, char *player_name)
+int account_add_player_to_account(const char *account_name, const char *player_name)
 {
     account_struct *ac;
 
@@ -544,7 +545,7 @@ int account_remove_player_from_account(const char *account_name, const char *pla
  * first entry being NULL in the case of a valid account with no characters added.
  * This returned data should not be altered in any way.
  */
-char **account_get_players_for_account(char *account_name)
+char **account_get_players_for_account(const char *account_name)
 {
     account_struct *ac;
 
@@ -556,9 +557,9 @@ char **account_get_players_for_account(char *account_name)
 
 /**
  * This looks at all the accounts and sees if charname is associated
- * with any of them. 
+ * with any of them.
  *
- * @params charname
+ * @param charname
  * character name to check for.
  * @return
  * Account name the character is associated with, NULL if none.
@@ -614,7 +615,8 @@ socket_struct *account_get_logged_in_init_socket(const char *name)
     int i;
 
     for (i=0; i < socket_info.allocated_sockets; i++) {
-        if (init_sockets[i].account_name &&
+        if (init_sockets[i].status == Ns_Add &&
+            init_sockets[i].account_name &&
             !strcasecmp(init_sockets[i].account_name, name)) return(&init_sockets[i]);
     }
     return NULL;
