@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *   "$Id: alchemy.c 14126 2010-11-24 19:07:22Z akirschbaum $";
  */
 
 /*
@@ -40,6 +40,7 @@
 #endif
 #include <skills.h>
 #include <spells.h>
+#include <assert.h>
 
 /** define this for some helpful debuging information */
 #if 0
@@ -71,16 +72,16 @@ static const char *const cauldron_effect [] = {
 
 
 static int is_defined_recipe(const recipe *rp, const object *cauldron, object *caster);
-static recipe *find_recipe(recipelist *fl, int formula, object *ingredients);
+static const recipe *find_recipe(const recipelist *fl, int formula, object *ingredients);
 static int content_recipe_value(object *op);
-static int numb_ob_inside(object *op);
-static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int danger);
-static object *attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches, int ignore_cauldron);
-static int calc_alch_danger(object *caster, object *cauldron, recipe *rp);
-static object *make_item_from_recipe(object *cauldron, recipe *rp);
+static int numb_ob_inside(const object *op);
+static void alchemy_failure_effect(object *op, object *cauldron, const recipe *rp, int danger);
+static object *attempt_recipe(object *caster, object *cauldron, int ability, const recipe *rp, int nbatches, int ignore_cauldron);
+static int calc_alch_danger(object *caster, object *cauldron, const recipe *rp);
+static object *make_item_from_recipe(object *cauldron, const recipe *rp);
 static void remove_contents(object *first_ob, object *save_item);
 static void adjust_product(object *item, int lvl, int yield);
-static object *find_transmution_ob(object *first_ingred, recipe *rp, size_t *rp_arch_index, int create_item);
+static object *find_transmution_ob(object *first_ingred, const recipe *rp, size_t *rp_arch_index, int create_item);
 static void attempt_do_alchemy(object *caster, object *cauldron);
 
 /** Returns a random selection from cauldron_effect[] */
@@ -90,6 +91,34 @@ static const char *cauldron_sound(void) {
     return cauldron_effect[rndm(0, size-1)];
 }
 
+/**
+ * Compute the success probability of a recipe.
+ *
+ * The scale is kind of linear, with 3 steps depending on the level difference between recipe and skill.
+ *
+ * @param rp recipe to attempt.
+ * @param skill skill being used.
+ * @param cauldron provides the magic of the used device.
+ * @return chance between 0.01 and .95.
+ */
+static float recipe_chance(const recipe *rp, const object *skill, const object *cauldron) {
+    int cauldron_add_skill;
+
+    assert(rp);
+    assert(skill);
+    assert(cauldron);
+
+    cauldron_add_skill = (cauldron->magic + 1) / 2;
+
+    if (skill->level + cauldron_add_skill < rp->diff - 10)
+        return MAX(.01, .3 - (rp->diff - 10 - skill->level - cauldron_add_skill) * .03);
+
+    if (skill->level + cauldron_add_skill <= rp->diff + 10)
+        return .5 + .02 * (float)(skill->level + cauldron_add_skill - rp->diff);
+
+    return MIN(.95, .70 + (skill->level + cauldron_add_skill - rp->diff - 10) * .01);
+}
+
 /**
  * Main part of the ALCHEMY code. From this we call fctns
  * that take a look at the contents of the 'cauldron' and, using these ingredients,
@@ -119,12 +148,11 @@ static const char *cauldron_sound(void) {
  * the cauldron in which alchemy should take place.
  */
 static void attempt_do_alchemy(object *caster, object *cauldron) {
-    recipelist *fl;
-    recipe *rp = NULL;
+    const recipelist *fl;
+    const recipe *rp = NULL;
     float success_chance;
     int numb, ability = 1;
     int formula = 0;
-    float ave_chance;
     object *item, *skop;
 
     if (caster->type != PLAYER)
@@ -133,7 +161,6 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
     /* if no ingredients, no formula! lets forget it */
     if (!(formula = content_recipe_value(cauldron))) {
         draw_ext_info_format(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "The %s is empty.",
                              "The %s is empty.",
                              cauldron->name);
         return;
@@ -163,13 +190,12 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
             uint64 value_item;
             int attempt_shadow_alchemy;
 
-            ave_chance = fl->total_chance/(float)fl->number;
             /* the caster gets an increase in ability based on thier skill lvl */
             if (rp->skill != NULL) {
                 skop = find_skill_by_name(caster, rp->skill);
                 if (!skop) {
                     draw_ext_info(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                                  "You do not have the proper skill for this recipe", NULL);
+                                  "You do not have the proper skill for this recipe");
                 } else {
                     ability += skop->level*((4.0+cauldron->magic)/4.0);
                 }
@@ -185,7 +211,7 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
 
             if (rp->min_level != 0 && skop->level < rp->min_level) {
                 draw_ext_info(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                    "You aren't skilled enough to try this recipe.", NULL);
+                    "You aren't skilled enough to try this recipe.");
                 return;
             }
 
@@ -200,9 +226,7 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
             /* create the object **FIRST**, then decide whether to keep it. */
             if ((item = attempt_recipe(caster, cauldron, ability, rp, formula/rp->index, attempt_shadow_alchemy)) != NULL) {
                 /*  compute base chance of recipe success */
-                success_chance = ((float)ability/(float)(rp->diff*(item->level+2)));
-                if (ave_chance == 0)
-                    ave_chance = 1;
+                success_chance = recipe_chance(rp, skop, cauldron);
 
 #ifdef ALCHEMY_DEBUG
                 LOG(llevDebug, "percent success chance =  %f ab%d / diff%d*lev%d\n", success_chance, ability, rp->diff, item->level);
@@ -267,7 +291,7 @@ static int content_recipe_value(object *op) {
  * @return
  * total item count.
  */
-static int numb_ob_inside(object *op) {
+static int numb_ob_inside(const object *op) {
     int o_number = 0;
 
     FOR_INV_PREPARE(op, tmp)
@@ -304,7 +328,7 @@ static int numb_ob_inside(object *op) {
  * generated item, can be NULL if contents were destroyed.
  * @todo check meaning of ability/nbatches.
  */
-static object *attempt_recipe(object *caster, object *cauldron, int ability, recipe *rp, int nbatches, int ignore_cauldron) {
+static object *attempt_recipe(object *caster, object *cauldron, int ability, const recipe *rp, int nbatches, int ignore_cauldron) {
     object *item = NULL, *skop;
     /* this should be passed to this fctn, not effiecent cpu use this way */
     int batches = abs(nbatches);
@@ -312,7 +336,7 @@ static object *attempt_recipe(object *caster, object *cauldron, int ability, rec
     /* is the cauldron the right type? */
     if (!ignore_cauldron && (strcmp(rp->cauldron, cauldron->arch->name) != 0)) {
         draw_ext_info(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "You are not using the proper facilities for this formula.", NULL);
+                      "You are not using the proper facilities for this formula.");
         return NULL;
     }
 
@@ -328,8 +352,7 @@ static object *attempt_recipe(object *caster, object *cauldron, int ability, rec
         tmp = object_find_by_type_and_slaying(caster, FORCE, rp->keycode);
         if (tmp == NULL) { /* failure--no code found */
             draw_ext_info(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                          "You know the ingredients, but not the technique.  Go learn how to do this recipe.",
-                          NULL);
+                          "You know the ingredients, but not the technique.  Go learn how to do this recipe.");
             return NULL;
         }
     }
@@ -345,12 +368,11 @@ static object *attempt_recipe(object *caster, object *cauldron, int ability, rec
         adjust_product(item, ability, rp->yield ? (rp->yield*batches) : batches);
         if (!item->env && (item = object_insert_in_ob(item, cauldron)) == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, caster, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                          "Nothing happened.", NULL);
+                          "Nothing happened.");
         } else {
             draw_ext_info_format(NDI_UNIQUE, 0, caster,
                                  MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
                                  "The %s %s.",
-                                 "The %s %s.",
                                  cauldron->name, cauldron_sound());
         }
     }
@@ -392,8 +414,8 @@ static void adjust_product(object *item, int lvl, int yield) {
  * @return the newly created object, NULL if something failed
  */
 
-static object *make_item_from_recipe(object *cauldron, recipe *rp) {
-    artifact *art = NULL;
+static object *make_item_from_recipe(object *cauldron, const recipe *rp) {
+    const artifact *art = NULL;
     object *item = NULL;
     size_t rp_arch_index;
 
@@ -444,7 +466,7 @@ static object *make_item_from_recipe(object *cauldron, recipe *rp) {
  * @return
  * NULL if no suitable item was found and create_item is 0, existing or new item else.
  */
-static object *find_transmution_ob(object *first_ingred, recipe *rp, size_t *rp_arch_index, int create_item) {
+static object *find_transmution_ob(object *first_ingred, const recipe *rp, size_t *rp_arch_index, int create_item) {
     object *item = NULL;
 
     *rp_arch_index = 0;
@@ -498,7 +520,7 @@ static object *find_transmution_ob(object *first_ingred, recipe *rp, size_t *rp_
  * @param danger
  * danger value, the higher the more evil the effect.
  */
-static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int danger) {
+static void alchemy_failure_effect(object *op, object *cauldron, const recipe *rp, int danger) {
     int level = 0;
 
     if (!op || !cauldron)
@@ -516,7 +538,7 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
 
         object_insert_in_ob(failure, cauldron);
         if (rp->failure_message) {
-            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, rp->failure_message, NULL);
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, rp->failure_message);
         }
         return;
     }
@@ -559,18 +581,27 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
         }
         remove_contents(cauldron->inv, item);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "The %s %s.",
                              "The %s %s.",
                              cauldron->name, cauldron_sound());
         return;
     } else if (level < 40) {                 /* MAKE TAINTED ITEM */
         object *tmp = NULL;
 
+        /*
+         * Note by Nicolas Weeger 2010-09-26
+         * This is an incorrect part.
+         * Calling again attempt_recipe in case of failure will apply again the artifact
+         * combination to the item.
+         * This leads to items with eg 100% resist, or more.
+         * So use the actual item in the cauldron, don't retry the recipe.
+         * This should fix bug #2020224: buggy(?) crafting yields.
+         *
         if (!rp)
             if ((rp = get_random_recipe((recipelist *)NULL)) == NULL)
                 return;
+         */
 
-        if ((tmp = attempt_recipe(op, cauldron, 1, rp, -1, 0))) {
+        if ((tmp = cauldron->inv)) /*attempt_recipe(op, cauldron, 1, rp, -1, 0)))*/ {
             if (!QUERY_FLAG(tmp, FLAG_CURSED)) { /* curse it */
                 SET_FLAG(tmp, FLAG_CURSED);
                 CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
@@ -592,6 +623,14 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
         }
         return;
     }
+#if 0
+    /*
+    Note: this does not work as expected...
+    At this point there is only one item in the cauldron, and get_formulalist(0) will return
+    the first formula list for recipes with 1 ingredient.
+    So disable this, and just use the next case.
+     */
+
     if (level == 40) {                   /* MAKE RANDOM RECIPE */
         recipelist *fl;
         int numb = numb_ob_inside(cauldron);
@@ -604,7 +643,9 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
             alchemy_failure_effect(op, cauldron, rp, level-1);
         return;
 
-    } else if (level < 45) {                 /* INFURIATE NPC's */
+    } else
+#endif
+        if (level < 45) {                 /* INFURIATE NPC's */
         /* this is kind of kludgy I know...*/
         object_set_enemy(cauldron, op);
         monster_npc_call_help(cauldron);
@@ -622,7 +663,6 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
             tmp->stats.dam = random_roll(1, level, op, PREFER_LOW);
             tmp->stats.hp = random_roll(1, level, op, PREFER_LOW);
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                 "The %s creates a bomb!",
                                  "The %s creates a bomb!",
                                  cauldron->name);
             break;
@@ -632,7 +672,6 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
             tmp->stats.dam = random_roll(1, level, op, PREFER_LOW)/5+1;
             tmp->stats.hp = random_roll(1, level, op, PREFER_LOW)/10+2;
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                 "The %s erupts in flame!",
                                  "The %s erupts in flame!",
                                  cauldron->name);
             break;
@@ -641,7 +680,7 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
         return;
     } else if (level < 60) {                 /* CREATE MONSTER */
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "The %s %s.", NULL, cauldron->name, cauldron_sound());
+                             "The %s %s.", cauldron->name, cauldron_sound());
         remove_contents(cauldron->inv, NULL);
         return;
     } else if (level < 80) {                 /* MAJOR FIRE */
@@ -651,7 +690,6 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
         fire_arch_from_position(cauldron, cauldron, cauldron->x, cauldron->y, 0, fb);
         object_free_drop_inventory(fb);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "The %s erupts in flame!",
                              "The %s erupts in flame!",
                              cauldron->name);
         return;
@@ -666,12 +704,10 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
         if (rndm(0, 1)) {
             remove_contents(cauldron->inv, NULL);
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                 "Your %s turns darker then makes a gulping sound!",
                                  "Your %s turns darker then makes a gulping sound!",
                                  cauldron->name);
         } else
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                 "Your %s becomes darker.",
                                  "Your %s becomes darker.",
                                  cauldron->name);
         return;
@@ -683,7 +719,6 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
             alchemy_failure_effect(op, cauldron, rp, level);
         else if (summon_hostile_monsters(cauldron, random_roll(1, 10, op, PREFER_LOW), tmp->arch->name))
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                 "The %s %s and then pours forth monsters!",
                                  "The %s %s and then pours forth monsters!",
                                  cauldron->name, cauldron_sound());
         return;
@@ -710,7 +745,6 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
                 draw_ext_info_format(NDI_UNIQUE, 0, op,
                                      MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
                                      "The %s %s.",
-                                     "The %s %s.",
                                      cauldron->name, cauldron_sound());
             }
         }
@@ -718,7 +752,7 @@ static void alchemy_failure_effect(object *op, object *cauldron, recipe *rp, int
     } else {                /* MANA STORM - watch out!! */
         object *tmp = create_archetype(LOOSE_MANA);
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                      "You unwisely release potent forces!", NULL);
+                      "You unwisely release potent forces!");
         remove_contents(cauldron->inv, NULL);
         cast_magic_storm(op, tmp, level);
         return;
@@ -751,7 +785,7 @@ static void remove_contents(object *first_ob, object *save_item) {
 /**
  *"Danger" level, will determine how bad the backfire
  * could be if the user fails to concoct a recipe properly. Factors include
- * the number of ingredients, the length of the name of each ingredient,
+ * the number of ingredients, the magical nature of ingredients,
  * the user's effective level, the user's Int and the enchantment on the
  * mixing device (aka "cauldron"). Higher values of 'danger' indicate more
  * danger. Note that we assume that we have had the caster ready the alchemy
@@ -767,9 +801,8 @@ static void remove_contents(object *first_ob, object *save_item) {
  * @return
  * danger value.
  */
-static int calc_alch_danger(object *caster, object *cauldron, recipe *rp) {
-    char name[MAX_BUF];
-    int danger = 0, nrofi = 0;
+static int calc_alch_danger(object *caster, object *cauldron, const recipe *rp) {
+    int danger = 0;
 
     /* Knowing alchemy skill reduces yer risk */
     danger -= caster->chosen_skill ? caster->chosen_skill->level : caster->level;
@@ -780,17 +813,14 @@ static int calc_alch_danger(object *caster, object *cauldron, recipe *rp) {
     /* Higher Int, lower the risk */
     danger -= 3*(caster->stats.Int-15);
 
-    /* Ingredients. Longer names usually mean rarer stuff.
-     * Thus the backfire is worse. Also, more ingredients
-     * means we are attempting a more powerfull potion,
-     * and thus the backfire will be worse.  */
+    /* Ingredients. */
     FOR_INV_PREPARE(cauldron, item) {
-        strcpy(name, item->name);
-        if (item->title)
-            snprintf(name, sizeof(name), "%s %s", item->name, item->title);
-        danger += (strtoint(name)/1000)+3;
-        nrofi++;
+        danger += item->weight / 100;
+        danger++;
+        if (QUERY_FLAG(item, FLAG_CURSED) || QUERY_FLAG(item, FLAG_DAMNED))
+            danger += 5;
     } FOR_INV_FINISH();
+
     if (rp == NULL)
         danger += 110;
     else
@@ -912,9 +942,9 @@ static int is_defined_recipe(const recipe *rp, const object *cauldron, object *c
  * @todo
  * document parameters.
  */
-static recipe *find_recipe(recipelist *fl, int formula, object *ingredients) {
-    recipe *rp;
-    recipe *result;       /* winning recipe, or NULL if no recipe found */
+static const recipe *find_recipe(const recipelist *fl, int formula, object *ingredients) {
+    const recipe *rp;
+    const recipe *result;       /* winning recipe, or NULL if no recipe found */
     int recipes_matching; /* total number of matching recipes so far */
     int transmute_found;  /* records whether a transmuting recipe was found so far */
     size_t rp_arch_index;
@@ -996,7 +1026,7 @@ int use_alchemy(object *op) {
     char name[MAX_BUF];
 
     if (QUERY_FLAG(op, FLAG_WIZ))
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM, "Note: alchemy in wizard-mode.\n", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM, "Note: alchemy in wizard-mode.\n");
 
     FOR_MAP_PREPARE(op->map, op->x, op->y, tmp) {
         if (QUERY_FLAG(tmp, FLAG_IS_CAULDRON)) {
@@ -1017,13 +1047,11 @@ int use_alchemy(object *op) {
     if (unpaid_cauldron) {
         query_base_name(unpaid_cauldron, 0, name, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                             "You must pay for your %s first!",
                              "You must pay for your %s first!",
                              name);
     } else if (unpaid_item) {
         query_base_name(unpaid_item, 0, name, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                             "You must pay for your %s first!",
                              "You must pay for your %s first!",
                              name);
     }
