@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_apply_c =
- *   "$Id: apply.c 12621 2010-03-13 09:35:43Z ryo_saeba $";
+ *   "$Id: apply.c 14015 2010-10-21 06:20:33Z mwedel $";
  */
 /*
   CrossFire, A Multiplayer game for X-windows
@@ -81,8 +81,13 @@ int transport_can_hold(const object *transport, const object *op, int nrof) {
  * @return
  * 1 if it should abort, 0 if it should continue.
  */
-int should_director_abort(object *op, object *victim) {
+int should_director_abort(const object *op, const object *victim) {
     int arch_flag, name_flag, race_flag;
+
+    /* Never move doors, it messes things. */
+    if (victim->type == DOOR)
+        return 1;
+
     /* Get flags to determine what of arch, name, and race should be
      * checked. This is stored in subtype, and is a bitmask, the LSB
      * is the arch flag, the next is the name flag, and the last is
@@ -242,25 +247,34 @@ int apply_container(object *op, object *sack) {
      * container.
      */
     if (op->container && QUERY_FLAG(sack, FLAG_APPLIED)) {
+        tag_t tmp_tag = op->container->count;
+
         if (op->container->env != op) { /* if container is on the ground */
             op->container->move_off = 0;
         }
+
+        /* Query name before the close event, as the container could be destroyed. */
+        query_name(op->container, name_tmp, MAX_BUF);
+
         /* Lauwenmark: Handle for plugin close event */
         if (execute_event(tmp, EVENT_CLOSE, op, NULL, NULL, SCRIPT_FIX_ALL) != 0)
             return 1;
 
-        query_name(op->container, name_tmp, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, op,
                              MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
                              "You close %s.",
-                             "You close %s.",
                              name_tmp);
-        CLEAR_FLAG(op->container, FLAG_APPLIED);
+
         op->container = NULL;
-        if (set_object_face_main(tmp))
-            esrv_update_item(UPD_FLAGS|UPD_FACE, op, tmp);
-        else
-            esrv_update_item(UPD_FLAGS, op, tmp);
+
+        /* The container may have been destroyed by the event handler. */
+        if (!object_was_destroyed(tmp, tmp_tag)) {
+            CLEAR_FLAG(tmp, FLAG_APPLIED);
+            if (set_object_face_main(tmp))
+                esrv_update_item(UPD_FLAGS|UPD_FACE, op, tmp);
+            else
+                esrv_update_item(UPD_FLAGS, op, tmp);
+        }
         if (tmp == sack)
             return 1;
     }
@@ -279,13 +293,11 @@ int apply_container(object *op, object *sack) {
             draw_ext_info_format(NDI_UNIQUE, 0, op,
                                  MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
                                  "You unlock %s with %s.",
-                                 "You unlock %s with %s.",
                                  name_sack, name_tmp);
         } else {
             draw_ext_info_format(NDI_UNIQUE, 0, op,
                                  MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
                                  "You don't have the key to unlock %s.",
-                                 "You don't have the key to unlock %s.",
                                  name_sack);
             return 0;
         }
@@ -311,7 +323,6 @@ int apply_container(object *op, object *sack) {
          */
         if (sack->env) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                                 "You can't open %s",
                                  "You can't open %s",
                                  name_sack);
             return 0;
@@ -330,7 +341,6 @@ int apply_container(object *op, object *sack) {
 
         CLEAR_FLAG(sack, FLAG_APPLIED);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                             "You open %s.",
                              "You open %s.",
                              name_sack);
         SET_FLAG(sack, FLAG_APPLIED);
@@ -344,7 +354,6 @@ int apply_container(object *op, object *sack) {
         if (QUERY_FLAG(sack, FLAG_APPLIED)) {  /* readied sack becoming open */
             CLEAR_FLAG(sack, FLAG_APPLIED);
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                 "You open %s.",
                                  "You open %s.",
                                  name_sack);
             SET_FLAG(sack, FLAG_APPLIED);
@@ -362,7 +371,6 @@ int apply_container(object *op, object *sack) {
 
             CLEAR_FLAG(sack, FLAG_APPLIED);
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                 "You readied %s.",
                                  "You readied %s.",
                                  name_sack);
             SET_FLAG(sack, FLAG_APPLIED);
@@ -415,7 +423,6 @@ void do_learn_spell(object *op, object *spell, int special_prayer) {
         SET_FLAG(tmp, FLAG_STARTEQUIP);
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                         "Type 'bind cast %s to store the spell in a key.",
                          "Type 'bind cast %s to store the spell in a key.",
                          spell->name);
 
@@ -444,7 +451,6 @@ void do_forget_spell(object *op, const char *spell) {
     }
 
     draw_ext_info_format(NDI_UNIQUE|NDI_NAVY, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_CURSED,
-                         "You lose knowledge of %s.",
                          "You lose knowledge of %s.",
                          spell);
     player_unready_range_ob(op->contr, spob);
@@ -484,7 +490,7 @@ static int apply_check_race_restrictions(object *who, object *item) {
         return 1;
 
     query_name(item, buf, sizeof(buf));
-    draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_PROHIBITION, "Somehow you can't seem to use the %s.", NULL, buf);
+    draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_PROHIBITION, "Somehow you can't seem to use the %s.", buf);
 
     return 0;
 }
@@ -515,7 +521,7 @@ int apply_manual(object *op, object *tmp, int aflag) {
     if (QUERY_FLAG(tmp, FLAG_UNPAID) && !QUERY_FLAG(tmp, FLAG_APPLIED)) {
         if (op->type == PLAYER) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                          "You should pay for it first.", NULL);
+                          "You should pay for it first.");
             return METHOD_SILENT_ERROR;
         }
         return 0;   /* monsters just skip unpaid items */
@@ -563,7 +569,7 @@ int apply_by_living(object *pl, object *op, int aflag, int quiet) {
         /* player is flying and applying object not in inventory */
         if (!QUERY_FLAG(pl, FLAG_WIZ) && !(op->move_type&MOVE_FLYING)) {
             draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                          "But you are floating high above the ground!", NULL);
+                          "But you are floating high above the ground!");
             return 0;
         }
     }
@@ -576,9 +582,9 @@ int apply_by_living(object *pl, object *op, int aflag, int quiet) {
     && !QUERY_FLAG(pl, FLAG_WAS_WIZ)) {
         play_sound_map(SOUND_TYPE_ITEM, op, 0, "evaporate");
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                      "The object disappears in a puff of smoke!", NULL);
+                      "The object disappears in a puff of smoke!");
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                      "It must have been an illusion.", NULL);
+                      "It must have been an illusion.");
         object_remove(op);
         object_free2(op, 0);
         return 1;
@@ -594,12 +600,11 @@ int apply_by_living(object *pl, object *op, int aflag, int quiet) {
 
             query_name(op, name, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                                 "I don't know how to apply the %s.",
                                  "I don't know how to apply the %s.",
                                  name);
         } else if (tmp == METHOD_ERROR)
             draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                                 "You must get it first!\n", NULL);
+                                 "You must get it first!\n");
         else if (tmp == METHOD_SILENT_ERROR)
             return tmp;
     }
@@ -687,7 +692,6 @@ static int unapply_special(object *who, object *op, int aflags) {
     case WEAPON:
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                 "You unwield %s.",
                                  "You unwield %s.",
                                  name);
         (void)change_abil(who, op);
@@ -706,13 +710,11 @@ static int unapply_special(object *who, object *op, int aflags) {
             if (!op->invisible) {
                 if (!(aflags&AP_NOPRINT))
                     draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                         "You stop using the %s.",
                                          "You stop using the %s.",
                                          name);
             } else {
                 if (!(aflags&AP_NOPRINT))
                     draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                         "You can no longer use the skill: %s.",
                                          "You can no longer use the skill: %s.",
                                          op->skill);
             }
@@ -734,7 +736,6 @@ static int unapply_special(object *who, object *op, int aflags) {
     case CLOAK:
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                 "You unwear %s.",
                                  "You unwear %s.",
                                  name);
         (void)change_abil(who, op);
@@ -746,7 +747,6 @@ static int unapply_special(object *who, object *op, int aflags) {
         clear_skill(who);
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                 "You unready %s.",
                                  "You unready %s.",
                                  name);
         if (who->type == PLAYER)
@@ -760,7 +760,6 @@ static int unapply_special(object *who, object *op, int aflags) {
     case BUILDER:
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                 "You unready %s.",
                                  "You unready %s.",
                                  name);
         who->contr->shoottype = range_none;
@@ -770,7 +769,6 @@ static int unapply_special(object *who, object *op, int aflags) {
     default:
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                 "You unapply %s.",
                                  "You unapply %s.",
                                  name);
         break;
@@ -870,7 +868,6 @@ static int unapply_for_ob(object *who, object *op, int aflags) {
                     if (!(aflags&AP_NOPRINT)) {
                         query_name(tmp, name, MAX_BUF);
                         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                             "No matter how hard you try, you just can't remove %s.",
                                              "No matter how hard you try, you just can't remove %s.",
                                              name);
                     }
@@ -880,7 +877,7 @@ static int unapply_for_ob(object *who, object *op, int aflags) {
                 if (aflags&AP_PRINT) {
                     query_name(tmp, name, MAX_BUF);
                     draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                  name, NULL);
+                                  name);
                 } else
                     unapply_special(who, tmp, aflags);
             }
@@ -907,7 +904,7 @@ static int unapply_for_ob(object *who, object *op, int aflags) {
                     if (aflags&AP_PRINT) {
                         query_name(tmp, name, MAX_BUF);
                         draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                      name, NULL);
+                                      name);
                     } else
                         unapply_special(who, tmp, aflags);
                 } else {
@@ -919,7 +916,6 @@ static int unapply_for_ob(object *who, object *op, int aflags) {
                     if (!(aflags&AP_NOPRINT)) {
                         query_name(tmp, name, MAX_BUF);
                         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                             "The %s just won't come off",
                                              "The %s just won't come off",
                                              name);
                     }
@@ -1068,46 +1064,9 @@ int apply_can_apply_object(const object *who, const object *op) {
  * improvement level.
  * @return
  * 1 if who can use the item, 0 else.
- * @todo
- * remove obsolete code.
- *
- * @note
- * check_weapon_power() has been renamed to apply_check_weapon_power()
  */
 int apply_check_weapon_power(const object *who, int improves) {
-    /* Old code is below (commented out).  Basically, since weapons
-     * are the only object players really have any control to improve,
-     * it's a bit harsh to require high level in some combat skill,
-     * so we just use overall level.
-     */
-#if 1
     return (who->level/5)+5 >= improves;
-#else
-    int level = 0;
-
-    /* The skill system hands out wc and dam bonuses to fighters
-     * more generously than the old system (see fix_object). Thus
-     * we need to curtail the power of player enchanted weapons.
-     * I changed this to 1 improvement per "fighter" level/5 -b.t.
-     * Note:  Nothing should break by allowing this ratio to be
-     * different or using normal level - it is just a matter of play
-     * balance.
-     */
-    if (who->type == PLAYER) {
-        FOR_INV_PREPARE(who, wc_obj)
-            if (wc_obj->type == SKILL && IS_COMBAT_SKILL(wc_obj->subtype) && wc_obj->level > level)
-                level = wc_obj->level;
-        FOR_INV_FINISH();
-
-        if (!level)  {
-            LOG(llevError, "Error: Player: %s lacks wc experience object\n", who->name);
-            level = who->level;
-        }
-    } else
-        level = who->level;
-
-    return improves <= (level/5)+5;
-#endif
 }
 
 /**
@@ -1155,7 +1114,6 @@ int apply_special(object *who, object *op, int aflags) {
             if (!(aflags&AP_NOPRINT)) {
                 query_name(op, name_op, MAX_BUF);
                 draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                                     "No matter how hard you try, you just can't remove %s.",
                                      "No matter how hard you try, you just can't remove %s.",
                                      name_op);
             }
@@ -1175,7 +1133,6 @@ int apply_special(object *who, object *op, int aflags) {
         if (!skop) {
             if (!(aflags&AP_NOPRINT))
                 draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                                     "You need the %s skill to use this item!",
                                      "You need the %s skill to use this item!",
                                      op->skill);
             if (who->type == PLAYER)
@@ -1208,8 +1165,7 @@ int apply_special(object *who, object *op, int aflags) {
         if (!apply_check_weapon_power(who, op->last_eat)) {
             if (!(aflags&AP_NOPRINT))
                 draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                    "That weapon is too powerful for you to use.  It would consume your soul!",
-                    NULL);
+                    "That weapon is too powerful for you to use.  It would consume your soul!");
             if (tmp != NULL)
                 (void)object_insert_in_ob(tmp, who);
             return 1;
@@ -1230,7 +1186,7 @@ int apply_special(object *who, object *op, int aflags) {
         if (!(aflags&AP_NOPRINT)) {
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                "You wield %s.", "You wield %s.",
+                "You wield %s.",
                 name_op);
         }
 
@@ -1251,7 +1207,6 @@ int apply_special(object *who, object *op, int aflags) {
         if (!(aflags&AP_NOPRINT)) {
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                 "You wear %s.",
                                  "You wear %s.",
                                  name_op);
         }
@@ -1277,8 +1232,7 @@ int apply_special(object *who, object *op, int aflags) {
         if (!apply_check_weapon_power(who, op->last_eat)) {
             if (!(aflags&AP_NOPRINT))
                 draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                    "That weapon is too powerful for you to use.  It would consume your soul!",
-                    NULL);
+                    "That weapon is too powerful for you to use.  It would consume your soul!");
             if (tmp != NULL)
                 (void)object_insert_in_ob(tmp, who);
             return 1;
@@ -1299,7 +1253,6 @@ int apply_special(object *who, object *op, int aflags) {
         if (!(aflags&AP_NOPRINT)) {
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                 "You ready %s.",
                                  "You ready %s.",
                                  name_op);
         }
@@ -1309,7 +1262,6 @@ int apply_special(object *who, object *op, int aflags) {
                 if (!(aflags&AP_NOPRINT)) {
                     query_name(op, name_op, MAX_BUF);
                     draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                         "You will now fire %s with %s.",
                                          "You will now fire %s with %s.",
                                          op->race ? op->race : "nothing",
                                          name_op);
@@ -1333,7 +1285,6 @@ int apply_special(object *who, object *op, int aflags) {
         if (!(aflags&AP_NOPRINT)) {
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                                 "You ready your %s.",
                                  "You ready your %s.",
                                  name_op);
         }
@@ -1342,7 +1293,6 @@ int apply_special(object *who, object *op, int aflags) {
     default:
         query_name(op, name_op, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                             "You apply %s.",
                              "You apply %s.",
                              name_op);
         break;
@@ -1365,7 +1315,7 @@ int apply_special(object *who, object *op, int aflags) {
     if (QUERY_FLAG(op, FLAG_CURSED) || QUERY_FLAG(op, FLAG_DAMNED)) {
         if (who->type == PLAYER) {
             draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_CURSED,
-                          "Oops, it feels deadly cold!", NULL);
+                          "Oops, it feels deadly cold!");
             SET_FLAG(op, FLAG_KNOWN_CURSED);
         }
     }
@@ -1575,7 +1525,7 @@ void scroll_failure(object *op, int failure, int power) {
         object *tmp;
 
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                      "Your spell warps!", NULL);
+                      "Your spell warps!");
         tmp = create_archetype(SPELL_WONDER);
         cast_wonder(op, op, 0, tmp);
         if (op->stats.sp < 0)
@@ -1588,21 +1538,21 @@ void scroll_failure(object *op, int failure, int power) {
     if (settings.spell_failure_effects == TRUE) {
         if (failure <= -35 && failure > -60) { /* confusion */
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                          "The magic recoils on you!", NULL);
+                          "The magic recoils on you!");
             confuse_living(op, op, power);
             return;
         }
 
         if (failure <= -60 && failure > -70) {/* paralysis */
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                          "The magic recoils and paralyzes you!", NULL);
+                          "The magic recoils and paralyzes you!");
             paralyze_living(op, op, power);
             return;
         }
 
         if (failure <= -70 && failure > -80) {/* blind */
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                          "The magic recoils on you!", NULL);
+                          "The magic recoils on you!");
             blind_living(op, op, power);
             return;
         }
@@ -1613,7 +1563,7 @@ void scroll_failure(object *op, int failure, int power) {
             tmp = create_archetype(LOOSE_MANA);
             cast_magic_storm(op, tmp, power);
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                          "You unlease uncontrolled mana!", NULL);
+                          "You unlease uncontrolled mana!");
             object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
             return;
         }
@@ -1622,7 +1572,7 @@ void scroll_failure(object *op, int failure, int power) {
      * in any case let's punish.
      */
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
-                  "Your mana is drained!", NULL);
+                  "Your mana is drained!");
     op->stats.sp -= random_roll(0, power-1, op, PREFER_LOW);
     if (op->stats.sp < 0)
         op->stats.sp = 0;
@@ -1634,92 +1584,120 @@ void scroll_failure(object *op, int failure, int power) {
  * object to change.
  * @param change
  * what kind of changes to apply. Should be of type CLASS.
+ * @param limit_stats
+ * If true, stats from class can not exceed racial maximum.
+ * if not true, then they can - this is used for point based
+ * character creation.
  */
-void apply_changes_to_player(object *pl, object *change) {
-    switch (change->type) {
-    case CLASS: {
-            int i, j;
-            int excess_stat = 0;  /* if the stat goes over the maximum
-                                   * for the race, put the excess stat some
-                                   * where else.
-                                   */
+void apply_changes_to_player(object *pl, object *change, int limit_stats) {
+    int i, j;
+    int excess_stat = 0;  /* if the stat goes over the maximum
+                           * for the race, put the excess stat some
+                           * where else.
+                           */
 
-            /* the following code assigns stats up to the stat max
-             * for the race, and if the stat max is exceeded,
-             * tries to randomly reassign the excess stat
-             */
-            for (i = 0; i < NUM_STATS; i++) {
-                sint8 stat = get_attr_value(&pl->contr->orig_stats, i);
-                int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
+    if (change->type != CLASS) return;
 
-                stat += get_attr_value(&change->stats, i);
-                if (stat > 20+race_bonus) {
-                    excess_stat++;
-                    stat = 20+race_bonus;
-                }
-                set_attr_value(&pl->contr->orig_stats, i, stat);
+    /* the following code assigns stats up to the stat max
+     * for the race, and if the stat max is exceeded,
+     * tries to randomly reassign the excess stat
+     */
+    for (i = 0; i < NUM_STATS; i++) {
+        sint8 stat = get_attr_value(&pl->contr->orig_stats, i);
+        int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
+
+        stat += get_attr_value(&change->stats, i);
+        if (limit_stats) {
+            if (stat > 20+race_bonus) {
+                excess_stat++;
+                stat = 20+race_bonus;
+            } else if (stat < 1) {
+                /* I didn't see any code here before to make sure minimum
+                 * stats were enforced - maybe it was just dumb
+                 * luck that one would not have a stat low enough that then
+                 * has a stat penalty for class that would bring it negative?
+                 * I imagine a negative stat would crash the server pretty
+                 * quickly - MSW, Sept 2010
+                 */
+                excess_stat += stat;
+                stat = 1;
             }
+        }
+        set_attr_value(&pl->contr->orig_stats, i, stat);
+    }
 
-            for (j = 0; excess_stat > 0 && j < 100; j++) {
-                /* try 100 times to assign excess stats */
-                int i = rndm(0, 6);
-                int stat = get_attr_value(&pl->contr->orig_stats, i);
-                int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
+    /* Maybe we should randomly deduct stats in this case?
+     * It's will go away sometime soon in any case.
+     */
+    if (excess_stat < 0) excess_stat = 0;
 
-                if (i == CHA)
-                    continue; /* exclude cha from this */
-                if (stat < 20+race_bonus) {
-                    change_attr_value(&pl->contr->orig_stats, i, 1);
-                    excess_stat--;
-                }
-            }
+    /* We don't put an explicit check for limit_stats here -
+     * excess stat will never be 0 if limit_stats is not
+     * true.
+     */
+    for (j = 0; excess_stat > 0 && j < 100; j++) {
+        /* try 100 times to assign excess stats */
+        int i = rndm(0, NUM_STATS-1);
+        int stat = get_attr_value(&pl->contr->orig_stats, i);
+        int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
 
-            /* insert the randomitems from the change's treasurelist into
-             * the player ref: player.c
-             */
-            if (change->randomitems != NULL)
-                give_initial_items(pl, change->randomitems);
-
-
-            /* set up the face, for some races. */
-
-            /* first, look for the force object banning changing the
-             * face.  Certain races never change face with class.
-             */
-            if (object_find_by_name(pl, "NOCLASSFACECHANGE") == NULL) {
-                pl->animation_id = GET_ANIM_ID(change);
-                pl->face = change->face;
-
-                if (QUERY_FLAG(change, FLAG_ANIMATE))
-                    SET_FLAG(pl, FLAG_ANIMATE);
-                else
-                    CLEAR_FLAG(pl, FLAG_ANIMATE);
-            }
-
-            if (change->anim_suffix) {
-                char buf[MAX_BUF];
-                int anim;
-
-                snprintf(buf, MAX_BUF, "%s_%s", animations[pl->animation_id].name, change->anim_suffix);
-                anim = try_find_animation(buf);
-                if (anim) {
-                    pl->animation_id = anim;
-                    pl->anim_speed = -1;
-                    CLEAR_FLAG(pl, FLAG_ANIMATE);
-                    animate_object(pl, pl->facing);
-                }
-            }
-
-            /* check the special case of can't use weapons */
-            /*if(QUERY_FLAG(change, FLAG_USE_WEAPON))
-             *    CLEAR_FLAG(pl, FLAG_USE_WEAPON);
-             */
-            if (!strcmp(change->name, "monk"))
-                CLEAR_FLAG(pl, FLAG_USE_WEAPON);
-
-            break;
+        if (i == CHA)
+            continue; /* exclude cha from this */
+        if (stat < 20+race_bonus) {
+            change_attr_value(&pl->contr->orig_stats, i, 1);
+            excess_stat--;
         }
     }
+
+    /* insert the randomitems from the change's treasurelist into
+     * the player ref: player.c
+     */
+    if (change->randomitems != NULL)
+        give_initial_items(pl, change->randomitems);
+
+
+    /* set up the face, for some races. */
+
+    /* first, look for the force object banning changing the
+     * face.  Certain races never change face with class.
+     */
+    if (object_find_by_name(pl, "NOCLASSFACECHANGE") == NULL) {
+        pl->animation_id = GET_ANIM_ID(change);
+        pl->face = change->face;
+
+        if (QUERY_FLAG(change, FLAG_ANIMATE))
+            SET_FLAG(pl, FLAG_ANIMATE);
+        else
+            CLEAR_FLAG(pl, FLAG_ANIMATE);
+    }
+
+    if (change->anim_suffix) {
+        char buf[MAX_BUF];
+        int anim;
+
+        snprintf(buf, MAX_BUF, "%s_%s", animations[pl->animation_id].name, change->anim_suffix);
+        anim = try_find_animation(buf);
+        if (anim) {
+            pl->animation_id = anim;
+            pl->anim_speed = -1;
+            CLEAR_FLAG(pl, FLAG_ANIMATE);
+            animate_object(pl, pl->facing);
+        }
+    }
+#if 0
+    /* check the special case of can't use weapons */
+    /*if(QUERY_FLAG(change, FLAG_USE_WEAPON))
+     *    CLEAR_FLAG(pl, FLAG_USE_WEAPON);
+     */
+    if (!strcmp(change->name, "monk"))
+        CLEAR_FLAG(pl, FLAG_USE_WEAPON);
+#endif
+    /* Hard coding in class name is a horrible idea - lets
+     * use the supported mechanism for this
+     */
+    if (object_present_in_ob_by_name(FORCE, "no weapon force", pl))
+        CLEAR_FLAG(pl, FLAG_USE_WEAPON);
+
 }
 
 void legacy_apply_container(object *op, object *sack) {
@@ -1734,6 +1712,8 @@ void legacy_apply_container(object *op, object *sack) {
  * the object applying the item
  * @param op
  * the item being applied
+ * @param aflags
+ * combinaison of @ref AP_xxx "AP_xxx" flags
  * @return
  * whether applying is possible
  */
@@ -1752,7 +1732,6 @@ static int apply_check_apply_restrictions(object *who, object *op, int aflags) {
 
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_BADBODY,
-                "You don't have the body to use a %s",
                 "You don't have the body to use a %s",
                 name_op);
         }
@@ -1765,7 +1744,6 @@ static int apply_check_apply_restrictions(object *who, object *op, int aflags) {
 
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_PROHIBITION,
-                "You have a prohibition against using a %s",
                 "You have a prohibition against using a %s",
                 name_op);
         }
@@ -1781,7 +1759,7 @@ static int apply_check_apply_restrictions(object *who, object *op, int aflags) {
     || (i&CAN_APPLY_UNAPPLY_CHOICE && who->contr->unapply == unapply_nochoice)) {
         if (!(aflags&AP_NOPRINT))
             draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
-                "You need to unapply some item(s):", NULL);
+                "You need to unapply some item(s):");
         unapply_for_ob(who, op, AP_PRINT);
         return 0;
     }
@@ -1801,6 +1779,8 @@ static int apply_check_apply_restrictions(object *who, object *op, int aflags) {
  * the object applying the item
  * @param op
  * the item being applied
+ * @param aflags
+ * combinaison of @ref AP_xxx "AP_xxx" flags
  * @return
  * whether applying is possible
  */
@@ -1814,7 +1794,7 @@ static int apply_check_item_power(const object *who, const object *op, int aflag
 
     if (!(aflags&AP_NOPRINT))
         draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-            "Equipping that combined with other items would consume your soul!", NULL);
+            "Equipping that combined with other items would consume your soul!");
     return 0;
 }
 
@@ -1853,7 +1833,6 @@ static int apply_check_personalized_blessings(object *who, const object *op) {
     item_will = will != NULL ? atol(will) : 0;
     if (item_will > who->stats.exp) {
         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-            "This %s refuses to serve you - it keeps evading your hand !",
             "This %s refuses to serve you - it keeps evading your hand !",
             op->name);
         return 0;
@@ -1882,14 +1861,13 @@ static int apply_check_personalized_blessings(object *who, const object *op) {
     }
     if (msg != NULL)
         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-            msg, msg, op->name);
+            msg, op->name);
     if (damage_percentile > 0) {
         int weapon_bite = (who->stats.hp*damage_percentile)/100;
         if (weapon_bite < 1)
             weapon_bite = 1;
         who->stats.hp -= weapon_bite;
         draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
-            "You get a nasty bite in the hand !",
             "You get a nasty bite in the hand !");
     }
 
@@ -1922,7 +1900,7 @@ static int apply_check_owner(const object *who, const object *op, int aflags) {
 
     if (!(aflags&AP_NOPRINT))
         draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-            "The weapon does not recognize you as its owner.", NULL);
+            "The weapon does not recognize you as its owner.");
     return 0;
 }
 
@@ -1946,7 +1924,6 @@ static void apply_update_ranged_skill(const object *who, object *op, int aflags)
     if (op->invisible) {
         if (!(aflags&AP_NOPRINT))
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                "Readied skill: %s.",
                 "Readied skill: %s.",
                 op->skill ? op->skill : op->name);
     } else {
@@ -1955,11 +1932,9 @@ static void apply_update_ranged_skill(const object *who, object *op, int aflags)
 
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                "You ready %s.",
                 "You ready %s.",
                 name_op);
             draw_ext_info_format(NDI_UNIQUE, 0, who, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                "You can now use the skill: %s.",
                 "You can now use the skill: %s.",
                 op->skill);
         }
