@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_attack_c =
- *   "$Id: attack.c 12951 2010-04-26 04:17:15Z mwedel $";
+ *   "$Id: attack.c 14118 2010-11-22 19:57:45Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -593,7 +593,7 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
                     play_sound_player_only(op->contr, SOUND_TYPE_HIT_BY, op, 0, "high");
             }
         }
-        draw_ext_info(NDI_BLACK, 0, op, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT, buf, NULL);
+        draw_ext_info(NDI_BLACK, 0, op, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT, buf);
     } /* end of player hitting player */
 
     /* scale down these messages too */
@@ -611,7 +611,7 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
                 play_sound_player_only(hitter->contr, SOUND_TYPE_HIT, hitter, 0, "high");
         }
         draw_ext_info(NDI_BLACK, 0, hitter, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
-                      buf, NULL);
+                      buf);
     } else if (owner != NULL && owner->type == PLAYER) {
         /* look for stacked spells and start reducing the message chances */
         if (hitter->type == SPELL_EFFECT
@@ -633,7 +633,6 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
             return;
         play_sound_map(SOUND_TYPE_HIT, owner, 0, "hit");
         draw_ext_info_format(NDI_BLACK, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_PET_HIT,
-                             "Your %s%s %s.",
                              "Your %s%s %s.",
                              hitter->name, buf2, op->name);
     }
@@ -776,17 +775,16 @@ static int attack_ob_simple(object *op, object *hitter, int base_dam, int base_w
             if (hitter->type == PLAYER && hitter->casting_time > -1) {
                 hitter->casting_time = -1;
                 draw_ext_info(NDI_UNIQUE, 0, hitter, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_FUMBLE,
-                              "You attacked and lost your spell!", NULL);
+                              "You attacked and lost your spell!");
             }
             if (op->casting_time > -1 && hitdam > 0) {
                 op->casting_time = -1;
                 if (op->type == PLAYER)  {
                     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_FUMBLE,
-                                  "You were hit and lost your spell!", NULL);
+                                  "You were hit and lost your spell!");
                     draw_ext_info_format(NDI_ALL|NDI_UNIQUE, 5, NULL,
                                          MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_FUMBLE,
                                          "%s was hit by %s and lost a spell.",
-                                         "%s was hit by %s and lost a spell.",
                                          op->name, hitter->name);
                 }
             }
@@ -809,8 +807,7 @@ static int attack_ob_simple(object *op, object *hitter, int base_dam, int base_w
                 make_visible(op);
                 if (op->type == PLAYER)
                     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
-                                  "You were hit by a wild attack. You are no longer hidden!",
-                                  NULL);
+                                  "You were hit by a wild attack. You are no longer hidden!");
             }
 
             /* thrown items (hitter) will have various effects
@@ -840,7 +837,7 @@ static int attack_ob_simple(object *op, object *hitter, int base_dam, int base_w
             && QUERY_FLAG(hitter, FLAG_ALIVE)) {
             if (op->attacktype&AT_ACID && hitter->type == PLAYER)
                 draw_ext_info(NDI_UNIQUE, 0, hitter, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
-                              "You are splashed by acid!\n", NULL);
+                              "You are splashed by acid!\n");
             hit_player(hitter, random_roll(0, (op->stats.dam), hitter, PREFER_LOW), op, op->attacktype, 1);
             if (object_was_destroyed(op, op_tag)
             || object_was_destroyed(hitter, hitter_tag)
@@ -1269,7 +1266,6 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
                             draw_ext_info_format(NDI_UNIQUE|NDI_RED, 0, op,
                                                  MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
                                                  "The %s's acid corrodes your %s!",
-                                                 "The %s's acid corrodes your %s!",
                                                  name_hitter, name_op);
                         }
                         flag = 1;
@@ -1467,21 +1463,19 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
  * damage done to it.
  * @param hitter
  * what is hitting it.
- * @param type
- * the attacktype.
  * @return
  * dealt damage.
  * @todo
  * finish commenting what it does exactly.
  */
-static int kill_object(object *op, int dam, object *hitter, int type) {
-    char buf[MAX_BUF];
+static int kill_object(object *op, int dam, object *hitter) {
+    char kill_message[MAX_BUF];
     const char *skill;
     int maxdam = 0;
     int battleg = 0;  /* true if op standing on battleground */
     int pk = 0;       /* true if op and what controls hitter are both players*/
     object *owner = NULL;
-    object *skop = NULL;
+    const object *skop = NULL;
     sstring death_animation;
 
     if (op->stats.hp >= 0)
@@ -1524,10 +1518,17 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
         remove_friendly_object(op);
         owner = object_get_owner(op);
         if (owner != NULL
-        && owner->type == PLAYER
-        && owner->contr->ranges[range_golem] == op) {
-            owner->contr->ranges[range_golem] = NULL;
-            owner->contr->golem_count = 0;
+        && owner->type == PLAYER) {
+            if (owner->contr->ranges[range_golem] == op) {
+                owner->contr->ranges[range_golem] = NULL;
+                owner->contr->golem_count = 0;
+            }
+
+            /*play_sound_player_only(owner1->contr, SOUND_PET_IS_KILLED, 0, 0);*/
+            /* Maybe we should include the owner that killed this, maybe not */
+            draw_ext_info_format(NDI_UNIQUE, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_PET_DIED,
+                                 "Your pet, the %s, is killed by %s.",
+                                 op->name, hitter->name);
         } else
             LOG(llevError, "BUG: hit_player(): Encountered golem without owner.\n");
 
@@ -1552,13 +1553,6 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
 
     /* Player killed something */
     if (owner->type == PLAYER) {
-        char name_op[MAX_BUF], name_hitter[MAX_BUF];
-
-        query_name(op, name_op, MAX_BUF);
-        if (hitter)
-            query_name(hitter, name_hitter, MAX_BUF);
-        else
-            name_hitter[0] = '\0';
 
         /* Log players killing other players - makes it easier to detect
          * and filter out malicious player killers - that is why the
@@ -1588,7 +1582,6 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
                 query_name(op, killed, MAX_BUF);
                 query_name(hitter, with, MAX_BUF);
                 draw_ext_info_format(NDI_BLACK, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_KILL,
-                                     "You killed %s with %s.",
                                      "You killed %s with %s.",
                                      killed, with);
             } else {
@@ -1596,7 +1589,6 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
 
                 query_name(op, killed, MAX_BUF);
                 draw_ext_info_format(NDI_BLACK, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_KILL,
-                                     "You killed %s.",
                                      "You killed %s.",
                                      killed);
             }
@@ -1660,7 +1652,7 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
 
         query_name(op, name_op, MAX_BUF);
         query_name(hitter, name_hitter, MAX_BUF);
-        snprintf(buf, sizeof(buf), "%s%s killed %s%s with %s%s.", owner_prefix, owner->name, op_prefix, name_op, name_hitter, battleg ? " (duel)" : (pk ? " (pk)" : ""));
+        snprintf(kill_message, sizeof(kill_message), "%s%s killed %s%s with %s%s.", owner_prefix, owner->name, op_prefix, name_op, name_hitter, battleg ? " (duel)" : (pk ? " (pk)" : ""));
     } else {
         const char *hitter_prefix;
         const char *op_prefix;
@@ -1668,7 +1660,7 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
         hitter_prefix = !battleg && pk && hitter->contr != NULL && !hitter->contr->peaceful ? "hostile " : "";
         op_prefix = !battleg && pk && op->contr != NULL && !op->contr->peaceful ? "hostile " : "";
 
-        snprintf(buf, sizeof(buf), "%s%s killed %s%s%s%s.", hitter_prefix, hitter->name, op_prefix, op->name,
+        snprintf(kill_message, sizeof(kill_message), "%s%s killed %s%s%s%s.", hitter_prefix, hitter->name, op_prefix, op->name,
                       (QUERY_FLAG(hitter, FLAG_MONSTER)) || hitter->type == PLAYER ?
                       " in hand to hand combat" : "", battleg ? " (duel)" : (pk ? " (pk)" : ""));
     }
@@ -1679,7 +1671,7 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
         skill = skop->skill;
 
     draw_ext_info(NDI_ALL, op->type == PLAYER ? 1 : 10, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
-                  buf, NULL);
+                  kill_message);
 
 
     /* If you didn't kill yourself, and your not the wizard */
@@ -1692,7 +1684,7 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
         if (op->type == PLAYER) {
             if (battleg) {
                 draw_ext_info(NDI_UNIQUE, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_KILL,
-                              "Your foe has fallen!\nVICTORY!!!", NULL);
+                              "Your foe has fallen!\nVICTORY!!!");
             } else {
                 exp = settings.pk_max_experience_percent*exp/100;
                 if (settings.pk_max_experience >= 0)
@@ -1729,19 +1721,6 @@ static int kill_object(object *op, int dam, object *hitter, int type) {
     } /* end if person didn't kill himself */
 
     if (op->type != PLAYER) {
-        if (QUERY_FLAG(op, FLAG_FRIENDLY)) {
-            object *owner1 = object_get_owner(op);
-
-            if (owner1 != NULL && owner1->type == PLAYER) {
-                /*play_sound_player_only(owner1->contr, SOUND_PET_IS_KILLED, 0, 0);*/
-                /* Maybe we should include the owner that killed this, maybe not */
-                draw_ext_info_format(NDI_UNIQUE, 0, owner1, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_PET_DIED,
-                                     "Your pet, the %s, is killed by %s.",
-                                     "Your pet, the %s, is killed by %s.",
-                                     op->name, hitter->name);
-            }
-            remove_friendly_object(op);
-        }
         object_remove(op);
         object_free_drop_inventory(op);
     /* Player has been killed! */
@@ -2034,7 +2013,7 @@ int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
     }
 
     /* See if the creature has been killed */
-    rtn_kill = kill_object(op, maxdam, hitter, type);
+    rtn_kill = kill_object(op, maxdam, hitter);
     if (rtn_kill != -1)
         return rtn_kill;
 
@@ -2114,13 +2093,12 @@ static void poison_living(object *op, object *hitter, int dam) {
                 fix_object(op);
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
-                              "You suddenly feel very ill.", NULL);
+                              "You suddenly feel very ill.");
             }
             if (hitter->type == PLAYER)
                 draw_ext_info_format(NDI_UNIQUE, 0, hitter,
                                      MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
                                      "You poison %s.",
-                                     "You poison %s.",
                                      op->name);
             else {
                 object *owner;
@@ -2130,7 +2108,6 @@ static void poison_living(object *op, object *hitter, int dam) {
                     draw_ext_info_format(NDI_UNIQUE, 0, owner,
                         MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_PET_HIT,
                         "Your %s poisons %s.",
-                        "Your %s poisons %s.",
                         hitter->name, op->name);
             }
             tmp->speed_left = 0;
@@ -2161,7 +2138,7 @@ static void slow_living(object *op, object *hitter, int dam) {
         tmp = arch_to_object(at);
         tmp = object_insert_in_ob(tmp, op);
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
-                      "The world suddenly moves very fast!", NULL);
+                      "The world suddenly moves very fast!");
     } else
         tmp->stats.food++;
     SET_FLAG(tmp, FLAG_APPLIED);
@@ -2204,7 +2181,7 @@ void confuse_living(object *op, object *hitter, int dam) {
 
     if (op->type == PLAYER && !QUERY_FLAG(op, FLAG_CONFUSED))
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
-                      "You suddenly feel very confused!", NULL);
+                      "You suddenly feel very confused!");
     SET_FLAG(op, FLAG_CONFUSED);
 }
 
@@ -2246,7 +2223,6 @@ void blind_living(object *op, object *hitter, int dam) {
 
         query_name(op, victim, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
-                             "Your attack blinds %s!",
                              "Your attack blinds %s!",
                              victim);
     }
@@ -2441,8 +2417,8 @@ static int adj_attackroll(object *hitter, object *target) {
         /* target is unseen */
         if (target->invisible || QUERY_FLAG(attacker, FLAG_BLIND))
             adjust -= 10;
-        /* dark map penalty for the hitter (lacks infravision if we got here). */
-        else if (target->map && target->map->darkness > 0 && !monster_stand_in_light(target))
+        /* dark map penalty for the hitter, though xray can help for a player */
+        else if (target->map && target->map->darkness > 0 && !monster_stand_in_light(target) && (hitter->type != PLAYER || !player_can_view(hitter, target)))
             adjust -= target->map->darkness;
     }
 
