@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_party_c =
- *   "$Id: c_party.c 12316 2009-12-20 18:34:13Z akirschbaum $";
+ *   "$Id: c_party.c 13594 2010-08-14 10:16:12Z ryo_saeba $";
  */
 
 /*
@@ -64,14 +64,14 @@ void receive_party_password(object *op) {
     if (confirm_party_password(op) == 0) {
         party_join(op, op->contr->party_to_join);
         op->contr->party_to_join = NULL;
-        op->contr->state = ST_PLAYING;
+        player_set_state(op->contr, ST_PLAYING);
         return;
     }
 
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                  "You entered the wrong password", NULL);
+                  "You entered the wrong password");
     op->contr->party_to_join = NULL;
-    op->contr->state = ST_PLAYING;
+    player_set_state(op->contr, ST_PLAYING);
 }
 
 /**
@@ -81,20 +81,17 @@ void receive_party_password(object *op) {
  * player.
  * @param params
  * message.
- * @return
- * 0.
  */
-int command_gsay(object *op, char *params) {
+void command_gsay(object *op, const char *params) {
     char party_params[MAX_BUF];
 
     if (*params == '\0') {
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR, "Say what?", NULL);
-        return 0;
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR, "Say what?");
+        return;
     }
     strcpy(party_params, "say ");
     strcat(party_params, params);
     command_party(op, party_params);
-    return 0;
 }
 
 /**
@@ -117,7 +114,7 @@ static void party_help(object *op) {
 #ifdef PARTY_KILL_LOG
                   "To see what you've killed, type: party kills"
 #endif
-                  , NULL);
+                 );
 }
 
 /**
@@ -130,25 +127,24 @@ static void party_help(object *op) {
  * 1.
  * @todo split in different functions. clean the 'form' mess.
  */
-int command_party(object *op, char *params) {
+void command_party(object *op, const char *params) {
     char buf[MAX_BUF];
 
     if (*params == '\0') {
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
                           "You are not a member of any party. "
-                          "For help try: party help", NULL);
+                          "For help try: party help");
         } else {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                                 "You are a member of party %s.",
                                  "You are a member of party %s.",
                                  op->contr->party->partyname);
         }
-        return 1;
+        return;
     }
     if (strcmp(params, "help") == 0) {
         party_help(op);
-        return 1;
+        return;
     }
 #ifdef PARTY_KILL_LOG
     if (!strncmp(params, "kills", 5)) {
@@ -159,22 +155,21 @@ int command_party(object *op, char *params) {
 
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "You are not a member of any party.", NULL);
-            return 1;
+                          "You are not a member of any party.");
+            return;
         }
         tmpparty = op->contr->party;
         if (!tmpparty->kills) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                          "You haven't killed anything yet.", NULL);
-            return 1;
+                          "You haven't killed anything yet.");
+            return;
         }
         max = tmpparty->kills-1;
         if (max > PARTY_KILL_LOG-1)
             max = PARTY_KILL_LOG-1;
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                       "[fixed]Killed          |          Killer|     Exp\n----------------+----------------+--------"
-                      "Killed          |          Killer|     Exp\n----------------+----------------+--------",
-                      NULL);
+                      "Killed          |          Killer|     Exp\n----------------+----------------+--------");
 
 
         for (i = 0; i <= max; i++) {
@@ -190,7 +185,6 @@ int command_party(object *op, char *params) {
 
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                                  "[fixed]%16s|%16s|%6.1f%c",
-                                 "%16s|%16s|%6.1f%c",
                                  tmpparty->party_kills[i].dead,
                                  tmpparty->party_kills[i].killer, exp, chr);
         }
@@ -205,10 +199,8 @@ int command_party(object *op, char *params) {
         }
 
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                      "[fixed]----------------+----------------+--------",
-                      "----------------+----------------+--------");
+                      "[fixed]----------------+----------------+--------");
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                             "Totals: %d kills, %.1f%c exp",
                              "Totals: %d kills, %.1f%c exp", tmpparty->kills,
                              exp, chr);
         return 1;
@@ -217,17 +209,16 @@ int command_party(object *op, char *params) {
     if (strncmp(params, "say ", 4) == 0) {
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "You are not a member of any party.", NULL);
-            return 1;
+                          "You are not a member of any party.");
+            return;
         }
         params += 4;
         snprintf(buf, MAX_BUF-1, "<%s> %s says: %s", op->contr->party->partyname, op->name, params);
         party_send_message(op, buf);
         draw_ext_info_format(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_PARTY,
-                             "<%s> You say: %s",
                              "<%s> You say: %s",
                              op->contr->party->partyname, params);
-        return 1;
+        return;
     }
 
     if (strncmp(params, "form ", 5) == 0) {
@@ -235,31 +226,30 @@ int command_party(object *op, char *params) {
 
         if (party_form(op, params) == NULL) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                                 "The party %s already exists, pick another name",
                                  "The party %s already exists, pick another name",
                                  params);
-            return 1;
+            return;
         }
-        return 0;
+        return;
     } /* form */
 
     if (strcmp(params, "leave") == 0) {
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "You are not a member of any party.", NULL);
-            return 1;
+                          "You are not a member of any party.");
+            return;
         }
         party_leave(op);
-        return 1;
+        return;
     }
     if (strcmp(params, "who") == 0) {
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "You are not a member of any party.", NULL);
-            return 1;
+                          "You are not a member of any party.");
+            return;
         }
         list_players(op, NULL, op->contr->party);
-        return 1;
+        return;
     } /* leave */
 
     if (strncmp(params, "passwd ", 7) == 0) {
@@ -267,26 +257,25 @@ int command_party(object *op, char *params) {
 
         if (op->contr->party == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "You are not a member of a party", NULL);
-            return 1;
+                          "You are not a member of a party");
+            return;
         }
 
         if (strlen(params) > 8) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "The password must not exceed 8 characters", NULL);
-            return 1;
+                          "The password must not exceed 8 characters");
+            return;
         }
 
         party_set_password(op->contr->party, params);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                             "The password for party %s is %s",
                              "The password for party %s is %s",
                              op->contr->party->partyname, party_get_password(op->contr->party));
 
         snprintf(buf, MAX_BUF, "Password for party %s is now %s, changed by %s",
                  op->contr->party->partyname, party_get_password(op->contr->party), op->name);
         party_send_message(op, buf);
-        return 0;
+        return;
     } /* passwd */
 
     if (strcmp(params, "list") == 0) {
@@ -294,22 +283,19 @@ int command_party(object *op, char *params) {
 
         if (party_get_first() == NULL) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "There are no parties active right now", NULL);
-            return 1;
+                          "There are no parties active right now");
+            return;
         }
 
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                      "[fixed]Party name                       Leader\n----------                       ------",
-                      "Party name                       Leader\n----------                       ------");
-
+                      "[fixed]Party name                       Leader\n----------                       ------");
         for (party = party_get_first(); party != NULL; party = party_get_next(party)) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                                  "[fixed]%-32s %s",
-                                 "%-32s %s",
                                  party->partyname, party_get_leader(party));
         }
 
-        return 0;
+        return;
     } /* list */
 
     if (strncmp(params, "join ", 5) == 0) {
@@ -320,30 +306,27 @@ int command_party(object *op, char *params) {
         party = party_find(params);
         if (party == NULL) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                                 "Party %s does not exist.  You must form it first.",
                                  "Party %s does not exist.  You must form it first.",
                                  params);
-            return 1;
+            return;
         }
 
         if (op->contr->party == party) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                                 "You are already a member of party: %s",
                                  "You are already a member of party: %s",
                                  party->partyname);
-            return 1;
+            return;
         }
 
         if (get_party_password(op, party)) {
-            return 0;
+            return;
         }
 
         party_join(op, party);
-        return 0;
+        return;
     } /* join */
 
     party_help(op);
-    return 1;
 }
 
 /** Valid modes for 'party_rejoin'. @todo document that */
@@ -360,28 +343,23 @@ static const char *rejoin_modes[] = {
  * player.
  * @param params
  * optional parameters.
- * @return
- * 1.
  */
-int command_party_rejoin(object *op, char *params) {
+void command_party_rejoin(object *op, const char *params) {
     int mode;
 
     if (*params == '\0') {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                             "party rejoin: %s", NULL, rejoin_modes[op->contr->rejoin_party]);
-        return 1;
+                             "party rejoin: %s", rejoin_modes[op->contr->rejoin_party]);
+        return;
     }
     for (mode = 0; rejoin_modes[mode] != NULL; mode++) {
         if (strcmp(rejoin_modes[mode], params) == 0) {
             op->contr->rejoin_party = mode;
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                                 "party rejoin is now: %s", NULL, rejoin_modes[op->contr->rejoin_party]);
-            return 1;
+                                 "party rejoin is now: %s", rejoin_modes[op->contr->rejoin_party]);
+            return;
         }
     }
-    if (strlen(params) > 50)
-        params[50] = '\0';
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                         "invalid mode: %s", NULL, params);
-    return 1;
+                         "invalid mode: %50s", params);
 }
