@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c 12945 2010-04-24 18:12:08Z kbulgrien $";
+ *   "$Id: init.c 14252 2011-01-27 05:43:40Z mwedel $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002,2010 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2002,2010-2011 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -285,6 +285,7 @@ static struct Command_Line_Options options[] = {
     { "-m9", 0, 3, set_dumpmon9 },
     { "-mt", 1, 3, set_dumpmont },
     { "-mexp", 0, 3, dump_experience },
+    { "-mq", 0, 3, dump_quests },
     { "-s", 0, 3, showscores },
     { "-score", 1, 3, showscoresparm }
 };
@@ -862,6 +863,44 @@ static void load_settings(void) {
         } else if (!strcasecmp(buf, "log_timestamp_format")) {
             free(settings.log_timestamp_format);
             settings.log_timestamp_format = strdup_local(cp);
+        } else if (!strcasecmp(buf, "starting_stat_min")) {
+            int val = atoi(cp);
+
+            if (val < 1 || val > MAX_STAT || val > settings.starting_stat_max)
+                LOG(llevError, "load_settings: starting_stat_min (%d) need to be within %d-%d (%d)\n",
+                    val, 1, MAX_STAT, settings.starting_stat_max);
+            else
+                settings.starting_stat_min = val;
+        } else if (!strcasecmp(buf, "starting_stat_max")) {
+            int val = atoi(cp);
+
+            if (val < 1 || val > MAX_STAT || val<settings.starting_stat_min)
+                LOG(llevError, "load_settings: starting_stat_max (%d) need to be within %d-%d (%d)\n",
+                    val, 1, MAX_STAT, settings.starting_stat_min);
+            else
+                settings.starting_stat_max = val;
+        } else if (!strcasecmp(buf, "starting_stat_points")) {
+            int val = atoi(cp);
+
+            if (val < NUM_STATS * settings.starting_stat_min ||
+                val > NUM_STATS * settings.starting_stat_max)
+                LOG(llevError, "load_settings: starting_stat_points (%d) need to be within %d-%d\n",
+                    val, NUM_STATS * settings.starting_stat_min, NUM_STATS * settings.starting_stat_max);
+            else
+                settings.starting_stat_points = val;
+        } else if (!strcasecmp(buf, "roll_stat_points")) {
+            int val = atoi(cp);
+
+            /* The 3 and 18 values are hard coded in because we know that
+             * roll_stat() generates a value between 3 and 18 - if that ever
+             * changed, this code should change also, but that code will eventually
+             * go away.
+             */
+            if (val < NUM_STATS * 3 || val > NUM_STATS * 18)
+                LOG(llevError, "load_settings: roll_stat_points need to be within %d-%d\n",
+                    NUM_STATS * 3,  NUM_STATS * 18);
+            else
+                settings.roll_stat_points = val;
         } else {
             LOG(llevError, "Unknown value in settings file: %s\n", buf);
         }
@@ -899,7 +938,7 @@ void init(int argc, char **argv) {
     load_materials();
     parse_args(argc, argv, 2);
     fprintf(logfile, "Welcome to CrossFire, v%s\n", FULL_VERSION);
-    fprintf(logfile, "Copyright (C) 1994 Mark Wedel.\n");
+    fprintf(logfile, "Copyright (C) 1994,2011 Mark Wedel & Crossfire Development Team\n");
     fprintf(logfile, "Copyright (C) 1992 Frank Tore Johansen.\n");
 
     if (strcmp(settings.dm_mail, "") != 0) {
@@ -926,7 +965,6 @@ void init(int argc, char **argv) {
 
     init_beforeplay();
     init_server();
-    metaserver_init();
     metaserver2_init();
     account_load_entries();
     reset_sleep();
@@ -942,6 +980,7 @@ void free_server(void) {
     free_materials();
     free_races();
     free_quest();
+    free_quest_definitions();
 }
 
 static void usage(void) {
@@ -987,6 +1026,8 @@ static void help(void) {
     printf(" -m8          Dumps out gods information.\n");
     printf(" -m9          Dumps out more alchemy information (formula checking).\n");
     printf(" -mt <name>   Dumps out list of treasures for a monster.\n");
+    printf(" -mexp        Dumps out the experience table.\n");
+    printf(" -mq          Dumps out the list of defined quests.\n");
     exit(0);
 }
 
@@ -1080,25 +1121,13 @@ static void compile_info(void) {
     char err[MAX_BUF];
 
     printf("Non-standard include files:\n");
-#if !defined(__STRICT_ANSI__) || defined(__sun__)
-#if !defined(Mips)
-    printf("<stdlib.h>\n");
-    i = 1;
-#endif
-#if !defined(MACH) && !defined(sony)
+#ifdef HAVE_MALLOC_H
     printf("<malloc.h>\n");
     i = 1;
 #endif
-#endif
-#ifndef __STRICT_ANSI__
-#ifndef MACH
+#ifdef HAVE_MEMORY_H
     printf("<memory.h\n");
     i = 1;
-#endif
-#endif
-#ifndef sgi
-    printf("<sys/timeb.h>\n");
-    i = 1;
 #endif
     if (!i)
         printf("(none)\n");
