@@ -65,14 +65,15 @@ During loading, obsolete items (eg formula changed) are discarded.
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
+#include <assert.h>
 
 struct knowledge_player;
 struct knowledge_type;
 
-/** Function to display the short description of an item. */
-typedef void (*knowledge_summary)(object *, const char *, int);
-/** Function to display a detailed description of an item. */
-typedef void (*knowledge_detail)(object *, const char *);
+/** Function to fill the StringBuffer with the short description of an item. */
+typedef void (*knowledge_summary)(const char *, StringBuffer *);
+/** Function to fill the StringBuffer with a detailed description of an item. */
+typedef void (*knowledge_detail)(const char *, StringBuffer *);
 /** Function to check if the specified item is valid. */
 typedef int (*knowledge_is_valid_item)(const char *);
 /** Function to add the specified item, should return how many actually written. */
@@ -84,6 +85,7 @@ typedef struct knowledge_type {
     knowledge_detail detail;            /**< Display the detailed description. */
     knowledge_is_valid_item validate;   /**< Validate the specific value. */
     knowledge_add_item add;             /**< Add the item to the knowledge store. */
+    const char *name;                   /**< Type name for player, to use with 'list'. */
 } knowledge_type;
 
 
@@ -144,12 +146,11 @@ static const recipe* knowledge_alchemy_get_recipe(const char *value) {
 
 /**
  * Give the title of the alchemy recpie.
- * @param pl player to give description for.
  * @param value recipe internal value.
- * @param index recipe index in the display list.
+ * @param buf where to put the information.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_alchemy_summary(object *pl, const char *value, int index) {
+static void knowledge_alchemy_summary(const char *value, StringBuffer *buf) {
     const recipe *rec = knowledge_alchemy_get_recipe(value);
     const archetype *arch;
 
@@ -163,26 +164,28 @@ static void knowledge_alchemy_summary(object *pl, const char *value, int index)
         return;
 
     if (strcmp(rec->title, "NONE"))
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s of %s", NULL, index, arch->clone.name, rec->title);
+        stringbuffer_append_printf(buf, "%s of %s", arch->clone.name, rec->title);
     else {
         if (arch->clone.title != NULL) {
-            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s %s", NULL, index, arch->clone.name, arch->clone.title);
+            stringbuffer_append_printf(buf, "%s %s", arch->clone.name, arch->clone.title);
         }
         else
-            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s", NULL, index, arch->clone.name);
+            stringbuffer_append_printf(buf, "%s", arch->clone.name);
     }
 }
 
 /**
  * Give the full description of the alchemy recpie.
- * @param pl player to give description for.
  * @param value recipe internal value.
+ * @param buf where to store the detail.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_alchemy_detail(object *pl, const char *value) {
+static void knowledge_alchemy_detail(const char *value, StringBuffer *buf) {
     const recipe *rec = knowledge_alchemy_get_recipe(value);
     const linked_char *next;
     const archetype *arch;
+    char name[MAX_BUF];
+
 
     if (!rec)
         /* warn? */
@@ -194,17 +197,25 @@ static void knowledge_alchemy_detail(object *pl, const char *value) {
         return;
 
     if (strcmp(rec->title, "NONE"))
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Recipe for %s of %s:", NULL, arch->clone.name, rec->title);
+        stringbuffer_append_printf(buf, "The %s of %s", arch->clone.name, rec->title);
     else {
         if (arch->clone.title != NULL) {
-            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Recipe for %s %s", NULL, arch->clone.name, arch->clone.title);
+            stringbuffer_append_printf(buf, "The %s %s", arch->clone.name, arch->clone.title);
         }
         else
-            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Recipe for %s", NULL, arch->clone.name);
+            stringbuffer_append_printf(buf, "The %s", arch->clone.name);
     }
 
+    arch = find_archetype(rec->cauldron);
+    if (arch)
+        query_name(&arch->clone, name, MAX_BUF);
+    else
+        snprintf(name, sizeof(name), "an unknown place");
+
+    stringbuffer_append_printf(buf, " is made at %s and uses the following ingredients:", name);
+
     for (next = rec->ingred; next != NULL; next = next->next) {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, " - %s", NULL, next->name);
+        stringbuffer_append_printf(buf, "\n - %s", next->name);
     }
 
 }
@@ -218,6 +229,26 @@ static int knowledge_achemy_validate(const char *item) {
     return knowledge_alchemy_get_recipe(item) != NULL;
 }
 
+/**
+ * Check whether a player already knows a knowledge item or not.
+ * @param current player to check.
+ * @param item what to check for.
+ * @param kt the knowledge type for item.
+ * @return 0 if item is known, 1 else.
+ */
+static int knowledge_known(const knowledge_player *current, const char *item, const knowledge_type *kt) {
+    const knowledge_item *check = current->items;
+
+    while (check) {
+        if (strcmp(kt->type, check->handler->type) == 0 && strcmp(item, check->item) == 0) {
+            /* already known, bailout */
+            return 1;
+        }
+        check = check->next;
+    }
+    return 0;
+}
+
 /**
  * Add a knowledge item to a player's store if not found yet.
  * @param current where to look for the knowledge.
@@ -226,14 +257,10 @@ static int knowledge_achemy_validate(const char *item) {
  * @return 0 if item was already known, 1 else.
  */
 static int knowledge_add(knowledge_player *current, const char *item, const knowledge_type *kt) {
-    knowledge_item *check = current->items;
+    knowledge_item *check;
 
-    while (check) {
-        if (strcmp(kt->type, check->handler->type) == 0 && strcmp(item, check->item) == 0) {
-            /* already known, bailout */
-            return 0;
-        }
-        check = check->next;
+    if (knowledge_known(current, item, kt)) {
+        return 0;
     }
 
     /* keep the knowledge */
@@ -247,35 +274,32 @@ static int knowledge_add(knowledge_player *current, const char *item, const know
 
 /**
  * Monster information summary.
- * @param pl who to give the information to.
  * @param item knowledge item.
- * @param index number of the item to display.
+ * @param buf where to put the information.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_monster_summary(object *pl, const char *item, int index) {
+static void knowledge_monster_summary(const char *item, StringBuffer *buf) {
     archetype *monster = find_archetype(item);
     if (!monster)
         return;
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s", NULL, index, monster->clone.name);
+    stringbuffer_append_printf(buf, "%s", monster->clone.name);
 }
 
 /**
  * Describe in detail a monster.
- * @param pl who to describe for.
  * @param item knowledge item for the monster (archetype name).
+ * @param buf where to put the description.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_monster_detail(object *pl, const char *item) {
-    char buf[HUGE_BUF];
+static void knowledge_monster_detail(const char *item, StringBuffer *buf) {
     archetype *monster = find_archetype(item);
 
     if (!monster)
         return;
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, " *** %s ***", NULL, monster->clone.name);
-    describe_item(&monster->clone, NULL, buf, sizeof(buf));
-    draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, buf, NULL);
+    stringbuffer_append_printf(buf, " *** %s ***\n", monster->clone.name);
+    describe_item(&monster->clone, NULL, buf);
 }
 
 /**
@@ -313,31 +337,29 @@ static int knowledge_monster_add(struct knowledge_player *current, const char *i
 
 /**
  * God information summary.
- * @param pl who to give the information to.
  * @param item knowledge item.
- * @param index number of the item to display.
+ * @param buf where to put the information.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_god_summary(object *pl, const char *item, int index) {
+static void knowledge_god_summary(const char *item, StringBuffer *buf) {
     char *dup = strdup_local(item), *pos = strchr(dup, ':');
 
     if (pos)
         *pos = '\0';
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s [god]", NULL, index, dup);
+    stringbuffer_append_printf(buf, "%s [god]", dup);
     free(dup);
 }
 
 /**
  * Describe in detail a god.
- * @param pl who to describe for.
  * @param item knowledge item for the god (object name and what is known).
+ * @param buf where to put the description.
  * @todo merge with stuff in readable.c
  */
-static void knowledge_god_detail(object *pl, const char *item) {
+static void knowledge_god_detail(const char *item, StringBuffer *buf) {
     char *dup = strdup_local(item), *pos = strchr(dup, ':'), *final;
     const archetype *god;
-    StringBuffer *buf;
     int what;
 
     if (!pos) {
@@ -356,11 +378,7 @@ static void knowledge_god_detail(object *pl, const char *item) {
         return;
     }
 
-    buf = stringbuffer_new();
     describe_god(&god->clone, what, buf, 0);
-    final = stringbuffer_finish(buf);
-    draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, final, NULL);
-    free(final);
 }
 
 /**
@@ -432,9 +450,9 @@ static int knowledge_god_add(struct knowledge_player *current, const char *item,
 
 /** All handled knowledge items. */
 static const knowledge_type const knowledges[] = {
-    { "alchemy", knowledge_alchemy_summary, knowledge_alchemy_detail, knowledge_achemy_validate, knowledge_add },
-    { "monster", knowledge_monster_summary, knowledge_monster_detail, knowledge_monster_validate, knowledge_monster_add },
-    { "god", knowledge_god_summary, knowledge_god_detail, knowledge_god_validate, knowledge_god_add },
+    { "alchemy", knowledge_alchemy_summary, knowledge_alchemy_detail, knowledge_achemy_validate, knowledge_add, "recipes" },
+    { "monster", knowledge_monster_summary, knowledge_monster_detail, knowledge_monster_validate, knowledge_monster_add, "monsters" },
+    { "god", knowledge_god_summary, knowledge_god_detail, knowledge_god_validate, knowledge_god_add, "gods" },
     { NULL, 0, 0, 0, 0 }
 };
 
@@ -470,7 +488,7 @@ static void knowledge_write_player_data(const knowledge_player *kp) {
     file = fopen(write, "w+");
     if (!file) {
         LOG(llevError, "knowledge: couldn't open player knowledge file %s!", write);
-        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!", NULL);
+        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
         return;
     }
 
@@ -541,7 +559,7 @@ static void knowledge_read_player_data(knowledge_player *kp) {
  * @param pl who to find data for.
  * @return data store, never NULL.
  */
-static knowledge_player *knowledge_get_or_create(player *pl) {
+static knowledge_player *knowledge_get_or_create(const player *pl) {
     knowledge_player *cur = knowledge_global;
 
     while (cur) {
@@ -600,39 +618,111 @@ void knowledge_read(player *pl, object *book) {
     free(copy);
 
     if (added) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You write that down for future reference.", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You write that down for future reference.");
         if (none) {
             /* first information ever written down, be nice and give hint to recover it. */
-            draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Use the 'knowledge' command to browse what you write down (this message will not appear anymore).", NULL);
+            draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Use the 'knowledge' command to browse what you write down (this message will not appear anymore).");
         }
     }
     knowledge_write_player_data(current);
 }
 
 /**
- * Display all a player's knowledge.
- * @param pl who to display knowledge of.
+ * Actually display knowledge list.
+ * @param pl who to display to.
+ * @param show_only if not NULL, only display this knowledge type.
+ * @param search if not NULL, only display items having the summary or detail contain the string. Must be either NULL or a non empty string.
  */
-static void knowledge_display(object *pl) {
+static void knowledge_do_display(object *pl, const knowledge_type *show_only, const char *search) {
     knowledge_player *kp;
     knowledge_item *item;
-    int index = 1, header = 0;
+    int index = 1, header = 0, show;
+    StringBuffer *summary;
+    char *final;
+
+    assert(search == NULL || search[0] != '\0');
 
     kp = knowledge_get_or_create(pl->contr);
     item = kp->items;
     while (item) {
-        if (header == 0) {
-            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You know this information:", NULL);
-            header = 1;
+        show = 1;
+
+        summary = stringbuffer_new();
+        item->handler->summary(item->item, summary);
+        final = stringbuffer_finish(summary);
+
+        if (show_only != NULL && item->handler != show_only) {
+            show = 0;
+        }
+        if (search != NULL && search[0] != '\0') {
+            if (strstr(final, search) == NULL) {
+                char *fd;
+                StringBuffer *detail = stringbuffer_new();
+                item->handler->detail(item->item, detail);
+                fd = stringbuffer_finish(detail);
+                if (strstr(fd, search) == NULL)
+                    show = 0;
+                free(fd);
+            }
         }
 
-        item->handler->summary(pl, item->item, index);
+        if (show == 1) {
+            if (header == 0) {
+                if (search != NULL)
+                    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You have knowledge of the following %s concerning '%s':", show_only == NULL ? "things" : show_only->type, search);
+                else if (show_only != NULL)
+                    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You have knowledge of those %s:", show_only->name);
+                else
+                    draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You have knowledge of:");
+                header = 1;
+            }
+
+            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s", index, final);
+        }
+
+        free(final);
         item = item->next;
         index++;
     }
 
-    if (header == 0)
-        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You don't know yet any relevant information.", NULL);
+    if (header == 0) {
+        if (search != NULL)
+            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You don't know yet any relevant information about '%s'.", search);
+        else if (show_only != NULL)
+            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You don't know yet any relevant information about %s.", show_only->name);
+        else
+            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "You don't know yet any relevant information.");
+    }
+}
+
+/**
+ * Display all a player's knowledge.
+ * @param pl who to display knowledge of.
+ * @param params additionnal parameters.
+ */
+static void knowledge_display(object *pl, const char *params) {
+    const knowledge_type *show_only = NULL;
+
+    if (params && params[0] == ' ') {
+        const char *type = params + 1;
+        int idx = 0;
+        for (; knowledges[idx].type != NULL; idx++) {
+            if (strcmp(knowledges[idx].name, type) == 0) {
+                show_only = &knowledges[idx];
+                break;
+            }
+        }
+
+        if (show_only == NULL) {
+            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "Invalid type, valid types are:");
+            for (idx = 0; knowledges[idx].type != NULL; idx++) {
+                draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "- %s", knowledges[idx].name);
+            }
+            return;
+        }
+    }
+
+    knowledge_do_display(pl, show_only, NULL);
 }
 
 /**
@@ -646,7 +736,7 @@ static void knowledge_show(object *pl, const char *params) {
     int count = atoi(params);
 
     if (count <= 0) {
-        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid knowledge number", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid knowledge number");
         return;
     }
 
@@ -654,46 +744,55 @@ static void knowledge_show(object *pl, const char *params) {
     item = kp->items;
     while (item) {
         if (count == 1) {
-            item->handler->detail(pl, item->item);
+            StringBuffer *buf = stringbuffer_new();
+            char *final;
+
+            item->handler->detail(item->item, buf);
+            final = stringbuffer_finish(buf);
+            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, final);
+            free(final);
             return;
         }
         item = item->next;
         count--;
     }
 
-    draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid knowledge number", NULL);
+    draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid knowledge number");
 }
 
 /**
  * Handle the 'knowledge' for a player.
  * @param pl who is using the command.
  * @param params additional parameters.
- * @return 0
  */
-int command_knowledge(object *pl, char *params) {
+void command_knowledge(object *pl, const char *params) {
 
     if (!pl->contr) {
         LOG(llevError, "command_knowledge: called for %s not a player!\n", pl->name);
-        return 0;
+        return;
     }
 
     if (!params || *params == '\0') {
         command_help(pl, "knowledge");
-        return 0;
+        return;
     }
 
-    if (strcmp(params, "list") == 0) {
-        knowledge_display(pl);
-        return 0;
+    if (strncmp(params, "list", 4) == 0) {
+        knowledge_display(pl, params + 4);
+        return;
+    }
+
+    if (strncmp(params, "search ", 7) == 0) {
+        knowledge_do_display(pl, NULL, params + 7);
+        return;
     }
 
     if (strncmp(params, "show ", 5) == 0) {
         knowledge_show(pl, params + 5);
-        return 0;
+        return;
     }
 
     command_help(pl, "knowledge");
-    return 0;
 }
 
 /**
@@ -728,3 +827,34 @@ void free_knowledge(void) {
     }
     knowledge_global = NULL;
 }
+
+int knowledge_player_knows(const player *pl, const char *knowledge) {
+    const knowledge_type *type;
+    char copy[MAX_BUF], *pos;
+    const knowledge_player *current;
+
+    if (strlen(knowledge) >= MAX_BUF - 1) {
+        LOG(llevError, "knowledge_player_knows: too long knowledge %s\n", knowledge);
+        return 0;
+    }
+
+    strncpy(copy, knowledge, MAX_BUF);
+    pos = strchr(copy, ':');
+    if (pos == NULL) {
+        LOG(llevError, "knowledge_player_knows: invalid knowledge item %s\n", knowledge);
+        return 0;
+    }
+
+    *pos = '\0';
+    pos++;
+
+    type = knowledge_find(copy);
+    if (type == NULL) {
+        LOG(llevError, "knowledge_player_knows: invalid knowledge type %s\n", knowledge);
+        return 0;
+    }
+
+    current = knowledge_get_or_create(pl);
+
+    return knowledge_known(current, pos, type);
+}
