@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c 12770 2010-03-31 03:20:56Z kbulgrien $";
+ *    "$Id: monster.c 14069 2010-11-06 17:59:53Z ryo_saeba $";
  */
 
 /*
@@ -62,7 +62,7 @@ static void monster_pace_moveh(object *ob);
 static void monster_pace2_movev(object *ob);
 static void monster_pace2_moveh(object *ob);
 static void monster_rand_move(object *ob);
-static int monster_talk_to_npc(object *op, object *npc, const char *txt, int *talked);
+static int monster_talk_to_npc(object *npc, talk_info *info);
 
 #define MIN_MON_RADIUS 3 /* minimum monster detection radius */
 
@@ -864,10 +864,10 @@ int monster_move(object *op) {
          */
         if (QUERY_FLAG(op, FLAG_RUN_AWAY)) {
             part->stats.wc += 10;
-            (void)skill_attack(enemy, part, 0, NULL, NULL);
+            skill_attack(enemy, part, 0, NULL, NULL);
             part->stats.wc -= 10;
         } else
-            (void)skill_attack(enemy, part, 0, NULL, NULL);
+            skill_attack(enemy, part, 0, NULL, NULL);
     } /* if monster is in attack range */
 
     if (QUERY_FLAG(part, FLAG_FREED))   /* Might be freed by ghost-attack or hit-back */
@@ -1536,13 +1536,15 @@ static void monster_check_pickup(object *monster) {
         } FOR_BELOW_FINISH();
 }
 
-/*
- * monster_can_pick(): If the monster is interested in picking up
- * the item, then return 0.  Otherwise 0.
- * Instead of pick_up, flags for "greed", etc, should be used.
- * I've already utilized flags for bows, wands, rings, etc, etc. -Frank.
+/**
+ * Check if the monster can and is interested in picking up
+ * an item.
+ * @param monster monster to check.
+ * @param item what item to check against
+ * @return
+ * - 0 if item can or should not be picked by monster.
+ * - 1 if monster picks item.
  */
-
 static int monster_can_pick(object *monster, object *item) {
     int flag = 0;
     int i;
@@ -1618,11 +1620,11 @@ static int monster_can_pick(object *monster, object *item) {
     return 0;
 }
 
-/*
- * monster_apply_below():
- * Vick's (vick@bern.docs.uu.se) @921107 -> If a monster who's
- * eager to apply things, encounters something apply-able,
- * then make him apply it
+/**
+ * If a monster who's eager to apply things encounters something apply-able,
+ * then make him apply it.
+ * @author Vick's (vick@bern.docs.uu.se)
+ * @date 921107
  */
 static void monster_apply_below(object *monster) {
     FOR_BELOW_PREPARE(monster, tmp) {
@@ -1653,18 +1655,13 @@ void monster_check_apply_all(object *monster) {
     FOR_INV_FINISH();
 }
 
-/*
- * monster_check_apply() is meant to be called after an item is
- * inserted in a monster.
- * If an item becomes outdated (monster found a better item),
- * a pointer to that object is returned, so it can be dropped.
- * (so that other monsters can pick it up and use it)
- * Note that as things are now, monsters never drop something -
- * they can pick up all that they can use.
+/**
+ * Called after an item is inserted in a monster.
+ * Will look if item should be applied to replace another item.
+ * @param mon monster who picked an item.
+ * @param item what was picked up.
+ * @note Sept 96, fixed this so skills will be readied -b.t.
  */
-
-/* Sept 96, fixed this so skills will be readied -b.t.*/
-
 void monster_check_apply(object *mon, object *item) {
     int flag = 0;
 
@@ -1764,6 +1761,8 @@ void monster_check_apply(object *mon, object *item) {
 }
 
 /**
+ * A monster calls for help against its enemy.
+ * @param op monster calling for help.
  * @note
  * npc_call_help() has been renamed to monster_npc_call_help()
  */
@@ -2001,9 +2000,78 @@ void monster_check_doors(object *op, mapstruct *m, int x, int y) {
     FOR_MAP_FINISH();
 }
 
+/**
+ * Output a NPC message on a map.
+ * @param map where to talk to.
+ * @param message what to say.
+ */
+void monster_do_say(const mapstruct *map, const char *message) {
+    ext_info_map(NDI_NAVY|NDI_UNIQUE, map, MSG_TYPE_DIALOG, MSG_TYPE_DIALOG_NPC,
+                 message);
+}
+
+/**
+ * Format an NPC message.
+ * @param npc who is talking.
+ * @param message what is being said.
+ * @return new StringBuffer containing the text.
+ */
+StringBuffer *monster_format_say(const object* npc, const char *message) {
+    char name[MAX_BUF];
+    StringBuffer *buf;
+
+    query_name(npc, name, sizeof(name));
+    buf = stringbuffer_new();
+    stringbuffer_append_printf(buf, "%s says: %s", name, message);
+    return buf;
+}
+
+/**
+ * Return the verb for the player's dialog type.
+ * @param rt dialog type.
+ * @return verb.
+ */
+static const char *get_reply_text_own(reply_type rt) {
+    switch (rt) {
+        case rt_say:
+            return "say";
+        case rt_reply:
+            return "reply";
+        case rt_question:
+            return "ask";
+    }
+    assert(0);
+    return NULL;
+}
+
+/**
+ * Return the verb for the player's dialog type seen from others (third person).
+ * @param rt dialog type.
+ * @return verb.
+ */
+static const char *get_reply_text_other(reply_type rt) {
+    switch (rt) {
+        case rt_say:
+            return "says";
+        case rt_reply:
+            return "replies";
+        case rt_question:
+            return "asks";
+    }
+    assert(0);
+    return NULL;
+}
+
 /**
  * This function looks for an object or creature that is listening to said text.
  *
+ * The process is such:
+ * - first, build up information on NPCs reacting to what is said, replies
+ * - second, figure what the player will be displayed as said
+ * - third, have the player actually talk
+ * - fourth, the NPCs talk too
+ * - fifth: show the player available replies
+ *
  * There is a rare event that the orig_map is used for - basically, if
  * a player says the magic word that gets him teleported off the map,
  * it can result in the new map putting the object count too high,
@@ -2018,20 +2086,19 @@ void monster_check_doors(object *op, mapstruct *m, int x, int y) {
  *
  * @param op who is saying something.
  * @param txt what is said.
- *
- * @note
- * communicate() has been renamed to monster_communicate()
  */
 void monster_communicate(object *op, const char *txt) {
-    int i, mflags, talked = 0;
+    int i, mflags;
     sint16 x, y;
     mapstruct *mp, *orig_map = op->map;
-    char buf[MAX_BUF];
+    char own[MAX_BUF], others[MAX_BUF];
+    talk_info info;
 
-    snprintf(buf, sizeof(buf), "%s says: %s", op->name, txt);
-    if (op->type == PLAYER) {
-        ext_info_map(NDI_WHITE, op->map, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, buf, NULL);
-    }
+    info.text = txt;
+    info.message = NULL;
+    info.replies_count = 0;
+    info.who = op;
+    info.npc_msg_count = 0;
 
     /* Note that this loop looks pretty inefficient to me - we look and try to talk
      * to every object within 2 spaces.  It would seem that if we trim this down to
@@ -2049,7 +2116,7 @@ void monster_communicate(object *op, const char *txt) {
             continue;
 
         FOR_MAP_PREPARE(mp, x, y, npc) {
-            monster_talk_to_npc(op, npc, txt, &talked);
+            monster_talk_to_npc(npc, &info);
             if (orig_map != op->map) {
                 LOG(llevDebug, "Warning: Forced to swap out very recent map - MAX_OBJECTS should probably be increased\n");
                 return;
@@ -2057,70 +2124,77 @@ void monster_communicate(object *op, const char *txt) {
         } FOR_MAP_FINISH();
     }
 
-    /* if talked is set, then the monster_talk_to_npc() wrote out this information, so
-     * don't do it again.
-     */
-    if (!talked) {
+    /* First, what the player says. */
+    if (info.message != NULL) {
+        snprintf(own, sizeof(own), "You %s: %s", get_reply_text_own(info.message_type), info.message);
+        snprintf(others, sizeof(others), "%s %s: %s", op->name, get_reply_text_other(info.message_type), info.message);
+        free_string(info.message);
+    } else {
+        snprintf(own, sizeof(own), "You say: %s", txt);
+        snprintf(others, sizeof(others), "%s says: %s", op->name, txt);
+    }
+    draw_ext_info(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, own);
+    ext_info_map_except(NDI_WHITE, op->map, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, others);
+
+    /* Then NPCs can actually talk. */
+    for (i = 0; i < info.npc_msg_count; i++) {
+        monster_do_say(orig_map, info.npc_msgs[i]);
+        free_string(info.npc_msgs[i]);
+    }
+
+    /* Finally, the replies the player can use. */
+    if (info.replies_count > 0) {
+        draw_ext_info(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, "Replies:");
+        for (i = 0; i < info.replies_count; i++) {
+            draw_ext_info_format(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, " - %s: %s", info.replies_words[i], info.replies[i]);
+            free_string(info.replies_words[i]);
+            free_string(info.replies[i]);
+        }
     }
 }
 
 /**
  * Checks the messages of a NPC for a matching text. Will not call
  * plugin events. Called by monster_talk_to_npc().
- * @param op who is saying something
- * @param npc object that gets a chance to reply.
- * @param txt text being said.
- * @param talked did op already talk? Will be modified if this function makes op talk.
- * @return 1 if npc talked, 0 else.
  *
- * @note
- * do_talk_npc() has been renamed to monster_do_talk_npc()
+ * @param npc object that gets a chance to reply.
+ * @param info message information.
+ * @return 1 if npc talked, 0 else.
  */
-static int monster_do_talk_npc(object *op, object *npc, const char *txt, int *talked) {
-    char buf[MAX_BUF];
+static int monster_do_talk_npc(object *npc, talk_info *info) {
     struct_dialog_reply *reply;
     struct_dialog_message *message;
 
-    if (!get_dialog_message(npc, txt, &message, &reply))
+    if (!get_dialog_message(npc, info->text, &message, &reply))
         return 0;
 
     if (reply) {
-        snprintf(buf, sizeof(buf), "%s %s: %s", op->name, (reply->type == rt_reply ? "replies" : "asks"), reply->message);
-        ext_info_map(NDI_WHITE, op->map, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, buf, NULL);
-        *talked = 1;
+        info->message = add_string(reply->message);
+        info->message_type = reply->type;
     }
-#if 0
-    /* let the caller handle this reply - no reason we need to.  Leaving this in for the
-     * time being, as I don't completely understand what all of this is trying to do.
-     * MSW 2009-04-14
-     */
-
-    else if (!*talked) {
-        *talked = 1;
-        snprintf(buf, sizeof(buf), "%s says: %s", op->name, txt);
-        ext_info_map(NDI_WHITE, op->map, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, buf, NULL);
-    }
-#endif
 
     if (npc->type == MAGIC_EAR) {
-        ext_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, MSG_TYPE_DIALOG, MSG_TYPE_DIALOG_MAGIC_EAR, message->message, NULL);
+        ext_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, MSG_TYPE_DIALOG, MSG_TYPE_DIALOG_MAGIC_EAR, message->message);
         use_trigger(npc);
     } else {
         char value[2];
 
-        monster_npc_say(npc, message->message);
-        reply = message->replies;
+        if (info->npc_msg_count < MAX_NPC) {
+            info->npc_msgs[info->npc_msg_count] = stringbuffer_finish_shared(monster_format_say(npc, message->message));
+            info->npc_msg_count++;
+        }
+
         /* mark that the npc was talked to, so it won't move randomly later on */
         value[0] = '3' + rand() % 6;
         value[1] = '\0';
         object_set_value(npc, "talked_to", value, 1);
 
-        if (reply) {
-            draw_ext_info(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, "Replies:", NULL);
-            while (reply) {
-                draw_ext_info_format(NDI_WHITE, 0, op, MSG_TYPE_COMMUNICATION, MSG_TYPE_COMMUNICATION_SAY, " - %s: %s", NULL, reply->reply, reply->message);
-                reply = reply->next;
-            }
+        reply = message->replies;
+        while (reply && info->replies_count < MAX_REPLIES) {
+            info->replies[info->replies_count] = add_string(reply->message);
+            info->replies_words[info->replies_count] = add_string(reply->reply);
+            info->replies_count++;
+            reply = reply->next;
         }
     }
 
@@ -2131,47 +2205,39 @@ static int monster_do_talk_npc(object *op, object *npc, const char *txt, int *ta
  * Simple function to have some NPC say something.
  * @param npc who should say something.
  * @param cp what is being said.
- *
- * @note
- * npc_say() has been renamed to monster_npc_say()
  */
 void monster_npc_say(object *npc, const char *cp) {
-    char buf[HUGE_BUF], name[MAX_BUF];
+    char *message;
+    StringBuffer *buf = monster_format_say(npc, cp);
 
-    query_name(npc, name, sizeof(name));
-    snprintf(buf, sizeof(buf), "%s says: %s", name, cp);
-    ext_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, MSG_TYPE_DIALOG, MSG_TYPE_DIALOG_NPC,
-                 buf, buf);
+    message = stringbuffer_finish(buf);
+    monster_do_say(npc->map, message);
+    free(message);
 }
 
 /**
  * Give an object the chance to handle something being said.
  * Plugin hooks will be called, including in the NPC's inventory.
  *
- * @param op who is talking.
  * @param npc object to try to talk to. Can be an NPC or a MAGIC_EAR.
- * @param txt what op is saying.
- * @param talked did op already talk? Can be modified by this function.
+ * @param info message information.
  * @return 0 if text was handled by a plugin or not handled, 1 if handled internally by the server.
- *
- * @note
- * talk_to_npc() has been renamed to monster_talk_to_npc()
  */
-static int monster_talk_to_npc(object *op, object *npc, const char *txt, int *talked) {
+static int monster_talk_to_npc(object *npc, talk_info *info) {
     /* Move this commone area up here - shouldn't cost much extra cpu
      * time, and makes the function more readable */
     /* Lauwenmark: Handle for plugin say event */
-    if (execute_event(npc, EVENT_SAY, op, NULL, txt, SCRIPT_FIX_ALL) != 0)
+    if (plugin_event_say(npc, info) != 0)
         return 0;
     /* Lauwenmark - Here we let the objects inside inventories hear and answer, too. */
     /* This allows the existence of "intelligent" weapons you can discuss with */
     FOR_INV_PREPARE(npc, cobj)
-        if (execute_event(cobj, EVENT_SAY, op, NULL, txt, SCRIPT_FIX_ALL) != 0)
+        if (plugin_event_say(cobj, info) != 0)
             return 0;
     FOR_INV_FINISH();
-    if (op == npc)
+    if (info->who == npc)
         return 0;
-    return monster_do_talk_npc(op, npc, txt, talked);
+    return monster_do_talk_npc(npc, info);
 }
 
 /* monster_find_throw_ob() - modeled on find_throw_ob
@@ -2322,7 +2388,6 @@ int monster_can_detect_enemy(object *op, object *enemy, rv_vector *rv) {
             /* inform players of new status */
             if (enemy->type == PLAYER && player_can_view(enemy, op))
                 draw_ext_info_format(NDI_UNIQUE, 0, enemy, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                     "You are discovered by %s!",
                                      "You are discovered by %s!",
                                      op->name);
             return 1; /* detected enemy */
@@ -2340,7 +2405,6 @@ int monster_can_detect_enemy(object *op, object *enemy, rv_vector *rv) {
 
                     query_name(op, name, MAX_BUF);
                     draw_ext_info_format(NDI_UNIQUE, 0, enemy, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                                         "You see %s noticing your position.",
                                          "You see %s noticing your position.",
                                          name);
                 }
@@ -2424,8 +2488,7 @@ int monster_can_see_enemy(object *op, object *enemy) {
             if (enemy->hide) {
                 make_visible(enemy);
                 draw_ext_info(NDI_UNIQUE, 0, enemy, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                              "Your light reveals your hiding spot!",
-                              NULL);
+                              "Your light reveals your hiding spot!");
             }
             return 1;
         } else if (enemy->hide)
