@@ -64,12 +64,22 @@ typedef struct quest_player {
 /** Player quest state. */
 static quest_player *player_states = NULL;
 
+typedef struct quest_condition {
+    sstring quest_code;          /**< The quest that triggers the condition */
+    int minstep;                  /**< The earliest step in the quest that triggers the condition,
+                                    -1 means finished, 0 means not started */
+    int maxstep;                  /**< The latest step that triggers the condition, to match,
+                                        the stages must be between minstep and maxstep */
+    struct quest_condition *next; /**< The next condition to check */
+} quest_condition;
+
 /** One step of a quest. */
 typedef struct quest_step_definition {
     int step;                           /**< Step identifier. */
     sstring step_description;           /**< Step description to show player. */
     int is_completion_step:1;           /**< Whether this step completes the quest (1) or not (0) */
     struct quest_step_definition *next; /**< Next step. */
+    quest_condition *conditions;        /**< The conditions that must be satisfied to trigger the step */
 } quest_step_definition;
 
 /** Definition of an in-game quest. */
@@ -79,6 +89,7 @@ typedef struct quest_definition {
     sstring quest_description;      /**< Quest longer description. */
     int quest_restart;              /**< If non zero, can be restarted. */
     quest_step_definition *steps;   /**< Quest steps. */
+    struct quest_definition *parent;/**< Parent for this quest, NULL if it is a 'top-level' quest */
     struct quest_definition *next;  /**< Next quest in the definition list. */
 } quest_definition;
 
@@ -96,6 +107,18 @@ static quest_step_definition *quest_create_step(void) {
     return step;
 }
 
+/**
+ * Allocate a quest_condition, will call fatal() if out of memory.
+ * @return new structure.
+ */
+static quest_condition *quest_create_condition(void) {
+    quest_condition *cond = calloc(1, sizeof(quest_condition));
+    if (!cond)
+        fatal(OUT_OF_MEMORY);
+    return cond;
+}
+
+
 /**
  * Allocate a quest_definition, will call fatal() if out of memory.
  * @return new structure.
@@ -107,35 +130,113 @@ static quest_definition *quest_create_definition(void) {
     return quest;
 }
 
-/** Load all quest definitions. Can be called multiple times, will be ignored. */
-static void quest_load_definitions(void) {
-    int found = 0, in = 0; /* 0: quest file, 1: one quest, 2: quest description, 3: quest step, 4: step description */
-    int i;
+/**
+ * Find a quest from its code. This is called by quest_get and also
+ * used in the quest loading code
+ * @param code quest to search.
+ * @return quest, or NULL if no such quest.
+ */
+static quest_definition *quest_get_by_code(sstring code) {
+    quest_definition *quest;
+
+    quest = quests;
+    while (quest) {
+        if (quest->quest_code == code)
+            return quest;
+
+        quest = quest->next;
+    }
+    return NULL;
+}
+
+#define QUESTFILE_NEXTQUEST 0
+#define QUESTFILE_QUEST 1
+#define QUESTFILE_QUESTDESC 2
+#define QUESTFILE_STEP 3
+#define QUESTFILE_STEPDESC 4
+#define QUESTFILE_STEPCOND 5
+
+/**
+ * Loads all of the quests which are found in the given file, any global states
+ * for quest loading are passed down into this function, but not back up again.
+ * @param filename filename to load quests from.
+ * @return number of quests loaded from file, negative value if there was an error.
+ */
+static int load_quests_from_file(const char *filename) {
+    int i, in = QUESTFILE_NEXTQUEST, condition_parsed;
+    int minstep, maxstep;
+    char namedquest[MAX_BUF];
     quest_definition *quest = NULL;
+    quest_condition *cond = NULL;
+    char includefile[MAX_BUF];
+    sstring questname;
     quest_step_definition *step = NULL;
     char final[MAX_BUF], read[MAX_BUF];
     FILE *file;
     StringBuffer *buf;
 
-    if (quests_loaded)
-        return;
-
-    quests_loaded = 1;
-
-    snprintf(final, sizeof(final), "%s/%s/default.quests", settings.datadir, settings.mapdir);
-
+    int loaded_quests =0, found =0;
+    snprintf(final, sizeof(final), "%s/%s/%s", settings.datadir, settings.mapdir, filename);
     file = fopen(final, "r");
     if (!file) {
-        /* no quest defined yet, no big deal */
-        return;
+        LOG(llevError, "Can't open %s for reading quests", filename);
+        return -1;
     }
 
     while (fgets(read, sizeof(read), file) != NULL) {
-        if (in == 4) {
+        if (in == QUESTFILE_STEPCOND) {
+            if (strcmp(read, "end_setwhen\n") == 0) {
+                in = QUESTFILE_STEP;
+                continue;
+            }
+            /* we are reading in a list of conditions for the 'setwhen' block for a quest step
+             * There will be one entry per line, containing the quest, and the steps that it applies to.
+             * This may be expressed as one of the following
+             * questcode 20 (the quest questcode must be at step 20)
+             * questcode <=20 (the quest questcode must not be beyond step 20)
+             * questcode 10-20 (the quest questcode must be between steps 10 and 20)
+             * questcode finished (the quest questcode must have been completed)
+             */
+
+            minstep = 0;
+            maxstep = 0;
+            condition_parsed = 0;
+            namedquest[0]='\0';
+            if (sscanf(read, "%s %d-%d\n", namedquest, &minstep, &maxstep)!=3) {
+                if (sscanf(read, "%s <=%d\n", namedquest, &maxstep)== 2) {
+                    minstep=0;
+                    condition_parsed =1;
+                } else if (sscanf(read, "%s %d\n", namedquest, &minstep)==2) {
+                    maxstep = minstep;
+                    condition_parsed =1;
+                } else if (strstr(read, "finished")) {
+                    if (sscanf(read, "%s finished\n", namedquest)==1) {
+                        minstep = maxstep = -1;
+                        condition_parsed =1;
+                    }
+                }
+            } else
+                condition_parsed =1;
+            if (!condition_parsed) {
+                LOG(llevError, "Invalid line '%s' in setwhen block for quest %s", read, quest->quest_code);
+                continue;
+            }
+
+            cond = quest_create_condition();
+            cond->minstep = minstep;
+            cond->maxstep = maxstep;
+            cond->quest_code = add_string(namedquest);
+            cond->next = step->conditions;
+            step->conditions = cond;
+            LOG(llevDebug, "condition added for step %d of quest %s, looking for quest %s between steps %d and %d\n",
+                    step->step, quest->quest_code, cond->quest_code, cond->minstep, cond->maxstep);
+            continue;
+        }
+        if (in == QUESTFILE_STEPDESC) {
             if (strcmp(read, "end_description\n") == 0) {
                 char *message;
 
-                in = 3;
+                in = QUESTFILE_STEP;
 
                 message = stringbuffer_finish(buf);
                 buf = NULL;
@@ -150,10 +251,10 @@ static void quest_load_definitions(void) {
             continue;
         }
 
-        if (in == 3) {
+        if (in == QUESTFILE_STEP) {
             if (strcmp(read, "end_step\n") == 0) {
                 step = NULL;
-                in = 1;
+                in = QUESTFILE_QUEST;
                 continue;
             }
             if (strcmp(read, "finishes_quest\n") == 0) {
@@ -162,18 +263,23 @@ static void quest_load_definitions(void) {
             }
             if (strcmp(read, "description\n") == 0) {
                 buf = stringbuffer_new();
-                in = 4;
+                in = QUESTFILE_STEPDESC;
                 continue;
             }
-            LOG(llevError, "quests: invalid line %s in file!\n", read);
+            if (strcmp(read, "setwhen\n") == 0) {
+                in = QUESTFILE_STEPCOND;
+                continue;
+            }
+            LOG(llevError, "quests: invalid line %s in definition of quest %s in file %s!\n",
+                    read, quest->quest_code, filename);
             continue;
         }
 
-        if (in == 2) {
+        if (in == QUESTFILE_QUESTDESC) {
             if (strcmp(read, "end_description\n") == 0) {
                 char *message;
 
-                in = 1;
+                in = QUESTFILE_QUEST;
 
                 message = stringbuffer_finish(buf);
                 buf = NULL;
@@ -187,15 +293,15 @@ static void quest_load_definitions(void) {
             continue;
         }
 
-        if (in == 1) {
+        if (in == QUESTFILE_QUEST) {
             if (strcmp(read, "end_quest\n") == 0) {
                 quest = NULL;
-                in = 0;
+                in = QUESTFILE_NEXTQUEST;
                 continue;
             }
 
             if (strcmp(read, "description\n") == 0) {
-                in = 2;
+                in = QUESTFILE_QUESTDESC;
                 buf = stringbuffer_new();
                 continue;
             }
@@ -211,7 +317,7 @@ static void quest_load_definitions(void) {
                 step->step = i;
                 step->next = quest->steps;
                 quest->steps = step;
-                in = 3;
+                in = QUESTFILE_STEP;
                 continue;
             }
 
@@ -219,6 +325,17 @@ static void quest_load_definitions(void) {
                 quest->quest_restart = i;
                 continue;
             }
+            if (strncmp(read, "parent ", 7) == 0) {
+                read[strlen(read) - 1] = '\0';
+                questname = add_string(read + 7);
+                if (!quest_get_by_code(questname)) {
+                    LOG(llevError, "Quest %s lists %s, as a parent, but this hasn't been defined\n", quest->quest_code, questname);
+                } else {
+                    quest->parent = quest_get_by_code(questname);
+                }
+                free_string(questname);
+                continue;
+            }
         }
 
         if (read[0] == '#')
@@ -228,20 +345,49 @@ static void quest_load_definitions(void) {
             quest = quest_create_definition();
             read[strlen(read) - 1] = '\0';
             quest->quest_code = add_string(read + 6);
+            if (quest_get_by_code(quest->quest_code)) {
+                LOG(llevError, "Quest %s is listed in file %s, but this quest has already been defined\n", quest->quest_code, filename);
+            }
             quest->next = quests;
             quests = quest;
-            in = 1;
-            found++;
+            in = QUESTFILE_QUEST;
+            loaded_quests++;
+            continue;
+        }
+        if (sscanf(read, "include %s\n", includefile)) {
+            char inc_path[HUGE_BUF];
+            path_combine_and_normalize(filename, includefile, inc_path, sizeof(inc_path));
+            found = load_quests_from_file(inc_path);
+            if (found >=0) {
+                LOG(llevDebug, "loaded %d quests from file %s\n", found, inc_path);
+                loaded_quests += found;
+            } else {
+                LOG(llevError, "Failed to load quests from file %s\n", inc_path);
+            }
             continue;
         }
 
         if (strcmp(read, "\n") == 0)
             continue;
 
-        LOG(llevError, "quest: invalid file format for %s\n", final);
+        LOG(llevError, "quest: invalid file format for %s, I don't know what to do with the line %s\n", final, read);
     }
+    LOG(llevError, "quest: quest definition file %s read in, ends with state %d\n", final, in);
+    return loaded_quests;
+}
 
-    LOG(llevInfo, "%d quests found.\n", found);
+/** Load all quest definitions. Can be called multiple times, will be ignored. */
+static void quest_load_definitions(void) {
+    int found = 0;
+    if (quests_loaded)
+        return;
+    quests_loaded = 1;
+    found = load_quests_from_file("world.quests");
+    if (found >= 0) {
+        LOG(llevInfo, "%d quests found.\n", found);
+    } else {
+        LOG(llevError, "Quest Loading Failed");
+    }
 }
 
 /**
@@ -274,16 +420,12 @@ static quest_definition *quest_get(sstring code) {
 
     quest_load_definitions();
 
-    quest = quests;
-    while (quest) {
-        if (quest->quest_code == code)
-            return quest;
-
-        quest = quest->next;
+    quest = quest_get_by_code(code);
+    if (!quest) {
+        LOG(llevError, "quest %s required but not found!\n", code);
+        return NULL;
     }
-
-    LOG(llevError, "quest %s required but not found!\n", code);
-    return NULL;
+    return quest;
 }
 
 /**
@@ -307,6 +449,10 @@ static void quest_read_player_data(quest_player *pq) {
     StringBuffer *buf = NULL;
     quest_state *qs = NULL;
     int warned = 0, state;
+    quest_definition *quest = NULL;
+
+    /* needed, so we can check ending steps. */
+    quest_load_definitions();
 
     snprintf(final, sizeof(final), "%s/%s/%s/%s.quest", settings.localdir, settings.playerdir, pq->player_name, pq->player_name);
 
@@ -322,6 +468,10 @@ static void quest_read_player_data(quest_player *pq) {
             qs->code = add_string(data);
             qs->next = pq->quests;
             pq->quests = qs;
+            quest = quest_get_by_code(qs->code);
+            if (quest == NULL) {
+                LOG(llevDebug, "Unknown quest %s in quest file %s", qs->code, final);
+            }
             continue;
         }
 
@@ -334,6 +484,14 @@ static void quest_read_player_data(quest_player *pq) {
 
         if (sscanf(read, "state %d\n", &state)) {
             qs->state = state;
+            if (quest != NULL && state != -1) {
+                quest_step_definition *step = quest_get_step(quest, state);
+                if (step == NULL) {
+                    LOG(llevError, "invalid quest step %d for %s in %s", state, quest->quest_code, final);
+                }
+                else if (step->is_completion_step)
+                    qs->is_complete = 1;
+            }
             continue;
         }
         if (strcmp(read, "end_quest\n") == 0) {
@@ -371,7 +529,7 @@ static void quest_write_player_data(const quest_player *pq) {
     file = fopen(write, "w+");
     if (!file) {
         LOG(llevError, "quest: couldn't open player quest file %s!", write);
-        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!", NULL);
+        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
         return;
     }
 
@@ -469,6 +627,71 @@ static quest_player *get_or_create_quest(player *pl) {
     return pq;
 }
 
+/* quest_set_state can call itself through the function update_quests, so it needs to be declared here */
+static void quest_set_state(player *pl, sstring quest_code, int state, int started);
+
+/**
+ * Checks whether the conditions for a given step are met.
+ * @param condition the linked list of conditions to check.
+ * @param pl the player to evaluate conditions for.
+ * @return 1 if the conditions match, 0 if they don't.
+ */
+static int evaluate_quest_conditions(const quest_condition *condition, player *pl) {
+    const quest_condition *cond;
+    int current_step;
+
+    if (!condition)
+        return 0;
+    cond = condition;
+    while (cond) {
+        current_step = quest_get_player_state(pl, cond->quest_code);
+        if (cond->minstep < 0 && cond->maxstep < 0) {
+            /* we are checking for the quest to have been completed. */
+            if (!quest_was_completed(pl, cond->quest_code))
+                return 0;
+        } else {
+            if (current_step < cond->minstep || current_step > cond->maxstep)
+                return 0;
+        }
+        cond = cond->next;
+    }
+    return 1;
+}
+
+/**
+ * Look through all of the quests for the given player, and see if any need to be updated.
+ * @param pl
+ */
+static void update_quests(player *pl) {
+    const quest_definition *quest;
+    const quest_step_definition *step;
+
+    /* we are going to check the conditions for every step, and then find the highest
+     * numbered step for which all conditions match, this will then be updated if that
+     * is a later stage than the player is at currently.
+     */
+    int new_step, current_step;
+    quest = quests;
+    while (quest) {
+        new_step=0;
+        step = quest->steps;
+        while (step) {
+            if (step->conditions)
+                if (evaluate_quest_conditions(step->conditions, pl)) {
+                    new_step=new_step<step->step?step->step:new_step;
+                }
+            step = step->next;
+        }
+        if (new_step > 0) {
+            current_step = quest_get_player_state(pl, quest->quest_code);
+            if (new_step > current_step) {
+                quest_set_state(pl, quest->quest_code, new_step, 0);
+            }
+        }
+        quest = quest->next;
+    }
+}
+
 /**
  * Set the state of a quest for a player.
  * @param pl player to set the state for.
@@ -506,7 +729,7 @@ static void quest_set_state(player *pl, sstring quest_code, int state, int start
     if (step->is_completion_step) {
         /* don't send an update note if the quest was already completed, this is just to show the outcome afterwards. */
         if (!qs->is_complete)
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest %s completed.", NULL, quest->quest_title);
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest %s completed.", quest->quest_title);
         qs->was_completed = 1;
         if (quest->quest_restart)
             qs->state = QC_CAN_RESTART;
@@ -514,11 +737,11 @@ static void quest_set_state(player *pl, sstring quest_code, int state, int start
             qs->is_complete =1;
 
     } else {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New objective for the quest '%s':", NULL, quest->quest_title);
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, step->step_description, NULL);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New objective for the quest '%s':", quest->quest_title);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, step->step_description);
     }
     quest_write_player_data(pq);
-
+    update_quests(pl);
     LOG(llevDebug, "quest_set_player_state %s %s %d\n", pl->ob->name, quest_code, state);
 
 }
@@ -533,53 +756,62 @@ static void quest_display(player *pl, quest_player *pq, int showall) {
     quest_state *state;
     quest_definition *quest;
     const char *restart;
-    int completed_count, restart_count, total_count, current_count;
+    int completed_count = 0, restart_count = 0, total_count = 0, current_count = 0;
 
     state = pq->quests;
     while (state) {
-        total_count++;
-        /* count up the number of completed quests first */
-        if (state->state == QC_CAN_RESTART) {
-            restart_count++;
-            completed_count++;
-        } else if(state->is_complete) {
-            completed_count++;
+        quest = quest_get(state->code);
+        if (quest->parent == NULL) {
+            total_count++;
+            /* count up the number of completed quests first */
+            if (state->state == QC_CAN_RESTART) {
+                restart_count++;
+                completed_count++;
+            } else if(state->is_complete) {
+                completed_count++;
             }
+        }
         state = state->next;
     }
     if (completed_count > 0) {
         if (!showall) {
             if (restart_count > 0)
                 draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                        "You have completed %d quests, of which %d may be restarted", NULL, completed_count, restart_count);
+                        "You have completed %d quests, of which %d may be restarted", completed_count, restart_count);
             else
                 draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                        "You have completed %d quests", NULL, completed_count);
+                        "You have completed %d quests", completed_count);
             current_count = completed_count;
         } else {
             draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                    "You have completed the following quests:", NULL);
+                    "You have completed the following quests:");
             state = pq->quests;
             while (state) {
-                if (state->state == QC_CAN_RESTART || state->is_complete) {
-                    quest = quest_get(state->code);
-                    restart = state->state == QC_CAN_RESTART?" (can be replayed)":"";
-                    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                            "(%3d) %s%s", NULL, ++current_count, quest->quest_title, restart);
-        }
-        state = state->next;
+                quest = quest_get(state->code);
+                if (quest->parent == NULL) {
+                    if (state->state == QC_CAN_RESTART || state->is_complete) {
+
+                        restart = state->state == QC_CAN_RESTART?" (can be replayed)":"";
+                        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                            "(%3d) %s%s", ++current_count, quest->quest_title, restart);
+                    }
+                }
+                state = state->next;
            }
         }
     }
     if (total_count > completed_count) {
         draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                "You have started the following quests:", NULL);
+                "You have started the following quests:");
         state = pq->quests;
         while (state) {
-            if (state->state != QC_CAN_RESTART && state->is_complete==0) {
-                quest = quest_get(state->code);
-                draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                        "(%3d) %s", NULL, ++current_count, quest->quest_title);
+            quest = quest_get(state->code);
+            if (quest->parent == NULL) {
+                if (state->state != QC_CAN_RESTART && state->is_complete==0) {
+                    quest = quest_get(state->code);
+                    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                        "(%3d) %s", ++current_count, quest->quest_title);
+                }
             }
             state = state->next;
         }
@@ -597,7 +829,7 @@ static void quest_list(player *pl, int showall) {
     /* ensure we load data if not loaded yet */
     pq = get_or_create_quest(pl);
     if (!pq->quests) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "You didn't start any quest.", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "You didn't start any quest.");
         return;
     }
 
@@ -609,9 +841,9 @@ static void quest_list(player *pl, int showall) {
  * @param pl player to display help for.
  */
 static void quest_help(player *pl) {
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest commands:", NULL);
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - list: displays quests you are currently attempting add 'all' to show completed quests also", NULL);
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - info: displays information about the specified (by number) quest", NULL);
+    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest commands:");
+    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - list: displays quests you are currently attempting add 'all' to show completed quests also");
+    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - info: displays information about the specified (by number) quest");
 }
 
 /**
@@ -624,7 +856,7 @@ static void quest_help(player *pl) {
 static quest_state *get_quest_by_number(player *pl, int number) {
     quest_state *state;
     quest_player *pq = get_or_create_quest(pl);
-    int questnum;
+    int questnum = 0;
 
     if (number <= 0 || !pq) {
         return NULL;
@@ -633,7 +865,7 @@ static quest_state *get_quest_by_number(player *pl, int number) {
     state = pq->quests;
     while (state) {
             /* count up the number of completed quests first */
-            if (state->state == QC_CAN_RESTART || state->is_complete)
+            if (!(quest_get(state->code)->parent) && (state->state == QC_CAN_RESTART || state->is_complete))
                 if (++questnum == number) return state;
             state = state->next;
         }
@@ -641,7 +873,7 @@ static quest_state *get_quest_by_number(player *pl, int number) {
     state = pq->quests;
     while (state) {
         /* count up the number of completed quests first */
-        if (state->state != QC_CAN_RESTART && state->is_complete ==0)
+        if (!(quest_get(state->code)->parent) && state->state != QC_CAN_RESTART && state->is_complete ==0)
             if (++questnum == number) return state;
         state = state->next;
     }
@@ -653,48 +885,62 @@ static quest_state *get_quest_by_number(player *pl, int number) {
  * Give details about a quest.
  * @param pl player to give quest details to.
  * @param qs quest_state to give details about
+ * @param level The level of recursion for the quest info that's being provided
  */
-static void quest_info(player *pl, quest_state *qs) {
+static void quest_info(player *pl, quest_state *qs, int level) {
+    quest_definition *quest, *child;
+    quest_state *state;
     quest_player *pq = get_or_create_quest(pl);
-    quest_definition *quest;
     quest_step_definition *step;
+    const char *prefix;
 
     if (!qs) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid quest number", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid quest number");
+        return;
+    }
+    quest = quest_get(qs->code);
+    if (!quest) {
+        /* already warned by quest_get */
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: (internal error)");
         return;
     }
-            quest = quest_get(qs->code);
-            if (!quest) {
-                /* already warned by quest_get */
-                draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: (internal error)", NULL);
-                return;
-            }
 
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: %s", NULL, quest->quest_title);
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Description: %s", NULL, quest->quest_description);
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: %s", quest->quest_title);
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Description: %s", quest->quest_description);
 
     step = quest_get_step(quest, qs->state);
     if (qs->state == QC_CAN_RESTART || qs->is_complete) {
-                const char *restart = "";
-                if (quest->quest_restart)
-                    restart = " (can be replayed)";
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "This quest has been completed%s.", NULL, restart);
+        const char *restart = "";
+        if (quest->quest_restart)
+            restart = " (can be replayed)";
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "This quest has been completed%s.", restart);
     }
-    const char *prefix = "";
+    prefix = "";
     if (qs->state != QC_CAN_RESTART) {
         /* ie, if we are in progress or completed for a non-restartable quest */
-                if (!step) {
-                    /* already warned by quest_get_step */
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \nOutcome: (invalid quest)", NULL);
-                    return;
-                }
-        if (qs->is_complete)
+        if (!step) {
+            /* already warned by quest_get_step */
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \nOutcome: (invalid quest)");
+            return;
+        }
+        if (level > 0) {
+            prefix = " * ";
+        } else if (qs->is_complete)
             prefix = "Outcome";
         else
             prefix = "Current Status";
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \n%s: %s", NULL, prefix, step->step_description);
-            }
-            return;
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \n%s: %s", prefix, step->step_description);
+    }
+
+    /* ok, now check all of the player's other quests for any children, and print those in order */
+    state = pq->quests;
+    while (state) {
+        child = quest_get(state->code);
+        if (child->parent == quest)
+            quest_info(pl, state, level+1);
+        state = state->next;
+    }
+    return;
 }
 
 /**
@@ -743,11 +989,19 @@ int quest_get_player_state(player *pl, sstring quest_code) {
  * @param state initial quest state, must be greater than 0 else forced to 100 and warning emitted.
  */
 void quest_start(player *pl, sstring quest_code, int state) {
-    quest_player *pq = get_or_create_quest(pl);
-    quest_state *q = get_or_create_state(pq, quest_code);
-    quest_definition *quest = quest_get(quest_code);
+    quest_player *pq;
+    quest_state *q;
+    quest_definition *quest;
     quest_step_definition *step;
 
+    quest = quest_get(quest_code);
+    if (!quest) {
+        LOG(llevError, "quest_start: requested unknown quest %s\n", quest_code);
+        return;
+    }
+    pq = get_or_create_quest(pl);
+    q = get_or_create_state(pq, quest_code);
+
     if (state <= 0) {
         state = 100;
         LOG(llevDebug, "quest_start: negative state %d for %s quest %s\n", state, pl->ob->name, quest_code);
@@ -760,7 +1014,7 @@ void quest_start(player *pl, sstring quest_code, int state) {
         LOG(llevDebug, "quest_start: warning: player %s has already started quest %s\n", pl->ob->name, quest_code);
     }
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New quest started: %s", NULL, quest->quest_title);
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New quest started: %s", quest->quest_title);
 
     quest_set_state(pl, quest_code, state, 0);
 
@@ -794,36 +1048,75 @@ int quest_was_completed(player *pl, sstring quest_code) {
  * Command handler for 'quest'.
  * @param op player asking for information, warning emitted if not a player.
  * @param params extra parameters for command.
- * @return 0
  */
-int command_quest(object *op, char *params) {
+void command_quest(object *op, const char *params) {
     if (!op->contr) {
         LOG(llevError, "command_quest called for a non player!\n");
-        return 0;
+        return;
     }
 
     if (!params || *params == '\0') {
         quest_help(op->contr);
-        return 0;
+        return;
     }
     if (strcmp(params, "list all") == 0) {
         quest_list(op->contr, 1);
-        return 0;
+        return;
     }
 
     if (strcmp(params, "list") == 0) {
         quest_list(op->contr, 0);
-        return 0;
+        return;
     }
 
     if (strncmp(params, "info ", 5) == 0) {
         int number = atoi(params+5);
-        quest_info(op->contr, get_quest_by_number(op->contr, number));
-        return 0;
+        quest_info(op->contr, get_quest_by_number(op->contr, number), 0);
+        return;
     }
 
     quest_help(op->contr);
-    return 0;
+}
+
+static void output_quests(quest_definition *parent, int level) {
+    quest_definition *quest;
+    quest_step_definition *step;
+    char prefix[MAX_BUF];
+    int questcount = 0, stepcount, i;
+
+    /* we only need to set the prefix once,
+     * all quests that are printed in this call will be at the same level */
+    prefix[0]='\0';
+    for (i=0; i<level; i++) {
+        strncat(prefix, "-", MAX_BUF - 1);
+    }
+    prefix[MAX_BUF - 1] = '\0';
+
+    quest = quests;
+    while (quest) {
+        if (quest->parent == parent) {
+            questcount++;
+            stepcount=0;
+            step = quest->steps;
+            while (step) {
+                stepcount++;
+                step= step->next;
+            }
+            fprintf(logfile, "%s%s - %s - %d steps (%srestartable)\n", prefix, quest->quest_code, quest->quest_title, stepcount, quest->quest_restart?"":"not ");
+            output_quests(quest, level+1);
+        }
+        quest = quest->next;
+    }
+}
+
+/**
+ * Dump all of the quests, then calls exit() - useful in terms of debugging to make sure that
+ * quests are set up and recognised correctly.
+ */
+void dump_quests(void) {
+    quest_load_definitions();
+    output_quests(NULL, 0);
+    exit(0);
 }
 
 /**
@@ -840,3 +1133,42 @@ void free_quest(void) {
     }
     player_states = NULL;
 }
+
+/**
+ * Free all quest definitions and steps.
+ * Can be called multiple times.
+ * Used by DMs through the 'purge_quests' command.
+ */
+void free_quest_definitions(void) {
+    quest_definition *quest = quests, *next_quest;
+    quest_step_definition *step, *next_step;
+    quest_condition *condition, *next_condition;
+
+    while (quest != NULL) {
+        next_quest = quest->next;
+        free_string(quest->quest_code);
+        if (quest->quest_description != NULL)
+            free_string(quest->quest_description);
+        if (quest->quest_title != NULL)
+            free_string(quest->quest_title);
+        step = quest->steps;
+        while (step != NULL) {
+            next_step = step->next;
+            free_string(step->step_description);
+            condition = step->conditions;
+            while (condition != NULL) {
+                next_condition = condition->next;
+                free_string(condition->quest_code);
+                free(condition);
+                condition = next_condition;
+            }
+            free(step);
+            step = next_step;
+        }
+        free(quest);
+        quest = next_quest;
+    }
+
+    quests = NULL;
+    quests_loaded = 0;
+}
