@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ *   "$Id: shop.c 14125 2010-11-24 19:01:48Z akirschbaum $";
  */
 
 /*
@@ -322,12 +322,16 @@ uint64 query_cost(const object *tmp, object *who, int flag) {
     if ((sint64)val < 0)
         val = 0;
 
-    /* Unidentified stuff won't sell for more than 60gp */
+    /* Unidentified stuff won't sell for more than 60gp each -
+     * it makes no sense to limit total number to 60, as all that
+     * does is force players to sell the itm in smaller blocks, which
+     * doesn't make much sense.
+     */
     if (flag == F_SELL
     && !QUERY_FLAG(tmp, FLAG_IDENTIFIED)
     && need_identify(tmp)
     && !identified) {
-        val = MIN(val, 600);
+        val = MIN(val, 600 * number);
     }
 
     /* if in a shop, check how the type of shop should affect the price */
@@ -360,13 +364,19 @@ uint64 query_cost(const object *tmp, object *who, int flag) {
                     /(shop_specialisation_ratio(tmp, who->map)
                       *shopkeeper_approval(who->map, who));
         }
-        /* We will also have an extra 0-5% variation between shops of
+        /* We will also have an extra +/-5% variation between shops of
          * the same type for valuable items (below a value of 50 this
          * effect wouldn't be very meaningful, and could give fun with
          * rounding.
+         * Change to use map reset time as a semi random but semi
+         * constant value - different maps are going to have different times,
+         * but it means the same shop will give consistent results until
+         * it resets - one can not save/load character hoping to get
+         * better prices.  Also, simplify this - really no reason
+         * to use cos for what is just a simple variation.
          */
         if (who->map->path != NULL && val > 50)
-            val = (sint64)val+0.05*(sint64)val*cos(tmp->count+strlen(who->map->path));
+            val += (val * (who->map->reset_time % 1000) - 500) / 10000;
     }
     return val;
 }
@@ -409,15 +419,22 @@ static archetype *find_next_coin(uint64 c, int *cointype) {
  *
  * @param cost
  * value to transform to currency.
+ * @param largest_coin
+ * maximum coin to give the price into, should be between 0 and NUM_COINS - 1.
  * @param buf
  * buffer to append to, if NULL a new one is returned.
  * @return
  * buffer containing the price, either buf or if NULL a new StringBuffer.
  */
-StringBuffer *cost_string_from_value(uint64 cost, StringBuffer *buf) {
+StringBuffer *cost_string_from_value(uint64 cost, int largest_coin, StringBuffer *buf) {
     archetype *coin, *next_coin;
     uint32 num;
-    int cointype = LARGEST_COIN_GIVEN;
+    int cointype = largest_coin;
+
+    if (cointype < 0)
+        cointype = 0;
+    else if (cointype >= NUM_COINS)
+        cointype = NUM_COINS - 1;
 
     if (!buf)
         buf = stringbuffer_new();
@@ -566,7 +583,7 @@ StringBuffer *query_cost_string(const object *tmp, object *who, int flag, String
             }
         }
     }
-    return cost_string_from_value(real_value, buf);
+    return cost_string_from_value(real_value, LARGEST_COIN_GIVEN, buf);
 }
 
 /**
@@ -952,7 +969,7 @@ int can_pay(object *pl) {
     if (unpaid_price > player_wealth) {
         char buf[MAX_BUF], coinbuf[MAX_BUF];
         int denominations = 0;
-        char *value = stringbuffer_finish(cost_string_from_value(unpaid_price, NULL));
+        char *value = stringbuffer_finish(cost_string_from_value(unpaid_price, LARGEST_COIN_GIVEN, NULL));
 
         snprintf(buf, sizeof(buf), "You have %d unpaid items that would cost you %s, ", unpaid_count, value);
         free(value);
@@ -970,7 +987,7 @@ int can_pay(object *pl) {
         else if (denominations > 1)
             make_list_like(buf);
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP,
-                      MSG_TYPE_SHOP_PAYMENT, buf, NULL);
+                      MSG_TYPE_SHOP_PAYMENT, buf);
         return 0;
     } else
         return 1;
@@ -1006,14 +1023,13 @@ int get_payment(object *pl, object *op) {
     if (op != NULL && QUERY_FLAG(op, FLAG_UNPAID)) {
         if (!pay_for_item(op, pl)) {
             uint64 i = query_cost(op, pl, F_BUY|F_SHOP)-query_money(pl);
-            char *missing = stringbuffer_finish(cost_string_from_value(i, NULL));
+            char *missing = stringbuffer_finish(cost_string_from_value(i, LARGEST_COIN_GIVEN, NULL));
 
             CLEAR_FLAG(op, FLAG_UNPAID);
             query_name(op, name_op, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, pl,
                                  MSG_TYPE_SHOP, MSG_TYPE_SHOP_PAYMENT,
                                  "You lack %s to buy %s.",
-                                 "You lack %s to buy %s.",
                                  missing, name_op);
             free(missing);
             SET_FLAG(op, FLAG_UNPAID);
@@ -1028,7 +1044,6 @@ int get_payment(object *pl, object *op) {
             draw_ext_info_format(NDI_UNIQUE, 0, pl,
                                  MSG_TYPE_SHOP, MSG_TYPE_SHOP_PAYMENT,
                                  "You paid %s for %s.",
-                                 "You paid %s for %s.",
                                  value, name_op);
             free(value);
             tmp = object_merge(op, NULL);
@@ -1055,7 +1070,8 @@ int get_payment(object *pl, object *op) {
  * player. Shouldn't be NULL or non player.
  */
 void sell_item(object *op, object *pl) {
-    uint64 i = query_cost(op, pl, F_SELL|F_SHOP), extra_gain;
+    uint64 i = query_cost(op, pl, F_SELL|F_SHOP);
+    sint64 extra_gain;
     int count;
     object *tmp;
     archetype *at;
@@ -1077,7 +1093,6 @@ void sell_item(object *op, object *pl) {
         draw_ext_info_format(NDI_UNIQUE, 0, pl,
                              MSG_TYPE_SHOP, MSG_TYPE_SHOP_SELL,
                              "We're not interested in %s.",
-                             "We're not interested in %s.",
                              name_op);
 
         /* Even if the character doesn't get anything for it, it may still be
@@ -1153,7 +1168,6 @@ void sell_item(object *op, object *pl) {
     value = stringbuffer_finish(query_cost_string(op, pl, F_SELL|F_SHOP, NULL));
 
     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SHOP, MSG_TYPE_SHOP_SELL,
-                         "You receive %s for %s.",
                          "You receive %s for %s.",
                          value,
                          name_op);
@@ -1326,8 +1340,7 @@ int describe_shop(const object *op) {
     || map->shopmin
     || map->shopmax) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING,
-                      "From looking at the nearby shop you determine that it trades in:",
-                      NULL);
+                      "From looking at the nearby shop you determine that it trades in:");
 
         if (map->shopitems) {
             for (i = 0; i < map->shopitems[0].index; i++) {
@@ -1343,24 +1356,22 @@ int describe_shop(const object *op) {
         /* format the string into a list */
         make_list_like(tmp);
         draw_ext_info(NDI_UNIQUE, 0, op,
-                      MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, tmp, NULL);
+                      MSG_TYPE_SHOP, MSG_TYPE_SHOP_LISTING, tmp);
 
         if (map->shopmax) {
-            value = stringbuffer_finish(cost_string_from_value(map->shopmax, NULL));
+            value = stringbuffer_finish(cost_string_from_value(map->shopmax, LARGEST_COIN_GIVEN, NULL));
             draw_ext_info_format(NDI_UNIQUE, 0, op,
                                  MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
                                  "It won't trade for items above %s.",
-                                 "It won't trade for items above %s.",
                                  value);
             free(value);
         }
 
         if (map->shopmin) {
-            value = stringbuffer_finish(cost_string_from_value(map->shopmin, NULL));
+            value = stringbuffer_finish(cost_string_from_value(map->shopmin, LARGEST_COIN_GIVEN, NULL));
             draw_ext_info_format(NDI_UNIQUE, 0, op,
                                  MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
                                  "It won't trade in items worth less than %s.",
-                                 "It won't trade in items worth less than %s.",
                                  value);
             free(value);
         }
@@ -1369,37 +1380,37 @@ int describe_shop(const object *op) {
             if (map->shopgreed > 2.0)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "It tends to overcharge massively.", NULL);
+                              "It tends to overcharge massively.");
             else if (map->shopgreed > 1.5)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "It tends to overcharge substantially.", NULL);
+                              "It tends to overcharge substantially.");
             else if (map->shopgreed > 1.1)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "It tends to overcharge slightly.", NULL);
+                              "It tends to overcharge slightly.");
             else if (map->shopgreed < 0.9)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "It tends to undercharge.", NULL);
+                              "It tends to undercharge.");
         }
         if (map->shoprace) {
             opinion = shopkeeper_approval(map, op);
             if (opinion > 0.8)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "You think the shopkeeper likes you.", NULL);
+                              "You think the shopkeeper likes you.");
             else if (opinion > 0.5)
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "The shopkeeper seems unconcerned by you.", NULL);
+                              "The shopkeeper seems unconcerned by you.");
             else
                 draw_ext_info(NDI_UNIQUE, 0, op,
                               MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                              "The shopkeeper seems to have taken a dislike to you.", NULL);
+                              "The shopkeeper seems to have taken a dislike to you.");
         }
     } else draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
-                             "There is no shop nearby.", NULL);
+                             "There is no shop nearby.");
 
     return 1;
 }
