@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c 12159 2009-08-27 19:07:58Z akirschbaum $";
+ *   "$Id: skill_util.c 14070 2010-11-06 18:02:34Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -55,9 +55,10 @@
 #endif
 #include <living.h>     /* for defs of STR,CON,DEX,etc. -b.t.*/
 #include <spells.h>
+#include <skills.h>
 
-static int attack_hth(object *pl, int dir, const char *string, object *skill);
-static int attack_melee_weapon(object *op, int dir, const char *string, object *skill);
+static void attack_hth(object *pl, int dir, const char *string, object *skill);
+static void attack_melee_weapon(object *op, int dir, const char *string, object *skill);
 
 /**
  * Will contain a number-name mapping for skills, initialized by init_skills().
@@ -360,7 +361,8 @@ int change_skill(object *who, object *new_skill, int flag) {
 
     old_range = who->contr->shoottype;
 
-    if (who->chosen_skill && who->chosen_skill == new_skill) {
+    /* The readied skill can be a skill tool, so check on actual skill instead of object. */
+    if (who->chosen_skill && who->chosen_skill->skill == new_skill->skill) {
         /* optimization for changing skill to current skill */
         if (!(flag&0x1))
             who->contr->shoottype = range_skill;
@@ -457,11 +459,11 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
         if (QUERY_FLAG(skill, FLAG_APPLIED)) {
             CLEAR_FLAG(skill, FLAG_APPLIED);
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
-                          "You come to earth.", NULL);
+                          "You come to earth.");
         } else {
             SET_FLAG(skill, FLAG_APPLIED);
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
-                          "You rise into the air!.", NULL);
+                          "You rise into the air!.");
         }
         fix_object(op);
         success = 1;
@@ -494,23 +496,23 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
         /* note that the following 'attack' skills gain exp through hit_player() */
 
     case SK_KARATE:
-        (void)attack_hth(op, dir, "karate-chopped", skill);
+        attack_hth(op, dir, "karate-chopped", skill);
         break;
 
     case SK_PUNCHING:
-        (void)attack_hth(op, dir, "punched", skill);
+        attack_hth(op, dir, "punched", skill);
         break;
 
     case SK_FLAME_TOUCH:
-        (void)attack_hth(op, dir, "flamed", skill);
+        attack_hth(op, dir, "flamed", skill);
         break;
 
     case SK_CLAWING:
-        (void)attack_hth(op, dir, "clawed", skill);
+        attack_hth(op, dir, "clawed", skill);
         break;
 
     case SK_WRAITH_FEED:
-        (void)attack_hth(op, dir, "fed upon", skill);
+        attack_hth(op, dir, "fed upon", skill);
         break;
 
     case SK_ONE_HANDED_WEAPON:
@@ -556,13 +558,13 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
 
     case SK_SET_TRAP:
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "This skill is not currently implemented.", NULL);
+                      "This skill is not currently implemented.");
         break;
 
     case SK_USE_MAGIC_ITEM:
     case SK_MISSILE_WEAPON:
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "There is no special attack for this skill.", NULL);
+                      "There is no special attack for this skill.");
         break;
 
     case SK_PRAYING:
@@ -583,11 +585,12 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
     case SK_FIRE_MAGIC:
     case SK_WATER_MAGIC:
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "This skill is already in effect.", NULL);
+                      "This skill is already in effect.");
         break;
 
     case SK_HARVESTING:
-        success = do_harvest(op, dir, skill);
+        do_harvest(op, dir, skill);
+        success = 0;
         break;
 
     default: {
@@ -648,11 +651,11 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
  * @param op
  * object that was 'defeated'.
  * @param skill
- * used skill.  If none, it should just point back to who.
+ * used skill.  If none, it should just point back to who or be NULL.
  * @return
  * experience for the skill use.
  */
-sint64 calc_skill_exp(object *who, object *op, object *skill) {
+sint64 calc_skill_exp(const object *who, const object *op, const object *skill) {
     sint64 op_exp;
     int op_lvl;
     float base, value, lvl_mult;
@@ -869,25 +872,24 @@ void show_skills(object *op, const char *search) {
             if (num_skills_found >= NUM_SKILLS) {
                 draw_ext_info(NDI_RED, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
                               "Your character has too many skills. "
-                              "Something isn't right - contact the server admin", NULL);
+                              "Something isn't right - contact the server admin");
                 break;
             }
         }
     } FOR_INV_FINISH();
 
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                  "Player skills:", NULL);
+                  "Player skills:");
 
     if (num_skills_found > 1)
         qsort(skills, num_skills_found, MAX_BUF, (int (*)(const void *, const void *))strcmp);
 
     for (i = 0; i < num_skills_found; i++) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                      skills[i], skills[i]);
+                      skills[i]);
     }
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                         "You can handle %d weapon improvements.",
                          "You can handle %d weapon improvements.",
                          op->level/5+5);
 
@@ -895,12 +897,10 @@ void show_skills(object *op, const char *search) {
     if (strcmp(cp, "none") == 0)
         cp = NULL;
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                         "You worship %s.",
                          "You worship %s.",
                          cp ? cp : "no god at current time");
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                         "Your equipped item power is %d out of %d\n",
                          "Your equipped item power is %d out of %d\n",
                          op->contr->item_power, op->level);
 }
@@ -943,7 +943,6 @@ int use_skill(object *op, const char *string) {
     } FOR_INV_FINISH();
     if (!skop) {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                             "Unable to find skill %s",
                              "Unable to find skill %s",
                              string);
         return 0;
@@ -1019,7 +1018,6 @@ static object *find_best_player_hth_skill(object *op) {
 
         if (!best_skill) {
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                                 "Unable to find skill %s - using default unarmed skill",
                                  "Unable to find skill %s - using default unarmed skill",
                                  op->contr->unarmed_skill);
         } else {
@@ -1088,10 +1086,8 @@ static object *find_best_player_hth_skill(object *op) {
  * describes the damage ("claw", "punch", ...).
  * @param skill
  * skill used to damage.
- * @return
- * true if the attack damaged the opponent.
  */
-static int do_skill_attack(object *tmp, object *op, const char *string, object *skill) {
+static void do_skill_attack(object *tmp, object *op, const char *string, object *skill) {
     int success;
 
     /* For Players only: if there is no ready weapon, and no "attack" skill
@@ -1109,7 +1105,9 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
                  * it if appropriate.
                  */
                 if (op->chosen_skill) {
-                    for (i = 0; i < sizeof(unarmed_skills); i++)
+                    /* the list is 0-terminated, and talismans, which can be in chosen_skill,
+                     * have a subtype of 0, therefore don't check the 0 */
+                    for (i = 0; unarmed_skills[i] != 0; i++)
                         if (op->chosen_skill->subtype == unarmed_skills[i]) {
                             skill = op->chosen_skill;
                             break;
@@ -1122,8 +1120,8 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
                     if (!skill) {
                         draw_ext_info(NDI_BLACK, 0, op,
                                       MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                                      "You have no unarmed combat skills!", NULL);
-                        return 0;
+                                      "You have no unarmed combat skills!");
+                        return;
                     }
                 }
             }
@@ -1133,9 +1131,8 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
                     draw_ext_info_format(NDI_UNIQUE, 0, tmp,
                                          MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
                                          "Couldn't change to skill %s",
-                                         "Couldn't change to skill %s",
                                          skill->name);
-                    return 0;
+                    return;
                 }
             }
         } else {
@@ -1151,7 +1148,7 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
                 if (!tmp) {
                     LOG(llevError, "Could not find applied weapon on %s\n", op->name);
                     op->current_weapon = NULL;
-                    return 0;
+                    return;
                 } else {
                     char weapon[MAX_BUF];
 
@@ -1188,14 +1185,12 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
             draw_ext_info_format(NDI_UNIQUE, 0, op,
                                  MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
                                  "You %s %s!",
-                                 "You %s %s!",
                                  string, op_name);
         } else if (tmp->type == PLAYER) {
             query_name(op, op_name, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, tmp,
                                  MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
                                  "%s %s you!",
-                                 "%s %s you!",
                                  op_name, string);
         }
     } else if (tmp && !QUERY_FLAG(tmp, FLAG_FREED)) {
@@ -1205,10 +1200,8 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
         draw_ext_info_format(NDI_UNIQUE, 0, op,
                              MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_MISS,
                              "You miss %s!",
-                             "You miss %s!",
                              op_name);
     }
-    return success;
 }
 
 /**
@@ -1233,10 +1226,8 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
  * describes the damage ("claw", "punch", ...).
  * @param skill
  * attack skill.
- * @return
- * 0 if no attack, non zero if an attack was done.
  */
-int skill_attack(object *tmp, object *pl, int dir, const char *string, object *skill) {
+void skill_attack(object *tmp, object *pl, int dir, const char *string, object *skill) {
     sint16 tx, ty;
     mapstruct *m;
     int mflags;
@@ -1257,12 +1248,12 @@ int skill_attack(object *tmp, object *pl, int dir, const char *string, object *s
 
         mflags = get_map_flags(m, &m, tx, ty, &tx, &ty);
         if (mflags&P_OUT_OF_MAP)
-            return 0;
+            return;
 
         /* space must be blocked for there to be anything interesting to do */
         if (!(mflags&P_IS_ALIVE)
         && !OB_TYPE_MOVE_BLOCK(pl, GET_MAP_MOVE_BLOCK(m, tx, ty))) {
-            return 0;
+            return;
         }
 
         FOR_MAP_PREPARE(m, tx, ty, tmp2)
@@ -1272,7 +1263,7 @@ int skill_attack(object *tmp, object *pl, int dir, const char *string, object *s
                 /* Don't attack party members */
                 if ((pl->type == PLAYER && tmp2->type == PLAYER)
                 && (pl->contr->party != NULL && pl->contr->party == tmp2->contr->party))
-                    return 0;
+                    return;
                 tmp = tmp2;
                 break;
             }
@@ -1281,11 +1272,11 @@ int skill_attack(object *tmp, object *pl, int dir, const char *string, object *s
     if (!tmp) {
         if (pl->type == PLAYER)
             draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                          "There is nothing to attack!", NULL);
-        return 0;
+                          "There is nothing to attack!");
+        return;
     }
 
-    return do_skill_attack(tmp, pl, string, skill);
+    do_skill_attack(tmp, pl, string, skill);
 }
 
 /**
@@ -1305,10 +1296,8 @@ int skill_attack(object *tmp, object *pl, int dir, const char *string, object *s
  * describes the attack ("claw", "punch", ...).
  * @param skill
  * attack skill used.
- * @return
- * 0 if no attack was done, non zero else.
  */
-static int attack_hth(object *pl, int dir, const char *string, object *skill) {
+static void attack_hth(object *pl, int dir, const char *string, object *skill) {
     object *weapon;
 
     if (QUERY_FLAG(pl, FLAG_READY_WEAPON)) {
@@ -1321,16 +1310,15 @@ static int attack_hth(object *pl, int dir, const char *string, object *skill) {
                 draw_ext_info_format(NDI_UNIQUE, 0, pl,
                                      MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
                                      "You are unable to unwield %s in order to attack with %s.",
-                                     "You are unable to unwield %s in order to attack with %s.",
                                      weaponname, skill->name);
-                return 0;
+                return;
             } else {
                 draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                              "You unwield your weapon in order to attack.", NULL);
+                              "You unwield your weapon in order to attack.");
             }
         }
     }
-    return skill_attack(NULL, pl, dir, string, skill);
+    skill_attack(NULL, pl, dir, string, skill);
 }
 
 /**
@@ -1352,15 +1340,13 @@ static int attack_hth(object *pl, int dir, const char *string, object *skill) {
  * describes the attack ("claw", "punch", ...).
  * @param skill
  * attack skill used.
- * @return
- * 0 if no attack was done, non zero else.
  */
-static int attack_melee_weapon(object *op, int dir, const char *string, object *skill) {
+static void attack_melee_weapon(object *op, int dir, const char *string, object *skill) {
     if (!QUERY_FLAG(op, FLAG_READY_WEAPON)) {
         if (op->type == PLAYER)
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                          "You have no ready weapon to attack with!", NULL);
-        return 0;
+                          "You have no ready weapon to attack with!");
+        return;
     }
-    return skill_attack(NULL, op, dir, string, skill);
+    skill_attack(NULL, op, dir, string, skill);
 }
