@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c 12735 2010-03-25 22:24:50Z ryo_saeba $";
+ *    "$Id: time.c 13561 2010-08-07 10:16:49Z ryo_saeba $";
  */
 
 /*
@@ -334,7 +334,7 @@ static void remove_force(object *op) {
             CLEAR_FLAG(op->env, FLAG_CONFUSED);
             draw_ext_info(NDI_UNIQUE, 0, op->env,
                           MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END,
-                          "You regain your senses.", NULL);
+                          "You regain your senses.");
         }
         break;
 
@@ -353,7 +353,6 @@ static void remove_force(object *op) {
 
                     query_short_name(inv, name, HUGE_BUF);
                     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_ITEM, MSG_TYPE_ITEM_CHANGE,
-                                         "Your %s recovers its original form.",
                                          "Your %s recovers its original form.",
                                          name);
                 }
@@ -689,6 +688,16 @@ void move_player_mover(object *op) {
     sint16 nx, ny;
     mapstruct *m;
 
+    if (!op->map) {
+        if (op->env && op->env->map)
+            LOG(llevError, "move_player_mover: mover not in a map at %s %d %d!\n", op->env->map->path, op->env->x, op->env->y);
+        else
+            LOG(llevError, "move_player_mover: mover not in a map at undefinite location!");
+        op->speed = 0;
+        object_update_speed(op);
+        return;
+    }
+
     /* Determine direction now for random movers so we do the right thing */
     if (!dir)
         dir = get_random_dir();
