@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *    "$Id: item.c 12236 2009-10-29 20:01:47Z akirschbaum $";
+ *    "$Id: item.c 13919 2010-09-26 18:38:29Z ryo_saeba $";
  */
 
 /*
@@ -99,6 +99,9 @@ static unsigned int query_flags(const object *op) {
     && ((op->env && op->env->container == op) || (!op->env && QUERY_FLAG(op, FLAG_APPLIED))))
         flags |= F_OPEN;
 
+    if (!QUERY_FLAG(op, FLAG_IDENTIFIED) && need_identify(op))
+        flags |= F_UNIDENTIFIED;
+
     if (QUERY_FLAG(op, FLAG_KNOWN_CURSED)) {
         if (QUERY_FLAG(op, FLAG_DAMNED))
             flags |= F_DAMNED;
@@ -621,7 +624,7 @@ void lock_item_cmd(uint8 *data, int len, player *pl) {
 
     if (!op) {
         draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                      "Could not find object to lock/unlock", NULL);
+                      "Could not find object to lock/unlock");
         return;
     }
     if (!flag)
@@ -660,14 +663,13 @@ void mark_item_cmd(uint8 *data, int len, player *pl) {
     op = esrv_get_ob_from_count(pl->ob, tag);
     if (!op) {
         draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                      "Could not find object to mark", NULL);
+                      "Could not find object to mark");
         return;
     }
     pl->mark = op;
     pl->mark_count = op->count;
     query_name(op, name, MAX_BUF);
     draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                         "Marked item %s",
                          "Marked item %s",
                          name);
 }
@@ -705,10 +707,10 @@ void look_at(object *op, int dx, int dy) {
         if (!flag) {
             if (dx || dy)
                 draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                              "There you see:", NULL);
+                              "There you see:");
             else {
                 draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                              "You see:", NULL);
+                              "You see:");
             }
             flag = 1;
         }
@@ -716,12 +718,10 @@ void look_at(object *op, int dx, int dy) {
         query_name(tmp, name, MAX_BUF);
         if (QUERY_FLAG(op, FLAG_WIZ))
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                 "- %s (%d).",
                                  "- %s (%d).",
                                  name, tmp->count);
         else
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                 "- %s.",
                                  "- %s.",
                                  name);
 
@@ -737,10 +737,10 @@ void look_at(object *op, int dx, int dy) {
     if (!flag) {
         if (dx || dy)
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                          "You see nothing there.", NULL);
+                          "You see nothing there.");
         else
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                          "You see nothing.", NULL);
+                          "You see nothing.");
     }
 }
 
@@ -843,7 +843,7 @@ void inscribe_scroll_cmd(char *buf, int len, player *pl) {
 
     inscription = find_skill_by_name(pl->ob, "inscription");
     if (!inscription) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You don't know how to write!", NULL);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You don't know how to write!");
         return;
     }
 
