@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c 12873 2010-04-08 07:13:16Z mwedel $";
+ *    "$Id: loop.c 14015 2010-10-21 06:20:33Z mwedel $";
  */
 
 /*
@@ -189,6 +189,10 @@ void request_info_cmd(char *buf, int len, socket_struct *ns) {
         send_file(ns, "motd");
     else if (!strcmp(buf, "news"))
         send_file(ns, "news");
+    else if (!strcmp(buf,"newcharinfo"))
+        send_new_char_info(ns);
+    else if (!strcmp(buf,"startingmap"))
+        send_map_info(ns);
     else
         Send_With_Handling(ns, &sl);
     SockList_Term(&sl);
