@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sockets_c =
- *    "$Id: lowlevel.c 12020 2009-06-25 09:24:32Z akirschbaum $";
+ *    "$Id: lowlevel.c 14015 2010-10-21 06:20:33Z mwedel $";
  */
 
 /*
@@ -44,15 +44,6 @@
 #include <sproto.h>
 #include <errno.h>
 
-/**
- * Checks that at least a given number of bytes is available in a SockList
- * instance. Returns normal if the space is available. Otherwise calls
- * fatal(OUT_OF_MEMORY);
- * @param sl the SockList instance to check
- * @param size the number of bytes to ensure
- */
-static void SockList_Ensure(const SockList *sl, size_t size);
-
 /***********************************************************************
  *
  * SockList functions/utilities
@@ -95,10 +86,24 @@ void SockList_ResetRead(SockList *sl) {
     sl->len = 0;
 }
 
+/**
+ * Checks that at least a given number of bytes is available in a SockList
+ * instance. Returns normal if the space is available. Otherwise calls
+ * fatal(OUT_OF_MEMORY);
+ *
+ * @param sl the SockList instance to check
+ * @param size the number of bytes to ensure
+ */
+static void SockList_Ensure(const SockList *sl, size_t size) {
+    if (sl->len+size > sizeof(sl->buf)) {
+        fatal(OUT_OF_MEMORY);
+    }
+}
+
 /**
  * Adds an 8 bit value.
- * @pram sl the SockList instance to add to
- * @param c the value to add
+ * @param sl the SockList instance to add to
+ * @param data the value to add
  */
 void SockList_AddChar(SockList *sl, char data) {
     SockList_Ensure(sl, 1);
@@ -107,7 +112,7 @@ void SockList_AddChar(SockList *sl, char data) {
 
 /**
  * Adds a 16 bit value.
- * @pram sl the SockList instance to add to
+ * @param sl the SockList instance to add to
  * @param data the value to add
  */
 void SockList_AddShort(SockList *sl, uint16 data) {
@@ -118,7 +123,7 @@ void SockList_AddShort(SockList *sl, uint16 data) {
 
 /**
  * Adds a 32 bit value.
- * @pram sl the SockList instance to add to
+ * @param sl the SockList instance to add to
  * @param data the value to add
  */
 void SockList_AddInt(SockList *sl, uint32 data) {
@@ -131,7 +136,7 @@ void SockList_AddInt(SockList *sl, uint32 data) {
 
 /**
  * Adds a 64 bit value.
- * @pram sl the SockList instance to add to
+ * @param sl the SockList instance to add to
  * @param data the value to add
  */
 void SockList_AddInt64(SockList *sl, uint64 data) {
@@ -179,6 +184,18 @@ void SockList_AddLen8Data(SockList *sl, const void *data, size_t len) {
     SockList_AddData(sl, data, len);
 }
 
+/**
+ * Adds a data block prepended with an 16 bit length field.
+ * @param sl the SockList instance to add to
+ * @param data the value to add
+ * @param len the length in byte; must not exceed 65535
+ */
+void SockList_AddLen16Data(SockList *sl, const void *data, size_t len) {
+    assert(len <= 65535);
+    SockList_AddShort(sl, len);
+    SockList_AddData(sl, data, len);
+}
+
 /**
  * Adds a printf like formatted string.
  * @param sl the SockList instance to add to
@@ -579,8 +596,3 @@ void write_cs_stats(void) {
 }
 #endif
 
-static void SockList_Ensure(const SockList *sl, size_t size) {
-    if (sl->len+size > sizeof(sl->buf)) {
-        fatal(OUT_OF_MEMORY);
-    }
-}
