@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_metaserver_c =
- *    "$Id: metaserver.c 12597 2010-03-07 20:37:38Z akirschbaum $";
+ *    "$Id: metaserver.c 14245 2011-01-23 06:06:03Z mwedel $";
  */
 
 /*
@@ -57,73 +57,6 @@ static int metafd = -1;
 
 static struct sockaddr_in sock;
 
-/**
- * Connects to metaserver.
- *
- * Its only called once.  If we are not
- * trying to contact the metaserver of the connection attempt fails, metafd will be
- * set to -1.  We use this instead of messing with the settings.meta_on so that
- * that can be examined to at least see what the user was trying to do.
- */
-void metaserver_init(void) {
-#ifdef WIN32 /* ***win32 metaserver_init(): init win32 socket */
-    struct hostent *hostbn;
-    int temp = 1;
-#endif
-
-    if (!settings.meta_on) {
-        metafd = -1;
-        return;
-    }
-
-    if (isdigit(settings.meta_server[0]))
-        sock.sin_addr.s_addr = inet_addr(settings.meta_server);
-    else {
-        struct hostent *hostbn = gethostbyname(settings.meta_server);
-
-        if (hostbn == NULL) {
-            LOG(llevDebug, "metaserver_init: Unable to resolve hostname %s\n", settings.meta_server);
-            return;
-        }
-        memcpy(&sock.sin_addr, hostbn->h_addr, hostbn->h_length);
-    }
-#ifdef WIN32 /* ***win32 metaserver_init(): init win32 socket */
-    ioctlsocket(metafd, FIONBIO , &temp);
-#else
-    fcntl(metafd, F_SETFL, O_NONBLOCK);
-#endif
-    if ((metafd = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
-        LOG(llevDebug, "metaserver_init: Unable to create socket, err %d\n", errno);
-        return;
-    }
-    sock.sin_family = AF_INET;
-    sock.sin_port = htons(settings.meta_port);
-
-    /* No hostname specified, so lets try to figure one out */
-    if (settings.meta_host[0] == 0) {
-        char hostname[MAX_BUF], domain[MAX_BUF];
-
-        if (gethostname(hostname, sizeof(hostname))) {
-            LOG(llevDebug, "metaserver_init: gethostname failed - will not report hostname\n");
-            return;
-        }
-
-#ifdef WIN32 /* ***win32 metaserver_init(): gethostbyname! */
-        hostbn = gethostbyname(hostname);
-        if (hostbn != (struct hostent *)NULL) /* quick hack */
-            memcpy(domain, hostbn->h_addr, hostbn->h_length);
-
-        if (hostbn == (struct hostent *)NULL) {
-#else
-        if (getdomainname(domain, sizeof(domain))) {
-#endif /* win32 */
-            LOG(llevDebug, "metaserver_init: getdomainname failed - will not report hostname\n");
-            return;
-        }
-        snprintf(settings.meta_host, sizeof(settings.meta_host), "%s.%s", hostname, domain);
-    }
-}
-
 /**
  * Updates our info in the metaserver
  * Note that this is used for both metaserver1 and metaserver2 -
@@ -156,17 +89,6 @@ void metaserver_update(void) {
         num_players++;
     }
 
-    /* Only do this if we have a valid connection */
-    if (metafd != -1) {
-        snprintf(data, sizeof(data), "%s|%d|%s|%s|%d|%d|%ld", settings.meta_host, num_players,
-                 FULL_VERSION,
-                 settings.meta_comment, cst_tot.ibytes, cst_tot.obytes,
-                 (long)time(NULL)-cst_tot.time_start);
-        if (sendto(metafd, data, strlen(data), 0, (struct sockaddr *)&sock, sizeof(sock)) < 0) {
-            LOG(llevDebug, "metaserver_update: sendto failed, err = %d\n", errno);
-        }
-    }
-
     /* Everything inside the pthread lock/unlock is related
      * to metaserver2 synchronization.
      */
@@ -199,7 +121,7 @@ typedef struct _MetaServer2 {
 static MetaServer2 *metaserver2;
 
 /**
- * LocalMeta2Info basically holds all the non metaserver2 information
+ * LocalMeta2Info basically holds all the non server metaserver2 information
  * that we read from the metaserver2 file.  Could just have
  * individual variables, but doing it as a structure is cleaner
  * I think, and might be more flexible if in the future, we
@@ -379,7 +301,7 @@ int metaserver2_init(void) {
 #ifndef HAVE_CURL_CURL_H
     if (local_info.notification) {
         LOG(llevError, "metaserver2 file is set to do notification, but libcurl is not found.\n");
-        LOG(llevError, "Either fix your compilation, or turn of metaserver2 notification in \n");
+        LOG(llevError, "Either fix your compilation, or turn off metaserver2 notification in \n");
         LOG(llevError, "the %s/metaserver2 file.\n", settings.confdir);
         LOG(llevError, "Exiting program.\n");
         exit(1);
