@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_request_c =
- *    "$Id: request.c 12882 2010-04-12 06:29:23Z mwedel $";
+ *    "$Id: request.c 14248 2011-01-24 06:00:53Z mwedel $";
  */
 
 /*
@@ -292,7 +292,7 @@ void set_up_cmd(char *buf, int len, socket_struct *ns) {
             loginmethod = atoi(param);
 
             /* Only support basic login right now */
-            if (loginmethod > 1) loginmethod=1;
+            if (loginmethod > 2) loginmethod=2;
 
             ns->login_method = loginmethod;
             SockList_AddPrintf(&sl, "%d", loginmethod);
@@ -468,7 +468,6 @@ void new_player_cmd(uint8 *buf, int len, player *pl) {
      */
     if (pl->state != ST_PLAYING) {
         draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                             "You can not issue commands - state is not ST_PLAYING (%s)",
                              "You can not issue commands - state is not ST_PLAYING (%s)",
                              buf);
         return;
@@ -882,10 +881,10 @@ void esrv_send_animation(socket_struct *ns, short anim_num) {
      * the face itself) down to the client.
      */
     for (i = 0; i < animations[anim_num].num_animations; i++) {
-        if (!(ns->faces_sent[animations[anim_num].faces[i]]&NS_FACESENT_FACE))
-            esrv_send_face(ns, animations[anim_num].faces[i], 0);
+        if (!(ns->faces_sent[animations[anim_num].faces[i]->number]&NS_FACESENT_FACE))
+            esrv_send_face(ns, animations[anim_num].faces[i]->number, 0);
         /* flags - not used right now */
-        SockList_AddShort(&sl, animations[anim_num].faces[i]);
+        SockList_AddShort(&sl, animations[anim_num].faces[i]->number);
     }
     Send_With_Handling(ns, &sl);
     SockList_Term(&sl);
@@ -1054,8 +1053,8 @@ static int map2_add_ob(int ax, int ay, int layer, object *ob, SockList *sl, sock
                  */
                 if (smoothlevel) {
                     for (i = 0; i < NUM_ANIMATIONS(ob); i++) {
-                        if (!(ns->faces_sent[animations[ob->animation_id].faces[i]]&NS_FACESENT_SMOOTH))
-                            send_smooth(ns, animations[ob->animation_id].faces[i]);
+                        if (!(ns->faces_sent[animations[ob->animation_id].faces[i]->number]&NS_FACESENT_SMOOTH))
+                            send_smooth(ns, animations[ob->animation_id].faces[i]->number);
                     }
                 }
             } else if (!(ns->faces_sent[face_num]&NS_FACESENT_FACE)) {
@@ -1445,241 +1444,6 @@ void send_plugin_custom_message(object *pl, char *buf) {
     SockList_Term(&sl);
 }
 
-/**
- * This sends the experience table the sever is using
- */
-void send_exp_table(socket_struct *ns, char *params) {
-    SockList sl;
-    int i;
-    extern sint64 *levels;
-
-    SockList_Init(&sl);
-    SockList_AddString(&sl, "replyinfo exp_table\n");
-    SockList_AddShort(&sl, settings.max_level+1);
-    for (i = 1; i <= settings.max_level; i++) {
-        if (SockList_Avail(&sl) < 8) {
-            LOG(llevError, "Buffer overflow in send_exp_table, not sending all information\n");
-            break;
-        }
-        SockList_AddInt64(&sl, levels[i]);
-    }
-    Send_With_Handling(ns, &sl);
-    SockList_Term(&sl);
-}
-
-/**
- * This sends the skill number to name mapping. We ignore
- * the params - we always send the same info no matter what.
- */
-void send_skill_info(socket_struct *ns, char *params) {
-    SockList sl;
-    int i;
-
-    SockList_Init(&sl);
-    SockList_AddString(&sl, "replyinfo skill_info\n");
-    for (i = 1; i < NUM_SKILLS; i++) {
-        size_t len;
-
-        len = 16+strlen(skill_names[i]); /* upper bound for length */
-        if (SockList_Avail(&sl) < len) {
-            LOG(llevError, "Buffer overflow in send_skill_info, not sending all skill information\n");
-            break;
-        }
-
-        SockList_AddPrintf(&sl, "%d:%s\n", i+CS_STAT_SKILLINFO, skill_names[i]);
-    }
-    Send_With_Handling(ns, &sl);
-    SockList_Term(&sl);
-}
-
-/**
- * This sends the spell path to name mapping. We ignore
- * the params - we always send the same info no matter what.
- */
-void send_spell_paths(socket_struct *ns, char *params) {
-    SockList sl;
-    int i;
-
-    SockList_Init(&sl);
-    SockList_AddString(&sl, "replyinfo spell_paths\n");
-    for (i = 0; i < NRSPELLPATHS; i++) {
-        size_t len;
-
-        len = 16+strlen(spellpathnames[i]); /* upper bound for length */
-        if (SockList_Avail(&sl) < len) {
-            LOG(llevError, "Buffer overflow in send_spell_paths, not sending all spell information\n");
-            break;
-        }
-
-        SockList_AddPrintf(&sl, "%d:%s\n", 1<<i, spellpathnames[i]);
-    }
-    Send_With_Handling(ns, &sl);
-    SockList_Term(&sl);
-}
-
-/**
- * Creates the appropriate reply to the 'race_list' request info.
- *
- * @param sl
- * suitable reply.
- */
-static void build_race_list_reply(SockList *sl) {
-    archetype *race;
-
-    SockList_AddString(sl, "replyinfo race_list ");
-
-    for (race = first_archetype; race; race = race->next) {
-        if (race->clone.type == PLAYER) {
-            SockList_AddPrintf(sl, "|%s", race->name);
-        }
-    }
-}
-
-/**
- * Send the list of player races to the client.
- * The reply is kept in a static buffer, as it won't change during server run.
- *
- * @param ns
- * where to send.
- * @param params
- * ignored.
- */
-void send_race_list(socket_struct *ns, char *params) {
-    static SockList sl;
-    static int sl_initialized = 0;
-
-    if (!sl_initialized) {
-        sl_initialized = 1;
-        SockList_Init(&sl);
-        build_race_list_reply(&sl);
-    }
-
-    Send_With_Handling(ns, &sl);
-}
-
-/**
- * Sends information on specified race to the client.
- *
- * @param ns
- * where to send.
- * @param params
- * race name to send.
- * @todo finish writing
- */
-void send_race_info(socket_struct *ns, char *params) {
-    archetype *race = try_find_archetype(params);
-    SockList sl;
-
-    SockList_Init(&sl);
-    SockList_AddPrintf(&sl, "replyinfo race_info %s", params);
-
-    if (race) {
-    }
-
-    Send_With_Handling(ns, &sl);
-    SockList_Term(&sl);
-}
-
-/**
- * Creates the appropriate reply to the 'class_list' request info.
- *
- * @param sl
- * reply.
- */
-static void build_class_list_reply(SockList *sl) {
-    archetype *cl;
-
-    SockList_Reset(sl);
-    SockList_AddString(sl, "replyinfo class_list ");
-
-    for (cl = first_archetype; cl; cl = cl->next) {
-        if (cl->clone.type == CLASS) {
-            SockList_AddPrintf(sl, "|%s", cl->name);
-        }
-    }
-}
-
-/**
- * Sends the list of classes to the client.
- * The reply is kept in a static buffer, as it won't change during server run.
- *
- * @param ns
- * client to send to.
- * @param params
- * ignored.
- */
-void send_class_list(socket_struct *ns, char *params) {
-    static SockList sl;
-    static int sl_initialized = 0;
-
-    if (!sl_initialized) {
-        sl_initialized = 1;
-        SockList_Init(&sl);
-        build_class_list_reply(&sl);
-    }
-
-    Send_With_Handling(ns, &sl);
-}
-
-/**
- * Send information on the specified class.
- *
- * @param ns
- * where to send.
- * @param params
- * class name to send.
- * @todo finish writing
- */
-void send_class_info(socket_struct *ns, char *params) {
-}
-
-/**
- * Sends the desired file to the client.  In all
- * three cases, we are basically just dumping file
- * contents to the client - nothing more.
- *
- * @parm ns
- * socket to send to
- * @param file
- * Which file to send - string of either motd, news, rules
- */
-void send_file(socket_struct *ns, char *file) {
-    char buf[MAX_BUF];
-    FILE *fp;
-    int comp;
-    SockList sl;
-
-    if (!strcmp(file,"motd"))
-        snprintf(buf, sizeof(buf), "%s/%s", settings.confdir, settings.motd);
-    else if (!strcmp(file,"rules"))
-        snprintf(buf, sizeof(buf), "%s/%s", settings.confdir, settings.rules);
-    else if (!strcmp(file,"news"))
-        snprintf(buf, sizeof(buf), "%s/%s", settings.confdir, settings.news);
-    else {
-        LOG(llevError,"send_file requested to send unknown file: %s\n", file);
-        return;
-    }
-    fp = open_and_uncompress(buf, 0, &comp, "r");
-    if (fp == NULL)
-        return;
-    SockList_Init(&sl);
-    SockList_AddString(&sl, "replyinfo ");
-    SockList_AddString(&sl, file);
-    SockList_AddString(&sl, "\n");
-
-    while (fgets(buf, MAX_BUF, fp) != NULL) {
-        if (*buf == '#')
-            continue;
-        SockList_AddString(&sl, buf);
-    }
-    close_and_delete(fp, comp);
-    SockList_AddChar(&sl, 0);   /* Null terminate it */
-    Send_With_Handling(ns, &sl);
-    SockList_Term(&sl);
-}
-
-
-
 /**
  * This looks for any spells the player may have that have changed
  * their stats. It then sends an updspell packet for each spell that
@@ -2011,7 +1775,7 @@ void send_account_players(socket_struct *ns)
      * in until the player has actually logged in with that character -
      * to fill the data in at time of adding the character to the account
      * requires a fair amount of work to check_login(), since the load
-     * of the player file and other initialization is fairly closely 
+     * of the player file and other initialization is fairly closely
      * intertwined.  So until that is done, we still at least have
      * account names we can get and send.
      * note: chars[] has the last entry NULL terminated - thus,
@@ -2065,16 +1829,34 @@ void send_account_players(socket_struct *ns)
 
     /* Now add real character data */
     for (acn = ns->account_chars; acn; acn = acn->next) {
+        uint16 faceno;
 
         add_char_field(&sl, ACL_NAME, acn->name);
         add_char_field(&sl, ACL_CLASS, acn->character_class);
         add_char_field(&sl, ACL_RACE, acn->race);
         add_char_field(&sl, ACL_FACE, acn->face);
+        if (acn->face[0] != 0 ) {
+            faceno = find_face(acn->face, 0);
+
+            if (faceno != 0) {
+                if (!(ns->faces_sent[faceno]&NS_FACESENT_FACE)) {
+                    esrv_send_face(ns, faceno, 0);
+                }
+            }
+        } else
+            faceno=0;
+
         add_char_field(&sl, ACL_PARTY, acn->party);
         add_char_field(&sl, ACL_MAP, acn->map);
         SockList_AddChar(&sl, 3);
         SockList_AddChar(&sl, ACL_LEVEL);
         SockList_AddShort(&sl, acn->level);
+        if (faceno) {
+            SockList_AddChar(&sl, 3);
+            SockList_AddChar(&sl, ACL_FACE_NUM);
+            SockList_AddShort(&sl, faceno);
+        }
+
         SockList_AddChar(&sl, 0);
     }
     /* Now for any characters where we just have the name */
@@ -2095,12 +1877,14 @@ void send_account_players(socket_struct *ns)
  * This is a basic routine which extracts the name/password
  * from the buffer.  Several of the account login routines
  * provide a length prefixed string for name, and another for
- * password. 
+ * password.
  *
  * @param buf
  * character data to process.
  * @param len
- * length of this buffer
+ * length of this buffer.  This will be updated to be the next
+ * byte to process in the case that there is additional data
+ * in this packet that the caller may need to process.
  * @param name
  * preallocated (MAX_BUF) buffer to return the name (really, first string) in
  * @param password
@@ -2111,24 +1895,26 @@ void send_account_players(socket_struct *ns)
  * 2 - password is too long
  * 3 - corrupt data - length of name & password > len
  */
-static int decode_name_password(const char *buf, int len, char *name, char *password)
+static int decode_name_password(const char *buf, int *len, char *name, char *password)
 {
     int nlen, plen;
 
     nlen = buf[0];
-    if (nlen >= MAX_BUF || nlen > len) {
+    if (nlen >= MAX_BUF || nlen > *len) {
         return 1;
     }
     strncpy(name, buf+1, nlen);
     name[nlen] = 0;
 
     plen = buf[nlen + 1];
-    if (plen >= MAX_BUF || (plen + nlen) > len) {
+    if (plen >= MAX_BUF || (plen + nlen) > *len) {
         return 2;
     }
     strncpy(password, buf+2+nlen, plen);
     password[plen] = 0;
 
+    *len = nlen + plen+2;
+
     return 0;
 }
 /**
@@ -2148,7 +1934,7 @@ void account_login_cmd(char *buf, int len, socket_struct *ns) {
 
     SockList_Init(&sl);
 
-    status = decode_name_password(buf, len, name, password);
+    status = decode_name_password(buf, &len, name, password);
 
     if (status == 1) {
         SockList_AddString(&sl, "failure accountlogin Name is too long");
@@ -2236,7 +2022,7 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
 
     SockList_Init(&sl);
 
-    status = decode_name_password(buf, len, name, password);
+    status = decode_name_password(buf, &len, name, password);
     if (status == 1) {
         SockList_AddString(&sl, "failure accountlogin Name is too long");
         Send_With_Handling(ns, &sl);
@@ -2249,6 +2035,16 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
         SockList_Term(&sl);
         return;
     }
+    /*The minimum length isn't exactly required, but in the current implementation,
+     * client will send the same password for character for which there is a
+     * 2 character minimum size. Thus an account with a one character password
+     * won't be able to create a character. */
+    if (strlen(password)<2) {
+        SockList_AddString(&sl, "failure accountlogin Password is too short");
+        Send_With_Handling(ns, &sl);
+        SockList_Term(&sl);
+        return;
+    }
 
     if (account_exists(name)) {
         SockList_AddString(&sl, "failure accountnew That account already exists on this server");
@@ -2259,7 +2055,7 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
 
     status = account_check_string(name);
     if (status == 1) {
-        SockList_AddString(&sl, 
+        SockList_AddString(&sl,
                        "failure accountnew That account name contains invalid characters.");
         Send_With_Handling(ns, &sl);
         SockList_Term(&sl);
@@ -2267,7 +2063,7 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
     }
 
     if (status == 2) {
-        SockList_AddString(&sl, 
+        SockList_AddString(&sl,
                            "failure accountnew That account name is too long");
         Send_With_Handling(ns, &sl);
         SockList_Term(&sl);
@@ -2276,7 +2072,7 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
 
     status = account_check_string(password);
     if (status == 1) {
-        SockList_AddString(&sl, 
+        SockList_AddString(&sl,
                        "failure accountnew That password contains invalid characters.");
         Send_With_Handling(ns, &sl);
         SockList_Term(&sl);
@@ -2284,7 +2080,7 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
     }
 
     if (status == 2) {
-        SockList_AddString(&sl, 
+        SockList_AddString(&sl,
                            "failure accountnew That password is too long");
         Send_With_Handling(ns, &sl);
         SockList_Term(&sl);
@@ -2313,14 +2109,15 @@ void account_new_cmd(char *buf, int len, socket_struct *ns) {
  */
 void account_add_player_cmd(char *buf, int len, socket_struct *ns) {
     char name[MAX_BUF], password[MAX_BUF];
-    int status, force;
+    int status, force, nlen;
     SockList sl;
     const char *cp;
 
     SockList_Init(&sl);
 
     force = buf[0];
-    status = decode_name_password(buf+1, len-1, name, password);
+    nlen = len - 1;
+    status = decode_name_password(buf+1, &nlen, name, password);
     if (status == 1) {
         SockList_AddString(&sl, "failure accountaddplayer Name is too long");
         Send_With_Handling(ns, &sl);
@@ -2345,7 +2142,7 @@ void account_add_player_cmd(char *buf, int len, socket_struct *ns) {
          */
         if (status == 1)
             SockList_AddString(&sl, "failure accountaddplayer 0 The character does not exist.");
-        else 
+        else
             SockList_AddString(&sl, "failure accountaddplayer 0 That password is incorrect.");
 
         Send_With_Handling(ns, &sl);
@@ -2447,7 +2244,7 @@ void account_play_cmd(char *buf, int len, socket_struct *ns)
     }
     /* Make sure a client is not trying to spoof us here */
     if (i == MAX_CHARACTERS_PER_ACCOUNT || !chars[i]) {
-        SockList_AddPrintf(&sl, 
+        SockList_AddPrintf(&sl,
                            "failure accountplay Character %s is not associated with account %s",
                            buf, ns->account_name);
         Send_With_Handling(ns, &sl);
@@ -2489,26 +2286,48 @@ void account_play_cmd(char *buf, int len, socket_struct *ns)
 
 }
 
+#define MAX_CHOICES 100
 /**
  * We have received an accountplay command.
  * try to log in and play the character.
  */
 void create_player_cmd(char *buf, int len, socket_struct *ns)
 {
-    char name[MAX_BUF], password[MAX_BUF];
-    int status;
+    char name[MAX_BUF], password[MAX_BUF], *choices[MAX_CHOICES];
+    int status, nlen, choice_num=0, i;
     SockList sl;
     player *pl;
+    archetype *map=NULL, *race_a=NULL, *class_a=NULL;
+
+    mapstruct *newmap;
 
     SockList_Init(&sl);
 
-    status = decode_name_password(buf, len, name, password);
+    nlen = len;
+    status = decode_name_password(buf, &nlen, name, password);
     if (status == 1) {
         SockList_AddString(&sl, "failure createplayer Name is too long");
         Send_With_Handling(ns, &sl);
         SockList_Term(&sl);
         return;
     }
+
+    /* 2 characters minimum for password */
+    if (strlen(password)<2) {
+        SockList_AddString(&sl, "failure createplayer Password is too short");
+        Send_With_Handling(ns, &sl);
+        SockList_Term(&sl);
+        return;
+    }
+
+    /** too long, buffer overflow */
+    if (status == 2) {
+        SockList_AddString(&sl, "failure createplayer Password is too long");
+        Send_With_Handling(ns, &sl);
+        SockList_Term(&sl);
+        return;
+    }
+
     /* This is a fairly ugly solution - we're truncating the password.
      * however, the password information for characters is really
      * a legacy issue - when every character is associated with
@@ -2517,15 +2336,9 @@ void create_player_cmd(char *buf, int len, socket_struct *ns)
      * to move characters from one account to another, but not sure
      * if that is something we want to allow.
      */
-    if (strlen(password)>17) 
+    if (strlen(password)>17)
         password[16] = 0;
 
-    if (status == 2 || strlen(password)<=1 || strlen(password) > 17) {
-        SockList_AddString(&sl, "failure createplayer Password is too long");
-        Send_With_Handling(ns, &sl);
-        SockList_Term(&sl);
-        return;
-    }
     /* We just can't call check_name(), since that uses draw_info() to
      * report status.  We are also more permissive on names, so we use
      * account_check_string() - if that is safe for account, also safe
@@ -2552,7 +2365,7 @@ void create_player_cmd(char *buf, int len, socket_struct *ns)
     /* from a protocol standpoint, accountplay can be used
      * before there is a player structure (first login) or after
      * (character has logged in and is changing characters).
-     * Checkthe sockets for that second case - if so,
+     * Check the sockets for that second case - if so,
      * we don't need to make a new player object, etc.
      */
     for (pl=first_player; pl; pl=pl->next)
@@ -2563,9 +2376,12 @@ void create_player_cmd(char *buf, int len, socket_struct *ns)
      * we don't really want to do.
      */
     if (!pl) {
-        pl = add_player(ns, 1);
+        int flags = ADD_PLAYER_NEW;
+        if (ns->login_method >= 2)
+            flags |= (ADD_PLAYER_NO_MAP | ADD_PLAYER_NO_STATS_ROLL);
+        pl = add_player(ns, flags);
         SockList_ResetRead(&pl->socket.inbuf);
-    } else {
+    } else if (ns->login_method < 2) {
         roll_again(pl->ob);
         pl->state = ST_ROLL_STAT;
         set_first_map(pl->ob);
@@ -2574,16 +2390,246 @@ void create_player_cmd(char *buf, int len, socket_struct *ns)
     /* add_player does a lot of the work, but there are a few
      * things we need to update, like starting name and
      * password.
+     * This is done before processing in login_method>2.
+     * The character creation process it does when
+     * applying the race/class will use this
+     * name information.
      */
     FREE_AND_COPY(pl->ob->name, name);
     FREE_AND_COPY(pl->ob->name_pl, name);
     pl->name_changed = 1;
     strcpy(pl->password, crypt_string(password, NULL));
 
+    /* In this mode, we have additional data */
+    if (ns->login_method >= 2) {
+        int i, j, stat_total=0;
+        char *key, *value, *race=NULL, *class=NULL;
+
+        /* By setting this to zero, then we can easily
+         * check to see if all stats have been set.
+         */
+        memset(&pl->orig_stats, sizeof(living), 0);
+
+        while (nlen < len) {
+            i = buf[nlen];  /* Length of this line */
+            /* Sanity check from client - don't want to loop
+             * forever if there is a 0 length, and don't
+             * want to read beyond size of packet.
+             * Likewise, client should have sent
+             * the string to us already null terminated,
+             * but we will just make sure.
+             */
+            if ((i == 0) || (nlen + i > len)) break;
+            buf[nlen + i] = 0;
+
+            /* What we have are a series of lines -
+             * 'key value' format.  Find that space,
+             * and null it out so we can do strcasecmp.
+             * If no space, abort processing
+             */
+            key = buf + nlen + 1;
+            value = strchr(key, ' ');
+            if (!value) break;
+            *value = 0;
+            value++;
+
+            if (!strcasecmp(key,"race"))    race = value;
+            else if (!strcasecmp(key,"class"))   class = value;
+            else if (!strcasecmp(key,"starting_map")) {
+                map = try_find_archetype(value);
+                if (!map || map->clone.type != MAP || map->clone.subtype !=MAP_TYPE_CHOICE) {
+                    SockList_AddString(&sl,
+                               "failure createplayer Invalid or map");
+                    Send_With_Handling(ns, &sl);
+                    SockList_Term(&sl);
+                    return;
+                }
+            }
+            else if (!strcasecmp(key,"choice")) {
+                /* In general, MAX_CHOICES should be large enough
+                 * to always handle the choices from the client - of
+                 * course, the client could be broken and send us many
+                 * more choices than we should have, so handle that.
+                 */
+                if (choice_num == MAX_CHOICES) {
+                    LOG(llevError,
+                        "Number of choices receive exceed max value: %d>%d\n",
+                        choice_num, MAX_CHOICES);
+                } else {
+                    choices[choice_num] = value;
+                    choice_num++;
+                }
+            }
+            else {
+                /* Do stat processing here */
+                for (j=0; j < NUM_STATS; j++) {
+                    if (!strcasecmp(key,short_stat_name[j])) {
+                        int val = atoi(value);
+
+                        set_attr_value(&pl->orig_stats, j, val);
+                        break;
+                    }
+                }
+                if (j >= NUM_STATS) {
+                    /* Bad clients could do this - we should at least report
+                     * it, and useful when trying to add new parameters.
+                     */
+                    LOG(llevError, "Got unknown key/value from client: %s %s\n", key, value);
+                }
+            }
+            nlen += i + 1;
+        }
+        /* Do some sanity checking now.  But checking the stat
+         * values here, we will catch any 0 values since we do
+         * a memset above.  A properly behaving client should
+         * never do any of these things, but we do not presume
+         * clients will behave properly.
+         */
+        for (j=0; j<NUM_STATS; j++) {
+            int val = get_attr_value(&pl->orig_stats, j);
+
+            stat_total += val;
+            if (val > settings.starting_stat_max ||
+                val < settings.starting_stat_min) {
+                SockList_AddPrintf(&sl,
+                    "failure createplayer Stat value is out of range - %d must be between %d and %d",
+                     val, settings.starting_stat_min, settings.starting_stat_max);
+                Send_With_Handling(ns, &sl);
+                SockList_Term(&sl);
+                return;
+            }
+        }
+        if (stat_total > settings.starting_stat_points) {
+            SockList_AddPrintf(&sl,
+                "failure createplayer Total allocated statistics is higher than allowed (%d>%d)",
+                stat_total, settings.starting_stat_points);
+            Send_With_Handling(ns, &sl);
+            SockList_Term(&sl);
+            return;
+        }
+
+        if (race)
+            race_a = try_find_archetype(race);
+
+        if (class)
+            class_a = try_find_archetype(class);
+
+        /* This should never happen with a properly behaving client, so the error message
+         * doesn't have to be that great.
+         */
+        if (!race_a || race_a->clone.type != PLAYER || !class_a || class_a->clone.type != CLASS) {
+            SockList_AddString(&sl,
+                               "failure createplayer Invalid or unknown race or class");
+            Send_With_Handling(ns, &sl);
+            SockList_Term(&sl);
+            return;
+        }
+        /* At current time, only way this can fail is if the adjusted
+         * stat is less than 1.
+         */
+        if (apply_race_and_class(pl->ob, race_a, class_a)) {
+            SockList_AddString(&sl,
+                               "failure createplayer Unable to apply race - stat is out of bounds");
+            Send_With_Handling(ns, &sl);
+            SockList_Term(&sl);
+            return;
+        }
+
+    }
+
     SockList_AddString(&sl, "addme_success");
     Send_With_Handling(ns, &sl);
     SockList_Term(&sl);
 
+    if (ns->login_method >= 2) {
+        /* The client could have provided us a map - if so, map will be set
+         * and we don't want to overwrite it
+         */
+        if (!map)
+            map = get_archetype_by_type_subtype(MAP, MAP_TYPE_DEFAULT);
+
+        if (!map) {
+            /* This should never happen - its not something that can
+             * be easily worked around without other weird issues,
+             * like having 2 classes.
+             */
+            LOG(llevError, "Can not find object of type MAP subtype MAP_TYPE_DEFAULT.\n");
+            LOG(llevError, "Are the archetype files up to date?  Can not continue.\n");
+            abort();
+        }
+        strcpy(pl->maplevel, map->clone.slaying);
+
+        pl->ob->x = map->clone.stats.hp;
+        pl->ob->y = map->clone.stats.sp;
+
+        /* copy information to bed of reality information, in case the player dies */
+        snprintf(pl->savebed_map, sizeof(pl->savebed_map), "%s", map->clone.slaying);
+        pl->bed_x = pl->ob->x;
+        pl->bed_y = pl->ob->y;
+
+        enter_exit(pl->ob, NULL);
+        player_set_state(pl, ST_PLAYING);
+    }
+
+    /* We insert any objects after we have put the player on the map -
+     * this makes things safer, as certain objects may expect a normal
+     * environment.  Note that choice_num will only be set in the
+     * loginmethod > 2, which also checks (and errors out) if the
+     * race/class is not set, which is why explicit checking for
+     * those is not need.
+     */
+    for (i=0; i < choice_num; i++) {
+        char *choiceval, *cp, *token, *lasts;
+        const char *value;
+        archetype *arch;
+        object *op;
+
+        choiceval = strchr(choices[i], ' ');
+        if (!choiceval) {
+            LOG(llevError, "Choice does not specify value: %s\n", choices[i]);
+            continue;
+        }
+        *choiceval=0;
+        choiceval++;
+        value = object_get_value(&race_a->clone, choices[i]);
+        if (!value)
+            value = object_get_value(&class_a->clone, choices[i]);
+
+        if (!value) {
+            LOG(llevError, "Choice not found in archetype: %s\n", choices[i]);
+            continue;
+        }
+        cp = strstr(value, choiceval);
+        if (!cp) {
+            LOG(llevError, "Choice value not found in archetype: %s %s\n",
+                choices[i], choiceval);
+            continue;
+        }
+
+        /* Check to make sure that the matched string is an entire word,
+         * and not a substring (eg, valid choice being great_sword but
+         * we just get sword) - the space after the match should either be a
+         * space or null, and space before match should also be a space
+         * or the start of the string.
+         */
+        if ((cp[strlen(choiceval)] != ' ') && (cp[strlen(choiceval)] != 0) &&
+            (cp != value) && (*(cp-1) != ' ')) {
+
+            LOG(llevError, "Choice value matches substring but not entire word: %s substring %s\n",
+                choiceval, value);
+            continue;
+        }
+        arch = try_find_archetype(choiceval);
+        if (!arch) {
+            LOG(llevError, "Choice value can not find archetype %s\n", choiceval);
+            continue;
+        }
+        op = arch_to_object(arch);
+        op = object_insert_in_ob(op, pl->ob);
+        if (QUERY_FLAG(op, FLAG_AUTO_APPLY))
+            ob_apply(op, pl->ob, 0);
+    }
+
     socket_info.nconns--;
     ns->status = Ns_Avail;
 }
