@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <check.h>
 #include <global.h>
+#include <sproto.h>
 
 void setup(void) {
     /* put any initialisation steps here, they will be run before each testcase */
@@ -251,7 +252,7 @@ static void local_check_loaded_object(object *op) {
             op->gen_sp_armour = op->last_heal;
             op->last_heal = 0;
         }
-        ip = calc_item_power(op, 0);
+        ip = calc_item_power(op);
         /* Legacy objects from before item power was in the game */
         if (!op->item_power && ip) {
             if (ip > 3) {
